/* ******************************************************************************

     XMLStudio - Editor integrato per documenti XML
     Programma per la Tesi di Laurea di Paolo Guagliumi - pguagliumi@libero.it
     Docente relatore: Prof.ssa Paola Giannini - giannini@di.unito.it

     ImpostaFont.java - Definizione classe per la selezione del carattere
    (C) 2002 Paolo Guagliumi - Il programma adotta la licenza GPL allegata

   ****************************************************************************** */


// **** importa i packages utili ****

import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;


// **** definizione classe per impostare i Font ****

class ImpostaFont extends JDialog implements ActionListener, ListSelectionListener
{

	GraphicsEnvironment ambiente = GraphicsEnvironment.getLocalGraphicsEnvironment(); // per la descrizione di oggetti Font in Java
	private String[] fontType = ambiente.getAvailableFontFamilyNames(); // restituisce string[] contenente l'elenco dei caratteri disponibili
	private String[] fontStyle = {"Grassetto","Corsivo","Normale"}; // font styles avialible
	private String[] fontSize = {"8","10","12","14","16","18","20","22","24","26","28","30"}; // dimensioni dei caratteri
	private String prev = "aAbByYzZ 123456789"; // oggetto stringa per il preview del font
	private JLabel preview = new JLabel(prev,JLabel.CENTER); // oggetto JLabel usato per la preview del font
	private Font fontScelto; // oggetto di tipo Font che contiene il font scelto dall'utente
	private JButton pulsanteOk; // pulsanteOk per confermare il font scelto
	private JButton pulsanteCancella; // pulsanteCancella per
	private JList listaFont; // lista per la visualizzazione dei font
	private JList listaStiliFont; // lista per la visualizzazione degli stili dei font
	private JList listaDimensioniFont; // lista per la visualizzazione delle dimensioni dei font


	// **** costruttore della classe ImpostaFont ****

	// il JDialog appartiene ad un'altra finestra, il JFrame

	public ImpostaFont(JFrame parent) // al costruttore della classe che estende la JDialog passa il JFrame, finestra principale del programma
	{
		// questo costruttore imposta il frame del font

		super(parent, "Impostazione del carattere", true); // richiama la superclasse per creare la JDialog relativa al font

		//this.setTitle("Impostazione del carattere"); // il titolo della JDialog  gi impostato grazie al costrutto super

		pulsanteOk = new JButton("Ok"); // definisce il pulsante Ok
		pulsanteOk.addActionListener(this); // aggiunge al pulsante la possibilit di rilevare eventi
		pulsanteCancella = new JButton("Annulla"); // definisce il pulsante Annulla
		pulsanteCancella.addActionListener(this); // aggiunge al pulsante la possibilit di rilevare eventi

		// definizione della Jlist relativa all'elenco dei font disponibili

		listaFont = new JList(fontType); // non bisogna definire ActionListener per le liste
		listaFont.setSelectionMode(ListSelectionModel.SINGLE_SELECTION); // imposta la selezione di un solo item nella Jlist
		listaFont.setVisibleRowCount(10);
		listaFont.setSelectedIndex(0);
		listaFont.addListSelectionListener(this);

		// definizione della Jlist relativa agli stili disponibili per i font

		listaStiliFont = new JList(fontStyle);
		listaStiliFont.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		listaStiliFont.setVisibleRowCount(10);
		listaStiliFont.setSelectedIndex(0);

		// definizione della Jlist relativa alla dimensione dei font

		listaDimensioniFont = new JList(fontSize);
		listaDimensioniFont.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		listaDimensioniFont.setVisibleRowCount(10);
		listaDimensioniFont.setSelectedIndex(0);

		// definizione di un JPanel per il posizionamento delle JList

		JPanel pannelloListe = new JPanel(new GridLayout(1,5,3,15)); // righe, colonne, spazio orizzontale e verticale
		pannelloListe.add(new JScrollPane(listaFont)); // aggiunge al JPanel i 3 controlli JList
		pannelloListe.add(new JScrollPane(listaStiliFont));
		pannelloListe.add(new JScrollPane(listaDimensioniFont));

		// definizione di un JPanel per il posizionamento dei pulsanti

		JPanel pannelloPulsanti = new JPanel();
		pannelloPulsanti.add(pulsanteCancella);
		pannelloPulsanti.add(pulsanteOk);

		// aggiunta dei singoli componenti alla JDialog

		getContentPane().setLayout(new BorderLayout(5,5)); // imposta il layout del Frame
		getContentPane().add(preview,BorderLayout.NORTH); // posiziona la JLabel relativa alla preview
		getContentPane().add(pannelloListe,BorderLayout.CENTER); // posiziona il JPanel pannelloListe sul Frame
		getContentPane().add(pannelloPulsanti,BorderLayout.SOUTH); // posiziona il JPanel dei pulsanti sul Frame

		// Un listener chiude la finestra quando l'utente preme il pulsante di chiusura della finestra

		addWindowListener(new WindowAdapter() // classe esistente per agevolare la creazione di oggetti listener definendo solo metodi necessari
		{

			// metodo che gestisce la chiusura della JDialog

	       	public void windowClosing(WindowEvent e)
			{
				fontScelto = null;
				dispose(); // chiude la JDialog con questo metodo di Java
			}
		});  // fine di addWindowListener

		preview.setFont(new Font((String)listaFont.getSelectedValue(),Font.PLAIN,14));
		pack(); // causa alla JDialog di essere ridimensionata secondo il layout e la dimensione dei suoi sottocomponenti
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		setLocation((screenSize.width - getWidth()) / 2, (screenSize.height - getHeight()) / 2); // sposta la JDialog nella posizione centrale
		setVisible(true); // la JDialog  visibile
	}


	// **** metodo che restituisce il Font scelto ****

	public Font returnFont()
	{
		if(fontScelto == null) // se non  stato scelto alcun font, ritorna null
			return null;
		else
			return fontScelto;
	}


	// **** risponde ad azioni dell'utente ed esegue del codice: in attesa di eventi dai pulsanti ****

	public void actionPerformed(ActionEvent evento) // rileva oggetti ActionEvent
	{
		Object oggettoScelto = evento.getSource(); // grazie ad oggettoScelto individua quale bottone ha causato l'evento

		if (oggettoScelto == pulsanteOk) // se viene premuto il pulsanteOK
		{
			int valoreStile; // intero corrispondente allo stile selezionato

			String stileSelezionato = (String)listaStiliFont.getSelectedValue(); // inserisce nell'oggetto il valore selezionato dalla lista

			if (stileSelezionato.equals("Grassetto"))
				valoreStile = Font.BOLD;
			else
				if (stileSelezionato.equals("Corsivo"))
					valoreStile = Font.ITALIC;
				else
					if(stileSelezionato.equals("Normale"))
						valoreStile = Font.PLAIN;
            		else // questo caso non si verifica
						valoreStile = Font.PLAIN;

			fontScelto = new Font((String)listaFont.getSelectedValue(), valoreStile, Integer.parseInt((String)listaDimensioniFont.getSelectedValue()));
			dispose(); // chiude la JDialog dopo aver premuto il pulsanteOK
		}
		else
			if(oggettoScelto == pulsanteCancella) //  stato premuto pulsanteCancella
			{
				fontScelto = null;
			 	dispose(); // chiude la JDialog dopo aver premuto il pulsanteCancella
			}
	}


	// **** richiamato ogni volta che cambia il valore selezionato dello stile carattere ****

	public void valueChanged(ListSelectionEvent evento)
	{
		int valoreStile; // intero corrispondente allo stile selezionato

		String stileSelezionato = (String)listaStiliFont.getSelectedValue(); // inserisce nell'oggetto il valore selezionato dalla lista

		//preview.setFont(new Font((String)listaFont.getSelectedValue(),Font.BOLD,12));

		if (stileSelezionato.equals("Grassetto"))
				valoreStile = Font.BOLD;
			else
				if (stileSelezionato.equals("Corsivo"))
					valoreStile = Font.ITALIC;
				else
					if(stileSelezionato.equals("Normale"))
						valoreStile = Font.PLAIN;
            		else // questo caso non si verifica
						valoreStile = Font.PLAIN;

		preview.setFont(new Font((String)listaFont.getSelectedValue(), valoreStile, Integer.parseInt((String)listaDimensioniFont.getSelectedValue())));
	}
}