/* ******************************************************************************

     XMLStudio - Editor integrato per documenti XML
     Programma per la Tesi di Laurea di Paolo Guagliumi - pguagliumi@libero.it
     Docente relatore: Prof.ssa Paola Giannini - giannini@di.unito.it

     RenderAlbero.java - Definizione classe per realizzare il render dell'Albero
    (C) 2002 Paolo Guagliumi - Il programma adotta la licenza GPL allegata

   ****************************************************************************** */


// **** importa i packages utili ****

import java.awt.*;
import java.awt.event.*;
import javax.swing.JTree;
//import javax.swing.tree.DefaultMutableTreeNode;
//import javax.swing.event.TreeSelectionListener;
//import javax.swing.event.TreeSelectionEvent;
//import javax.swing.tree.TreeSelectionModel;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.ImageIcon;
//import javax.swing.ToolTipManager;
import org.w3c.dom.*;

/* Negli alberi JTree, se si desidera avere un controllo sulle icone (o sui tooltip) e' necessario creare
   una sottoclasse di nome "DefaultTreeCellRenderer" sovrapponendo un metodo getTreeCellRendererComponent.
   Restituisce una nuova istanza di DefaultTreeCellRenderer per mostrare un'appropriata icona grazie al
   metodo setIcon in base all'elemento del documento Xml. Icone e colori del testo sono determinati in base
   ai settaggi di UIManager.
   Quando un albero analizza e mostra ogni nodo, ne' il JTree e neppure il suo look-and-feel contengono
   informazioni relative alla visualizzazione del nodo. Invece l'albero utilizza il codice incluso in questa
   classe di rendering per mostrare ogni nodo. Per esempio, per mostrare una foglia dell'albero che ha un
   determinato valore stringa, l'albero interroga il Cell Renderer in modo che restituisca un componente che possa
   visualizzare un nodo foglia con quello specifico valore di stringa. Se il Cell Renderer e' un
   DefaultTreeCellRenderer, allora ritornera' una label che mostra l'iconda di default per le foglie seguita
   dalla stringa. Un Cell Renderer consente soltanto la visualizzazione, non gestisce eventi. */


// **** definizione classe per realizzare il render dell'albero ****

class RenderAlbero extends DefaultTreeCellRenderer
{
	// ImageIcon iconaFoglia;

	// **** costruttore della classe RenderAlbero ****

     public RenderAlbero()
     {
     	// iconaFoglia = new ImageIcon("images/esempio.gif");
     }

     // configura il render basato sui componenti passati per mostrare icone personalizzate (ed eventualmente testo)
	public Component getTreeCellRendererComponent(JTree albero, Object valore, boolean sel, boolean espanso, boolean foglia, int riga, boolean haFocus)
	{
		// setLeafIcon(new ImageIcon("images/middle.gif"));
		Component componente = super.getTreeCellRendererComponent(albero, valore, sel, espanso, foglia, riga, haFocus);
		if (valore instanceof XmlNodo) // se value e' uguale al tipo XmlNodo che estende DefaultMutableTreeNode
		{
			// casting esplicito di value nel tipo XmlNodo in node
			Node nodoesaminato = ( (XmlNodo) valore).restituisciXmlNodo(); // estrae da valore il nodo esaminato
			if (nodoesaminato instanceof Element)
				setIcon(espanso ? openIcon : closedIcon);
			else
				setIcon(leafIcon);
			}
			return componente; // restituisce un component ovvero un oggetto grafico
	}

// per le icone vedere TreeIconDemo2.java in f:\tesi\jtree o 1 o 2 o 3

//      public Component getTreeCellRendererComponent(JTree tree, Object value, boolean sel, boolean expanded, boolean leaf, int row, boolean hasFocus)
//      {
//      	super.getTreeCellRendererComponent(tree, value, sel, expanded, leaf, row, hasFocus);
//           if (leaf && isTutorialBook(value))
//           {
//           	setIcon(iconaFoglia);
//                setToolTipText("This book is in the Tutorial series.");
//           }
//           else
//           {
//           	setToolTipText(null);
//           }
//           return this;
// 	}


// 		protected boolean isTutorialBook(Object value) {
//             DefaultMutableTreeNode node =
//                     (DefaultMutableTreeNode)value;
//             BookInfo nodeInfo =
//                     (BookInfo)(node.getUserObject());
//             String title = nodeInfo.bookName;
//             if (title.indexOf("Tutorial") >= 0) {
//                 return true;
//             }
//             return false;
//         }

}