/* ******************************************************************************

     XMLStudio - Editor integrato per documenti XML
     Programma per la Tesi di Laurea di Paolo Guagliumi - pguagliumi@libero.it
     Docente relatore: Prof.ssa Paola Giannini - giannini@di.unito.it

     StampaGrafica.java - Classe per la stampa del documento in modo grafico
    (C) 2002 Paolo Guagliumi - Il programma adotta la licenza GPL allegata

   ****************************************************************************** */


// **** importa i packages utili ****

import java.awt.*; // libreria grafica AWT
import java.io.StringReader; // uno stream di caratteri il cui input e' una stringa
import java.io.LineNumberReader; // tiene traccia dei numeri di linea: setLineNumber e getLineNumber imposta e restituisce il numero di linea
import java.io.EOFException; // segnala che la fine del file o dello stream e' stata raggiunta inaspettatamente durante l'input


// **** definizione classe per effettuare la stampa del documento utilizzando la metodologia grafica ****

// stampa del testo grazie all'uso di PrintJob; non considera il Word Wrap o i tab

public class StampaGrafica
{
  	// costruttore della classe StampaGrafica

  	public StampaGrafica(PrintJob jobStampa, Graphics oggettoGrafico, String testo, String nomeDocumento)
  	{
    		int numeroPagina = 1; // numero della pagina corrente da stampare; numero di pagine complessive
    		int lineePagina = 0; // numero di linee della pagina corrente
    		int lineeComplessive = 0; // numero di linee dell'itero documento
    		int altezzaCorrente = 0; // altezza corrente del testo stampato nella pagina
    		String lineaSuccessiva; // contiene una linea di testo letta dal parametro "testo"

    		// nota: String non e' mutabile e quindi non muta durante la stampa
    		if (!(oggettoGrafico instanceof PrintGraphics))
    		{
      		throw new IllegalArgumentException ("Errore di istanza PrintGraphics.");
    		}
    		StringReader readerStringa = new StringReader (testo); // oggetto stream di carattere il cui parametro in input e' stringa
    		LineNumberReader readerNumeriLinea = new LineNumberReader (readerStringa); // crea un nuovo reader di numeri di linea
    		int altezzaPagina = jobStampa.getPageDimension().height;

    		// e' necessario impostare il font per poter avere l'output
    		Font carattereHelvetica = new Font("carattereHelveticaetica", Font.PLAIN, 12);
    		oggettoGrafico.setFont(carattereHelvetica); // imposta Helvetica come carattere di stampa
    		FontMetrics oggettofm = oggettoGrafico.getFontMetrics(carattereHelvetica); // restituisce oggetto LineMetrics per lo specifico carattere
    		int altezzaFont = oggettofm.getHeight(); // restituisce l'altezza standard di una linea di testo che usa questo carattere
    		int descentFont = oggettofm.getDescent(); // restituisce il font descent per questo carattere
    		try
    		{
	    		System.out.println ("----> Stampa del documento " + nomeDocumento + " in corso <----");
      		do // fai mentre vi sono righe da stampare nel documento
      		{
        			lineaSuccessiva = readerNumeriLinea.readLine(); // legge una linea di testo
        			if (lineaSuccessiva != null) // se vi e' ancora del testo da stampare
        			{
	          		if ((altezzaCorrente + altezzaFont) > altezzaPagina) // inizia una nuova pagina
          			{
	            			System.out.println ("" + lineePagina + " linee stampate nella pagina " + numeroPagina);
            				numeroPagina++; // incrementa il numero di pagina
            				lineePagina = 0; // il numero di linee e' reinizializzato a zero
            				altezzaCorrente = 0; // altezza corrente del testo stampato nella pagina
            				oggettoGrafico.dispose(); // rilascia risorse legate al precedente oggettoGrafico
            				oggettoGrafico = jobStampa.getGraphics(); // preleva il successivo oggettoGrafico
            				if (oggettoGrafico != null) // se l'oggettoGrafico non e' nullo
            				{
	              				oggettoGrafico.setFont(carattereHelvetica); // imposta questo carattere a oggettoGrafico
            				}
          			}
          			altezzaCorrente += altezzaFont; // altezzaCorrente della pagina e' incrementato dall'altezza del Font attuale
          			if (oggettoGrafico != null) // se l'oggettoGrafico non e' null allora
          			{
	          			// scrive il testo incluso in lineaSuccessiva utilizzando l'attuale font e colore
	          			// la base del carattere piu' a sinistra ha coordinate x = 0 e y = altezzaCorrente - descentFont
	            			oggettoGrafico.drawString(lineaSuccessiva, 0, altezzaCorrente - descentFont);
            				lineePagina++; // e' stata appena stampata una linea della pagina
            				lineeComplessive++; // e' stata appena stampata una linea del documento
          			}
          			else
          			{
	            			System.out.println ("oggettoGrafico e' vuoto: errore di stampa.");
          			}
        			}
      		} while (lineaSuccessiva != null);
      	} catch (EOFException eof)
      	{
	      	System.out.println ("Errore nella lettura del documento da stampare.");
      	}
      	catch (Throwable e)
    		{
    			System.out.println ("Eccezione rilevata durante la stampa del documento: " + e);
      		e.printStackTrace();
    		}
    		System.out.println ("" + lineePagina + " linee stampate nella pagina " + numeroPagina);
    		System.out.println ("Numero complessivo di linee stampate: " + lineeComplessive);
    		System.out.println ("Numero di pagine stampate: " + numeroPagina);
  	}
}