/* ******************************************************************************

     XMLStudio - Editor integrato per documenti XML
     Programma per la Tesi di Laurea di Paolo Guagliumi - pguagliumi@libero.it
     Docente relatore: Prof.ssa Paola Giannini - giannini@di.unito.it

     TavolaAttributi.java - Definizione classe per gestire attributi Xml
    (C) 2002 Paolo Guagliumi - Il programma adotta la licenza GPL allegata

   ****************************************************************************** */


// **** importa i packages utili ****

import javax.swing.table.*;
//import javax.xml.parsers.*;
import org.w3c.dom.*;

import javax.swing.tree.*;
import javax.swing.event.*;
import org.w3c.dom.*;
import javax.swing.JTree;


/* Questa classe astratta fornisce un'implementazione di default per la maggior parte dei metodi
   nell'interfaccia TableModel in javax.swing.table. Si occupa di gestire i listener e fornisce alcuni strumenti
   per generare TableModelEvents e inviarli agli opportuni listeners. Per creare un concreto TableModel come
   sottoclasse di AbstractTableModel e' necessario implementare i 3 seguenti metodi:

    public int getRowCount();
    public int getColumnCount();
    public Object getValueAt(int row, int column); */

// **** definizione classe per realizzare la gestione degli attributi nell'albero xml di parsing ****

class TavolaAttributi extends AbstractTableModel
{
	// l'interfaccia Node e' il principale tipo di dati per il documento DOM
  	protected Node nodoCorrente; // istanza di nodo i cui attributi sono rappresentati nel modello.
  	protected NamedNodeMap attributiCorrenti; /* oggetto che implementa l'interfaccia NamedNodeMap per rappresentare
  	                                             collezioni di nodi accessibili con il nome - questa istanza rappresenta
  	                                             tutti gli attributi dei nodi */

  	// **** dato un nodo, in attributiCorrenti mette i suoi attributi ****

	public void assegnaNodo(Node nodo)
	{
		nodoCorrente = nodo;

		// attributiCorrenti = (node == null ? null : node.getAttributes());
		if (nodo == null)
			attributiCorrenti = null; // se il nodo e' null allora non conterra' attributi
		else
			attributiCorrenti = nodo.getAttributes(); // da nodo preleva i suoi attributi e li mette in un tipo NamedNodeMap
	}


	// **** restituisce il nodo esaminato ****

	public Node restituisciNodoCorrente()
	{
		return nodoCorrente;
	}


	/* **** restituisce il numero di righe nel modello; una JTable usa questo metodo per determinare quante righe
	        dovrebbe visualizzare; ritorna quindi il numero di attributi presenti in attributiCorrenti **** */

	public int getRowCount()
	{
		if (attributiCorrenti == null)
			return 0;
		return attributiCorrenti.getLength();
	}


	/* restituisce il numero di colonne del modello corrente; ua JTable lo usa per determinare il numero
	   di colonne che dovrebbe creare e visualizzare per default */

	public int getColumnCount()
	{
		return 2;
	}


	/* **** dato un determinato indice di colonna restituisce il nome della colonna **** */

	public String getColumnName(int numeroColonna)
	{
		return numeroColonna == 0 ? "Attributo" : "Valore"; // la colonna 0 ha nome "Attributo" e la colonna 1 ha nome "Valore"
	}


	// **** restituisce il valore relativo ad una determinata cella identificata da un indice colonna e riga ****

	public Object getValueAt(int numeroRiga, int numeroColonna)
	{
		if (attributiCorrenti == null || numeroRiga < 0 || numeroRiga >= getRowCount())
			return "";
		Attr attributi = (Attr)attributiCorrenti.item(numeroRiga); // interfaccia attr che contiene gli attributi
		if (attributi == null) // se non vi sono attributi, restituisci un oggetto vuoto
			return "";
		switch (numeroColonna) // se vi sono attributi allora considera il numero della colonna
		{
			case 0: // se si desidera la colonna 0, restituisci il nome dell'attributo
				return attributi.getName();
			case 1: // se si desidera la colonna 1, restituisci il valore dell'attributo
				return attributi.getValue();
		}
		return "";
	}


	// **** restituisce vero se la cella identificata da una riga e da una colonna e' editabile ****

	public boolean isCellEditable(int numeroRiga, int numeroColonna)
	{
		return (numeroColonna == 1 ); // se il numero di colonna e' relativo al valore, allora la cella della JTable e' editabile
	}


	// **** metodo da utilizzare per l'inserimento manuale dei valori nella tavola JTable ****

	public void setValueAt(Object valoreAttributo, int numeroRiga, int numeroColonna)
	{
		if (numeroRiga < 0 || numeroRiga >= getRowCount()) // se numeroRiga non e' un valore di JTable valido
			return;
		if (nodoCorrente instanceof Element) // se nodoCorrente e' un nodo di tipo Element
		{
			String nome = getValueAt(numeroRiga, 0).toString(); // restituisce il nome dell'attributo da modificare
			((Element)nodoCorrente).setAttribute(nome, valoreAttributo.toString()); // reimposta il nome dell'attributo esistente associando un valoreAttributo
		}
	}
}
