/* ******************************************************************************

     XMLStudio - Editor integrato per documenti XML
     Programma per la Tesi di Laurea di Paolo Guagliumi - pguagliumi@libero.it
     Docente relatore: Prof.ssa Paola Giannini - giannini@di.unito.it

     UndoableTextArea.java - Estensione JTextArea per undo-redo
    (C) 2002 Paolo Guagliumi - Il programma adotta la licenza GPL allegata

   ****************************************************************************** */


/* Tutte le classi interessate ad implementare le operazioni di tipo 'undo' (annulla) dovrebbero
   implementare le operazioni UndoableEditListener interface dal package javax.swing.event.
   La classe implementa un metodo pubblico void undoableEditHappened(UndoableEditEvent e), richiamato
   per notificare ai listerer che l'edit 'cancellabile', 'annullabile' (undo)  avvenuto. L'azione che
   viene svolta da questo metodo e' notificare il testo editato dall'UndoManager. L'UndoManager colleziona
   tutte le azioni di editing avvenute.

    Il numero massimo di azioni di editing pu essere impostato richiamando il metodo setLimit(int)
   della classe UndoManager: imposta il numero massimo di azioni di editing che l'UndoManager pu contenere.

    Per annullare un cambiamento e' sufficiente richiamare il metodo undo() di UndoManager.

    Per ripristinare un cambiamento e' sufficiente richiamare il metodo redo() di UndoManager.

   Se UndoManager non puo' realizzare una operazione di undo, verra' generata un'eccezione CannotUndoException.
   Similarmente se UndoManager non puo' realizzare una operazione di ripristino (redo), verra' generata un'eccezione
   CannotRedoException. */


// **** importa i packages utili ****

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.undo.*; // package specifico che implementa le funzioni di undo-redo
import javax.swing.text.JTextComponent;



/* **** definizione classe per definire la JTextArea ****

    Per realizzare le operazioni di undo-redo e' importante implementare la JTextArea con UndoableEditListener

    Per utilizzare i metodi KeyPressed, KeyReleased, KeyTyped e' necessario implementare la JTextArea con KeyListener

    Per utilizzare i metodi FocusGained e FocusLost e' necessario implementare la JTextArea con FocusListener.
     Questi metodi sono invocati quando un componente guadagna o perde il focus della tastiera.*/

class UndoableTextArea extends JTextArea implements UndoableEditListener, FocusListener//, KeyListener //, FocusListener
{
	public static final int limiteundo = 100; // imposta come limite di 'undo', 100 caratteri

	public UndoManager oggettoUndoManager; // definizione dell'oggetto UndoManager

	public Color coloreTestoEvidenziato;


	// **** costruttore della classe UndoableTextArea ****

	public UndoableTextArea()
	{
		//super("testo"); // inserisce 'testo' all'interno dell'editor

		creaUndoManager(); // crea la TextArea dodata di proprieta' undo-redo
		addFocusListener(this); // per l'intercettazione di eventi Focus e il richiamo dei metodi focusGained e focusLost
	}


	// **** metodo che crea l'UndoManager ****

	public void creaUndoManager()
	{
		// aggiunge l'UndoableEditListener alla TextArea
		getDocument().addUndoableEditListener(this);

		// aggiunge il KeyListener necessario per keyPressed
		// this.addKeyListener(this);

		// aggiunge il focus listener per ricevere un evento di focus da questo
		// componente (JTextArea) quando acquisisce il focus relativo all'input
		// this.addFocusListener(this);

		oggettoUndoManager = new UndoManager();
		oggettoUndoManager.setLimit(limiteundo);
	}


	// **** metodo che rimuove l'UndoManager ****

	public void rimuoviUndoManager()
	{
		oggettoUndoManager.end(); // rimuove l'UndoManager
	}


	// **** metodo che rimuove i dati presenti nell'UndoManager ****

	public void cancelladatiUndoManager()
	{
		oggettoUndoManager.discardAllEdits(); // rimuove l'UndoManager
	}


	// **** undoableEditHappened e' richiamato non appena avviene una azione di edit su JTextArea ****

	public void undoableEditHappened(UndoableEditEvent e)
	{
		oggettoUndoManager.addEdit(e.getEdit()); // aggiunge l'azione di editing all'UndoManager
	}


	// **** metodo che realizza l'operazione di undo-annulla ****

	public void annulla()
	{
		try
		{
			oggettoUndoManager.undo(); // annulla l'ultimo cambiamento
		}
		catch(CannotUndoException cue)
		{
			Toolkit.getDefaultToolkit().beep();
		}
	}


	// **** metodo che realizza l'operazione di ripristina-redo ****

	public void ripristina()
	{
		try
		{
			oggettoUndoManager.redo(); // ripristina l'ultimo cambiamento
		}
		catch(CannotRedoException cue)
		{
			Toolkit.getDefaultToolkit().beep();
		}
	}


	// **** metodo invocato quando e' premuto un tasto ****

	/*public void keyPressed(KeyEvent e)
	{
		//identify the key pressed is the combination of
		//Controlkey and Z key
		if((e.getKeyCode() == KeyEvent.VK_Z) && (e.isControlDown()))
		{
			annulla();
		}

		//identify the key pressed is the combination of
		//Controlkey and Y key
		if((e.getKeyCode() == KeyEvent.VK_R) && (e.isControlDown()))
		{
			ripristina();
		}
    	}*/


	/*// **** metodo invocato quando un tasto viene rilasciato ****

	public void keyReleased(KeyEvent e)
	{
	}


	// **** metodo invocato quando un tasto e' premuto e poi rilasciato ****

	public void keyTyped(KeyEvent e)
	{
	}*/


	// **** metodo invocato quando la JTextArea guadagna il focus della tastiera **** ooo

	public void focusGained(FocusEvent fe)
	{
	}


	// **** metodo invocato quando la JTextArea perde il focus della tastiera **** ooo

	public void focusLost(FocusEvent fe)
	{
		this.getCaret().setSelectionVisible(true);
	}
}