/* ******************************************************************************

     XMLStudio - Editor integrato per documenti XML
     Programma per la Tesi di Laurea di Paolo Guagliumi - pguagliumi@libero.it
     Docente relatore: Prof.ssa Paola Giannini - giannini@di.unito.it

     XMLRoutine.java - Definizione classe per la scrittura del file xml
    (C) 2002 Paolo Guagliumi - Il programma adotta la licenza GPL allegata

   ****************************************************************************** */


// **** importa i packages utili ****

import java.io.*;
import org.w3c.dom.*;


// **** classe per la scrittura del file xml ****

class XMLRoutine
{
	static int spaziTab = 0;

	// **** dato il documento xml in formato document lo scrive su file attraverso la classe Writer ****

	public static void writeDocumento(Document documento, Writer out) throws Exception
	{
		out.write("<?xml version=\"1.0\" encoding=\"iso-8859-1\" standalone=\"yes\"?>\n");
		write(documento.getDocumentElement(), out); // getoDocumentElement restituisce diretto accesso ai nodi che sono figli della radice
	}

	// **** dato il documento xml in formato document lo scrive su file attraverso la classe Writer ****

	public static void write(Document documento, Writer out) throws Exception
	{
		write(documento.getDocumentElement(), out); // getoDocumentElement restituisce diretto accesso ai nodi che sono figli della radice
	}

	// **** scrive un singolo nodo su file ****

	public static void write(Node nodo, Writer out) throws Exception
	{
		if (nodo == null || out == null)
			return;

		int tipo = nodo.getNodeType(); // restituisce il tipo del nodo in esame
		switch (tipo)
		{
			case Node.DOCUMENT_NODE: // il nodo identifica a sua volta un documento xml
				write(((Document)nodo).getDocumentElement(), out); // richiama ricorsivamente il metodo write
				out.flush();
				break;

			case Node.ELEMENT_NODE: // il nodo e' un elemento e puo' avere uno o piu' attributi

				for (int i = 0; i<spaziTab; i++)
					out.write('\t');
				spaziTab =  spaziTab + 1;

				out.write('<');
				out.write(nodo.getNodeName()); // scrive il tag del nodo
				NamedNodeMap attributi = nodo.getAttributes();
				for (int k = 0; k < attributi.getLength(); k++ ) // nell'eventualita' che il nodo abbia attributi
				{
					Node attributo = attributi.item(k);
					out.write(' ');
					out.write(attributo.getNodeName()); // scrive il nome dell'attributo
					out.write("=\"");
					out.write(attributo.getNodeValue());
					out.write('\"');
				}
				out.write('>');
				out.write('\n');
				break;

			case Node.ENTITY_REFERENCE_NODE: // il nodo e' di tipo entit reference
				out.write('&');
				out.write(nodo.getNodeName());
				out.write(';');
				break;

			case Node.CDATA_SECTION_NODE: // Il nodo rappresenta una sezione CDATA
				out.write("<![CDATA[");
				out.write(nodo.getNodeValue());
				out.write("]]>");
				break;

			case Node.TEXT_NODE: // il nodo e' testuale

 				for (int i = 0; i<spaziTab; i++)
 					out.write('\t');

				out.write(nodo.getNodeValue());

 				out.write('\n');
				break;

			case Node.PROCESSING_INSTRUCTION_NODE: // il nodo ha informazioni specifiche al processore
				out.write("<?");
				out.write(nodo.getNodeName());
				String data = nodo.getNodeValue();
				if ( data != null && data.length() > 0 )
				{
					out.write(' ');
					out.write(data);
				}
				out.write("?>");
				break;

			default:
				out.write("<TYPE=" + tipo);
				out.write(nodo.getNodeName());
				out.write("?>");
				break;
		}

		NodeList figli = nodo.getChildNodes(); // in figli vengono posti i nodi figli del nodo
		if (figli != null) // se vi sono dei nodi figli, richiama ricorsivamente il metodo write
		{
			for ( int k = 0; k < figli.getLength(); k++ ) // per tutti i nodi figli
			{
				write(figli.item(k), out); // richiama il metodo write passando il k-esimo nodo figlio
			}
		}

		if (nodo.getNodeType() == Node.ELEMENT_NODE) // il nodo e' di tipo elemento e lo scrive su disco
		{
			spaziTab =  spaziTab - 1;
			for (int i = 0; i<spaziTab; i++)
				out.write('\t');

			out.write("</");
			out.write(nodo.getNodeName());
			out.write('>');

			out.write('\n');
		}
		out.flush();
	}
}