/* ******************************************************************************

     XMLStudio - Editor integrato per documenti XML
     Programma per la Tesi di Laurea di Paolo Guagliumi - pguagliumi@libero.it
     Docente relatore: Prof.ssa Paola Giannini - giannini@di.unito.it

     XMLStudio.java - Programma principale
    (C) 2002 Paolo Guagliumi - Il programma adotta la licenza GPL allegata

    *  =================================================== *
    *  This program contains code from the book "Swing"    *
    *  2nd Edition by Matthew Robinson and Pavel Vorobiev  *
    *  http://www.spindoczine.com/sbe                      *
    *  =================================================== *

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

    http://www.gnu.org/copyleft/gpl.html

   ****************************************************************************** */


// **** importa i packages utili ****
import java.io.*; // classi per la gestione dell'input-output
import java.awt.*; // libreria grafica AWT
import java.awt.event.*; // classi di eventi legati alle librerie AWT
import java.awt.Cursor; // per modificare il puntatore del mouse
import javax.swing.*; // libreria grafica Swing
import javax.swing.event.*; // classi di eventi legati alle librerie Swing
import javax.swing.tree.*; // classi per l'uso degli alberi in ambiente Swing
import java.util.Properties; // classi per la definizione di oggetto proprieta' per le impostazioni predefinite
import javax.help.*; // classi che consentono l'uso di JavaHelp
import java.net.URL; // classi che definiscono gli oggetti di tipo URL
import javax.xml.parsers.*; // classi usate per processare documenti Xml
import org.w3c.dom.*; // serie di interfacce che definiscono il DOM (Document Object Model)
import org.xml.sax.SAXException;  // eccezione utilizzata per la verifica di documento xml ben formato


// **** definizione classe ****

public class XMLStudio extends JFrame implements ActionListener, WindowListener, CaretListener
{
    // dichiarazioni generali
    private Dimension screenSize; // oggetto contenente le dimensioni dello schermo
    public JLabel stato; // riporta lo stato corrente sulla StatusBar
    private JLabel colonnerighe; // riporta l'attuale riga e colonna del cursore nell'editor

    // impostazioni oggetti propriet dell'editor
    private Font carattereDefault; // oggetto contenente il Font di default impostato nelle proprieta'
    private Color coloreTestoDefault; // oggetto contenente il colore di default del testo
    private Color coloreSfondoDefault; // oggetto contenente il colore di default dello sfondo del testo
    private Boolean wordWrapDefault; // variabile globale che determina se ad areaTesto e' applicato o meno il word-wrap
    private String directoryCorrente; // oggetto stringa contenente la directory attuale di lavoro
    private String fileXmlTagCorrente; // oggetto stringa contenente il nome del file dei tag da caricare per default all'avvio
    private String finestraX, finestraY, divisoreOrizzontale, divisoreVerticale; // dimensioni del JFrame dell'editor
    private String lookandfeelCorrente; // oggetto stringa contenente il lookandfeel attualmente visualizzato
    private String memorizzaLinea = null; // memorizza il numero di riga del cursore sul documento
    private int riga; // memorizza il valore relativo alla riga raggiunto dal cursore
    private File fileCorrente = new File("Vuoto"); // file salvato e attualmente aperto
    private boolean fileGiaSalvato; // vero se fileCorrente e' gia' stato salvato
    private String parola; // stringa ricercata in Trova, Trova successivo, Sostituisci
    private StoriaFile ultimiFile; // memorizza in questo oggetto i cammini degli ultimi file aperti
    HelpSet hs; // definisce l'oggetto HelpSet per l'uso di JavaHelp
    HelpBroker hb; // definisce l'oggetto HelpBroker per l'uso di JavaHelp
    public UndoableTextArea areaTesto; // JTextArea areaTesto che contiene il documento testuale;
    public JPopupMenu popupAreaTesto; // implementa i menu' popup relativi all'area testo
    private JSplitPane splitPaneCentrale, splitPaneCentrale2; // contenitori dei JTree, JTable, ecc.
    private XmlTag oggettoXmlTag; // oggetto contenente il JTree e JTable relativi agli attributi
    protected Document documentoXml; // interfaccia che rappresenta l'intero documento Xml
    protected JTree alberoXml; // definisce il JTree che permette la visualizzazione dei dati secondo gerarchia
    protected DefaultTreeModel modelloAlbero; // implementa l'interfaccia TreeModel
    protected JTable tavolaAttributiXml; // definisce la JTable per gli attributi dell'albero di parsing
    protected TavolaAttributi modellotavolaAttributiXml; // definisce un'estensione di AbstractTableModel per implementare la JTable

    JButton nuovo, apri, salva, stampa, taglia, copia, incolla, annulla, ripristina, aggiornaxml, benformatoxml, // pulsanti della toolbar
            aggiungitagxml, memolinea, vaimemolinea, creaxml, trova, trovasuccessivo, sostituisci, helpcontesto;

    public JMenuItem jmi_salva; // per l'abilitazione/disabilitazione dell'icona relativa al salvataggio del file
    public JMenuItem jmi_aggiunginodotag, jmi_modificanodotag, jmi_eliminanodotag, // menu' pubblici in quanto sono abilitati/disabilitati da XmlTag
				 jmi_aggiungiattributonodotag, jmi_modificaattributonodotag, jmi_eliminaattributonodotag;

    JCheckBoxMenuItem jmi_wordwrap; // per la selezione dell'opzione "A capo linea" nell'area testo

	// **** inizializza il look e feel dell'editor ****

	private void initLookAndFeel(String look)
	{
    	   String lookAndFeel = null; // definizione oggetto per l'impostazione del look e feel

        if (look != null)
        {
        	if (look.equals("Metal")) // stile 'Metal' di look e feel
          {
          	lookAndFeel = UIManager.getCrossPlatformLookAndFeelClassName();
          }
          else
          	if (look.equals("System")) // stile 'System' di look e feel
            	{
            		lookAndFeel = UIManager.getSystemLookAndFeelClassName();
            	}
            	else
            		if (look.equals("Mac")) // stile 'Mac' di look e feel
            		{
                		lookAndFeel = "com.sun.java.swing.plaf.mac.MacLookAndFeel";
            	    	}
            	    	else
            	    	if (look.equals("Windows")) // stile 'Windows' di look e feel
            	    	{
                		lookAndFeel = "com.sun.java.swing.plaf.windows.WindowsLookAndFeel";
            		}
            		else
            			if (look.equals("Motif")) // stile 'Motif' di look e feel
            			{
	               		lookAndFeel = "com.sun.java.swing.plaf.motif.MotifLookAndFeel";
    	        			}

            try
            {
            	UIManager.setLookAndFeel(lookAndFeel); // imposta il look e feel dell'interfaccia grafica

            	//SwingUtilities.updateComponentTreeUI(contenitoreComponenti); //servirebbe ad aggiornare a run-time il lookefeel ma necessita di Frame
            	SwingUtilities.updateComponentTreeUI(getContentPane());
            }
            catch (ClassNotFoundException e)
            {
                System.err.println("Non trovo la classe relativa al look e feel specificato: " + lookAndFeel);
                System.err.println("Uso del look e feel di default in questa sessione.");
            }
            catch (UnsupportedLookAndFeelException e)
            {
                System.err.println("Non e' possibile usare il look e feel specificato (" + lookAndFeel + ") su questo computer.");
                System.err.println("Uso del look e feel di default in questa sessione.");
            }
            catch (Exception e)
            {
                System.err.println("Il look e feel specificato (" + lookAndFeel + "), non e' disponibile.");
                System.err.println("Uso del look e feel di default in questa sessione.");
                e.printStackTrace();
            }
        }
    }


     // **** creaJavaHelp definisce gli oggetti per usare l'HelpSet ****

	private void creaJavaHelp()
	{
		// il codice seguente mostra una tipica istanzazione dell'HelpSet

		ClassLoader loader = XMLStudio.class.getClassLoader(); // usata per identificare classi richieste da chi consulta l'HelpSet
    		try
    		{
      		URL url = HelpSet.findHelpSet(loader, "./Help/XMLStudio.hs"); // url del principale file di HelpSet
      		hs = new HelpSet(loader, url); // per creare un'istanza di HelpSet e' necessario fornire 2 parametri

      		// codice per definire l'interfaccia HelpBroker che descrive un componente usato per gestire l'iterazione con HelpSet

      		hb = hs.createHelpBroker(); // crea l'oggetto HelpBroker
      		hb.enableHelpKey(getRootPane(), "top", hs); // specifichiamo il valore ID dell'argomento di apertura dell'help
    		}
    		catch (Exception ex)
    		{
	    		ex.printStackTrace();
	    	}
	}


    	// **** imposta le proprieta' di default nell'editor ****

    	private void eseguiImpostazioniDefault(Properties p)
	{
		// carica le impostazioni di default in variabili

		carattereDefault = new Font(p.getProperty("font"),Integer.parseInt(p.getProperty("tipofont")),Integer.parseInt(p.getProperty("dimensionefont")));
		coloreTestoDefault = new Color(Integer.parseInt(p.getProperty("colorefont")));
		coloreSfondoDefault = new Color(Integer.parseInt(p.getProperty("coloresfondo")));

		finestraX = p.getProperty("finestrax");
		finestraY = p.getProperty("finestray");
		setBounds(0, 0, Integer.parseInt(finestraX), Integer.parseInt(finestraY)); // inizializzazione dimensione dell'editor

		divisoreOrizzontale = p.getProperty("divisorex");
		divisoreVerticale = p.getProperty("divisorey");

		directoryCorrente = p.getProperty("directory");
		File verifica = new File(directoryCorrente); // verifica se la directory correntemente salvata nelle impostazioni esiste realmente
		if (verifica.isDirectory() == false) // imposta come directory corrente la directory del programma
		{
			System.out.println("- Si consiglia di impostare la Directory di lavoro relativa ai documenti testuali sui quali lavorare, nel menu' Impostazioni\n");
			directoryCorrente = ".";
		}

		fileXmlTagCorrente = p.getProperty("filexmltag");
		verifica = new File(fileXmlTagCorrente); // verifica se il file di tag xml salvato nelle impostazioni esiste realmente
		if (verifica.isFile() == false) // imposta nessun file come file di tag xml
		{
			System.out.println("- Si consiglia di impostare il File tag di default (.xml) precaricato all'avvio di XMLStudio, nel menu' Impostazioni\n");
			fileXmlTagCorrente = null;
		}

		lookandfeelCorrente = p.getProperty("look");

		String wordWrapTemp = p.getProperty("wordwrap");

		if (wordWrapTemp.equals("true"))
			wordWrapDefault = new Boolean("TRUE");
		else
			wordWrapDefault = new Boolean("FALSE");
	}


    // **** carica le impostazioni di default dell'editor ****

    private void carica_ImpostazioniDefault()
    {
       	try
		{
			InputStream is = new FileInputStream("./Risorse/XMLStudio.default"); // creazione di oggetto is per la lettura
			BufferedInputStream bis = new BufferedInputStream(is); // creazione di oggetto bis per la lettura con buffer
			Properties proprieta_XMLStudioEditor = new Properties(); // creazione di oggetto proprieta' per l'editor
			proprieta_XMLStudioEditor.load(bis); // carica le proprieta' dall'input stream
			eseguiImpostazioniDefault(proprieta_XMLStudioEditor);
			bis.close();
		}

		catch(FileNotFoundException e)
		{
			System.err.println("File impostazioni default XMLStudio.default non trovato.");
		}

		catch(IOException e)
		{
			System.err.println("Lettura del file impostazioni default XMLStudio.default fallita.");
		}
	}


	// **** salva le impostazioni di default ****

	private void salva_ImpostazioniDefault()
	{
		// salva tutte le preferenze in un oggetto di tipo Properties

		try
		{
			OutputStream os = new FileOutputStream("./Risorse/XMLStudio.default");
			BufferedOutputStream bos = new BufferedOutputStream(os);
			Properties proprieta_XMLStudioEditor = new Properties();

			proprieta_XMLStudioEditor.setProperty("font", carattereDefault.getName());
			proprieta_XMLStudioEditor.setProperty("tipofont", "" + carattereDefault.getStyle());
			proprieta_XMLStudioEditor.setProperty("dimensionefont", "" + carattereDefault.getSize());
			proprieta_XMLStudioEditor.setProperty("colorefont", "" + coloreTestoDefault.getRGB());
			proprieta_XMLStudioEditor.setProperty("coloresfondo", "" + coloreSfondoDefault.getRGB());
			proprieta_XMLStudioEditor.setProperty("directory", directoryCorrente);
			proprieta_XMLStudioEditor.setProperty("filexmltag", fileXmlTagCorrente);
			proprieta_XMLStudioEditor.setProperty("look", lookandfeelCorrente);
			proprieta_XMLStudioEditor.setProperty("finestrax", finestraX);
			proprieta_XMLStudioEditor.setProperty("finestray", finestraY);
			proprieta_XMLStudioEditor.setProperty("divisorex", divisoreOrizzontale);
			proprieta_XMLStudioEditor.setProperty("divisorey", divisoreVerticale);
			proprieta_XMLStudioEditor.setProperty("wordwrap", wordWrapDefault.toString());

			proprieta_XMLStudioEditor.store(bos,"File proprieta' di default di XMLStudio."); // memorizza le proprieta' sul file

			bos.close();

			stato.setText("Aggiornamento del file XMLStudio.default avvenuto con successo.");
		}
		catch(Exception e)
		{
			System.err.println("Salvataggio del file XMLStudio.default fallito.");
			stato.setText("Salvataggio del file XMLStudio.default fallito.");
			return;
		}
	}


	// **** carica un file di impostazioni predefinito ****

	private void ripristinaImpostazioniDefault()
	{
		int n;
		byte buffer[] = new byte[256]; // crea un buffer costituito da 256 byte

		try
		{	// carica le impostazioni predefinite dal file XMLStudio.predefinito

			FileInputStream in = new FileInputStream("./Risorse/XMLStudio.predefinito"); // creazione di oggetto in per la lettura
			FileOutputStream out = new FileOutputStream("./Risorse/XMLStudio.default"); // creazione di oggetto out per la scrittura

			while ((n = in.read(buffer)) > -1) // legge un byte di dati dall'inputstream
				out.write(buffer, 0, n); // scrive n byte di dati sull'outputstream a partire dall'offset 0

			out.close(); // chiudi il file output stream e rilascia le risorse associate a questo stream
			in.close(); // chiudi il file input stream e rilascia le risorse associate a questo stream


			// riavvia l'editor per rendere effettive le nuove impostazioni

			azioneRiavviaEditor();
		}

		catch(FileNotFoundException e)
		{
			System.err.println("File impostazioni default XMLStudio.predefinito non trovato.");
		}

		catch(IOException e)
		{
			System.err.println("Lettura del file impostazioni default XMLStudio.predefinito fallita.");
		}
	}


	// **** crea i menu della finestra dell'editor ****

	private JMenuBar crea_MenuBar()
	{
		JMenuItem jmi_nuovo, jmi_apri, jmi_chiudi, jmi_salvacome, jmi_uscita, // definizione oggetti menu File
				jmi_stampawindows, jmi_stampastringa, jmi_stampa, // definizione oggetti per stampa menu File
				jmi_ultimo1, jmi_ultimo2, jmi_ultimo3, jmi_ultimo4, // definizione oggetti menu File relativi a ultimi file aperti
				jmi_memlinea, jmi_vaiamemlinea, // definizione oggetti menu
		          jmi_annulla, jmi_ripristina, jmi_taglia, jmi_copia, jmi_incolla, jmi_selezionatutto, // definizione oggetti menu Modifica
		          jmi_trova, jmi_trovatag, jmi_trovasuccessivo, jmi_sostituisci, jmi_linea, // definizione oggetti menu Ricerca
		          jmi_creaxml, jmi_benformatoxml, jmi_validoxml, jmi_creaalberoxml, jmi_aggiungitagxml, jmi_eliminatagxml, jmi_selezionatagxml, // definizione operazioni xml
		          jmi_modificaxml, jmi_modificaxml1, jmi_modificaxml2, jmi_modificaxml3, jmi_modificaxml4, jmi_modificaxml5, // modifica documento xml
		          jmi_ripristinacaratterispeciali, jmi_creadocumentovalido, jmi_creadtd, jmi_css,// definizione oggetti menu per modifiche al documento XML
		          jmi_carattere, jmi_coloretesto, jmi_coloresfondo, jmi_directory, jmi_filexmltag, jmi_dimensionefinestra, jmi_dimensionipredefinite, // definizione oggetti menu Impostazioni
		          jmi_nuovotag, jmi_apritag, jmi_salvatag, jmi_salvanometag, jmi_cancellaSelezioneJTable, // definizione oggetti menu Tag
		          jmi_javahelp, jmi_javahelpcontesto, jmi_guida, jmi_info; // definizione oggetti menu Aiuto

		JMenuBar menuBar = new JMenuBar(); // definizione oggetto per la barra dei menu

		JRadioButtonMenuItem jmi_aspetto1, jmi_aspetto2, jmi_aspetto3, jmi_aspetto4, jmi_aspetto5;

	    // definizione menu fileMenu

	     JMenu fileMenu = new JMenu("File");

		jmi_nuovo = new JMenuItem("Nuovo");
		jmi_nuovo.addActionListener(this);
		jmi_nuovo.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N, ActionEvent.CTRL_MASK));
		jmi_nuovo.setIcon(new ImageIcon(ClassLoader.getSystemResource("./Risorse/nuovo.gif")));
		fileMenu.add(jmi_nuovo);

		jmi_apri = new JMenuItem("Apri..");
		jmi_apri.addActionListener(this);
		jmi_apri.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_A, ActionEvent.CTRL_MASK));
		jmi_apri.setIcon(new ImageIcon(ClassLoader.getSystemResource("./Risorse/apri.gif")));
		fileMenu.add(jmi_apri);

		jmi_chiudi = new JMenuItem("Chiudi");
		jmi_chiudi.addActionListener(this);
		jmi_chiudi.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_D, ActionEvent.CTRL_MASK));
		fileMenu.add(jmi_chiudi);

		fileMenu.addSeparator();

		jmi_salva = new JMenuItem("Salva");
		jmi_salva.addActionListener(this);
		jmi_salva.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, ActionEvent.CTRL_MASK));
		jmi_salva.setIcon(new ImageIcon(ClassLoader.getSystemResource("./Risorse/salva.gif")));
		fileMenu.add(jmi_salva);

		jmi_salvacome = new JMenuItem("Salva come...");
		jmi_salvacome.addActionListener(this);
		fileMenu.add(jmi_salvacome);

		fileMenu.addSeparator();

		jmi_stampawindows = new JMenuItem("Stampa (Windows)");
		jmi_stampawindows.addActionListener(this);
		fileMenu.add(jmi_stampawindows);

		jmi_stampastringa = new JMenuItem("Stampa (passando stringa)");
		jmi_stampastringa.addActionListener(this);
		fileMenu.add(jmi_stampastringa);

		jmi_stampa = new JMenuItem("Stampa (grafica)");
		jmi_stampa.addActionListener(this);
		jmi_stampa.setIcon(new ImageIcon(ClassLoader.getSystemResource("./Risorse/stampa.gif")));
		fileMenu.add(jmi_stampa);

		fileMenu.addSeparator();

		jmi_uscita = new JMenuItem("Uscita");
		jmi_uscita.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F4, ActionEvent.ALT_MASK));
		jmi_uscita.addActionListener(this);
		fileMenu.add(jmi_uscita);

		fileMenu.addSeparator();

		if (ultimiFile.primo != null) // la condizione non e' messa in and altrimenti il programma va in crash se la riga del file e' nulla
		{
			if ((new File(ultimiFile.primo)).exists()) // il file realtivo al path del cammino caricato e' esistente
			{
				jmi_ultimo1 = new JMenuItem(ultimiFile.primo);
				jmi_ultimo1.addActionListener(this);
				fileMenu.add(jmi_ultimo1);
			}
			else
				ultimiFile.primo = null; // ""
		}

		if (ultimiFile.secondo != null)
		{
			if ((new File(ultimiFile.secondo)).exists()) // il file realtivo al path del cammino caricato e' esistente
			{
				jmi_ultimo2 = new JMenuItem(ultimiFile.secondo);
				jmi_ultimo2.addActionListener(this);
				fileMenu.add(jmi_ultimo2);
			}
			else
				ultimiFile.secondo = null;
		}

		if (ultimiFile.terzo != null)
		{
			if ((new File(ultimiFile.terzo)).exists()) // il file realtivo al path del cammino caricato e' esistente
			{
				jmi_ultimo3 = new JMenuItem(ultimiFile.terzo);
				jmi_ultimo3.addActionListener(this);
				fileMenu.add(jmi_ultimo3);
			}
			else
				ultimiFile.terzo = null;
		}

		if (ultimiFile.quarto != null)
		{
			if ((new File(ultimiFile.quarto)).exists()) // il file realtivo al path del cammino caricato e' esistente
			{
				jmi_ultimo4 = new JMenuItem(ultimiFile.quarto);
				jmi_ultimo4.addActionListener(this);
				fileMenu.add(jmi_ultimo4);
			}
			else
				ultimiFile.quarto = null;
		}

		// definizione menu modificaMenu

		JMenu modificaMenu = new JMenu("Modifica");

		jmi_annulla = new JMenuItem("Annulla");
		jmi_annulla.addActionListener(this);
		jmi_annulla.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Z, ActionEvent.CTRL_MASK));
		jmi_annulla.setIcon(new ImageIcon(ClassLoader.getSystemResource("./Risorse/annulla.gif")));
		modificaMenu.add(jmi_annulla);

		jmi_ripristina = new JMenuItem("Ripristina");
		jmi_ripristina.addActionListener(this);
		jmi_ripristina.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_R, ActionEvent.CTRL_MASK));
		jmi_ripristina.setIcon(new ImageIcon(ClassLoader.getSystemResource("./Risorse/ripristina.gif")));
		modificaMenu.add(jmi_ripristina);

		modificaMenu.addSeparator();

		jmi_taglia = new JMenuItem("Taglia");
		jmi_taglia.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_X, ActionEvent.CTRL_MASK));
		jmi_taglia.addActionListener(this);
		jmi_taglia.setIcon(new ImageIcon(ClassLoader.getSystemResource("./Risorse/taglia.gif")));
		modificaMenu.add(jmi_taglia);

		jmi_copia = new JMenuItem("Copia");
		jmi_copia.addActionListener(this);
		jmi_copia.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_C, ActionEvent.CTRL_MASK));
		jmi_copia.setIcon(new ImageIcon(ClassLoader.getSystemResource("./Risorse/copia.gif")));
		modificaMenu.add(jmi_copia);

		jmi_incolla = new JMenuItem("Incolla");
		jmi_incolla.addActionListener(this);
		jmi_incolla.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_V, ActionEvent.CTRL_MASK));
		jmi_incolla.setIcon(new ImageIcon(ClassLoader.getSystemResource("./Risorse/incolla.gif")));
		modificaMenu.add(jmi_incolla);

		modificaMenu.addSeparator();

		jmi_selezionatutto = new JMenuItem("Seleziona tutto");
		jmi_selezionatutto.addActionListener(this);
		//jmi_selezionatutto.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_H, ActionEvent.CTRL_MASK));
		modificaMenu.add(jmi_selezionatutto);

		// definizione menu ricercaMenu

		JMenu ricercaMenu = new JMenu("Ricerca");

		jmi_trova = new JMenuItem("Trova");
		jmi_trova.addActionListener(this);
		jmi_trova.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_T, ActionEvent.CTRL_MASK));
		jmi_trova.setIcon(new ImageIcon(ClassLoader.getSystemResource("./Risorse/trova.gif")));
		ricercaMenu.add(jmi_trova);

		jmi_trovatag = new JMenuItem("Trova tag");
		jmi_trovatag.addActionListener(this);
		jmi_trovatag.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F, ActionEvent.CTRL_MASK));
		jmi_trovatag.setIcon(new ImageIcon(ClassLoader.getSystemResource("./Risorse/trova.gif")));
		ricercaMenu.add(jmi_trovatag);

		jmi_trovasuccessivo = new JMenuItem("Trova successivo");
		jmi_trovasuccessivo.addActionListener(this);
		jmi_trovasuccessivo.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_U, ActionEvent.CTRL_MASK));
		jmi_trovasuccessivo.setIcon(new ImageIcon(ClassLoader.getSystemResource("./Risorse/trovas.gif")));
		ricercaMenu.add(jmi_trovasuccessivo);

		jmi_sostituisci = new JMenuItem("Sostituisci");
		jmi_sostituisci.addActionListener(this);
		jmi_sostituisci.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O, ActionEvent.CTRL_MASK));
		jmi_sostituisci.setIcon(new ImageIcon(ClassLoader.getSystemResource("./Risorse/sostituisci.gif")));
		ricercaMenu.add(jmi_sostituisci);

		ricercaMenu.addSeparator();

		jmi_linea = new JMenuItem("Vai a linea");
		jmi_linea.addActionListener(this);
		jmi_linea.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_L, ActionEvent.CTRL_MASK));
		ricercaMenu.add(jmi_linea);

		ricercaMenu.addSeparator();

		jmi_memlinea = new JMenuItem("Memorizza numero linea");
		jmi_memlinea.addActionListener(this);
		jmi_memlinea.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_E, ActionEvent.CTRL_MASK));
		jmi_memlinea.setIcon(new ImageIcon(ClassLoader.getSystemResource("./Risorse/memlinea.gif")));
		ricercaMenu.add(jmi_memlinea);

		jmi_vaiamemlinea = new JMenuItem("Vai a numero linea");
		jmi_vaiamemlinea.addActionListener(this);
		jmi_vaiamemlinea.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_M, ActionEvent.CTRL_MASK));
		jmi_vaiamemlinea.setIcon(new ImageIcon(ClassLoader.getSystemResource("./Risorse/vailinea.gif")));
		ricercaMenu.add(jmi_vaiamemlinea);

		// definizione menu xmlMenu

		JMenu xmlMenu = new JMenu("Xml");

		jmi_creaxml = new JMenuItem("Crea documento xml");
		jmi_creaxml.addActionListener(this);
		jmi_creaxml.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_G, ActionEvent.CTRL_MASK));
		jmi_creaxml.setIcon(new ImageIcon(ClassLoader.getSystemResource("./Risorse/creaxml.gif")));
		xmlMenu.add(jmi_creaxml);

		jmi_benformatoxml = new JMenuItem("Verifica documento xml ben formato");
		jmi_benformatoxml.addActionListener(this);
		jmi_benformatoxml.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_H, ActionEvent.CTRL_MASK));
		jmi_benformatoxml.setIcon(new ImageIcon(ClassLoader.getSystemResource("./Risorse/benformato.gif")));
		xmlMenu.add(jmi_benformatoxml);

		jmi_validoxml = new JMenuItem("Verifica documento xml valido");
		jmi_validoxml.addActionListener(this);
		jmi_validoxml.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_I, ActionEvent.CTRL_MASK));
		jmi_validoxml.setIcon(new ImageIcon(ClassLoader.getSystemResource("./Risorse/benformato.gif")));
		xmlMenu.add(jmi_validoxml);

		jmi_creaalberoxml = new JMenuItem("Aggiorna albero xml");
		jmi_creaalberoxml.addActionListener(this);
		jmi_creaalberoxml.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_J, ActionEvent.CTRL_MASK));
		jmi_creaalberoxml.setIcon(new ImageIcon(ClassLoader.getSystemResource("./Risorse/aggiorna.gif")));
		xmlMenu.add(jmi_creaalberoxml);

		jmi_aggiungitagxml = new JMenuItem("Aggiungi tag xml");
		jmi_aggiungitagxml.addActionListener(this);
		jmi_aggiungitagxml.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_K, ActionEvent.CTRL_MASK));
		jmi_aggiungitagxml.setIcon(new ImageIcon(ClassLoader.getSystemResource("./Risorse/tag.gif")));
		xmlMenu.add(jmi_aggiungitagxml);

		jmi_selezionatagxml = new JMenuItem("Seleziona paragrafo tag xml");
		jmi_selezionatagxml.addActionListener(this);
		jmi_selezionatagxml.setIcon(new ImageIcon(ClassLoader.getSystemResource("./Risorse/selblocco.gif")));
		xmlMenu.add(jmi_selezionatagxml);

		jmi_eliminatagxml = new JMenuItem("Elimina tag xml");
		jmi_eliminatagxml.addActionListener(this);
		jmi_eliminatagxml.setIcon(new ImageIcon(ClassLoader.getSystemResource("./Risorse/canctag.gif")));
		xmlMenu.add(jmi_eliminatagxml);

		xmlMenu.addSeparator();

		jmi_modificaxml = new JMenu("Modifica per documento ben formato");
		jmi_modificaxml.addActionListener(this);
		xmlMenu.add(jmi_modificaxml);

		jmi_modificaxml1 = new JMenuItem("Sostituisci &");
		jmi_modificaxml1.addActionListener(this);
		jmi_modificaxml.add(jmi_modificaxml1);

		jmi_modificaxml2 = new JMenuItem("Sostituisci <");
		jmi_modificaxml2.addActionListener(this);
		jmi_modificaxml.add(jmi_modificaxml2);

		jmi_modificaxml3 = new JMenuItem("Sostituisci >");
		jmi_modificaxml3.addActionListener(this);
		jmi_modificaxml.add(jmi_modificaxml3);

		jmi_modificaxml4 = new JMenuItem("Sostituisci '");
		jmi_modificaxml4.addActionListener(this);
		jmi_modificaxml.add(jmi_modificaxml4);

		jmi_modificaxml5 = new JMenuItem("Sostituisci \"");
		jmi_modificaxml5.addActionListener(this);
		jmi_modificaxml.add(jmi_modificaxml5);

		xmlMenu.addSeparator();

		jmi_ripristinacaratterispeciali = new JMenuItem("Ripristina caratteri speciali");
		jmi_ripristinacaratterispeciali.addActionListener(this);
		xmlMenu.add(jmi_ripristinacaratterispeciali);

		xmlMenu.addSeparator();

		jmi_creadtd = new JMenuItem("Crea dtd da documento xml");
		jmi_creadtd.addActionListener(this);
		xmlMenu.add(jmi_creadtd);

		jmi_creadocumentovalido = new JMenuItem("Crea dtd e documento xml valido");
		jmi_creadocumentovalido.addActionListener(this);
		xmlMenu.add(jmi_creadocumentovalido);

		jmi_css = new JMenuItem("Crea css associato al file xml");
		jmi_css.addActionListener(this);
		xmlMenu.add(jmi_css);

		// definizione menu tagMenu

		JMenu tagMenu = new JMenu("Tag");

		jmi_nuovotag = new JMenuItem("Crea nuovo file tag (.xml)");
		jmi_nuovotag.addActionListener(this);
		jmi_nuovotag.setIcon(new ImageIcon(ClassLoader.getSystemResource("./Risorse/nuovo.gif")));
		tagMenu.add(jmi_nuovotag);

		jmi_apritag = new JMenuItem("Apre file tag (.xml)");
		jmi_apritag.addActionListener(this);
		jmi_apritag.setIcon(new ImageIcon(ClassLoader.getSystemResource("./Risorse/apri.gif")));
		tagMenu.add(jmi_apritag);

		jmi_salvatag = new JMenuItem("Salva file tag (.xml)");
		jmi_salvatag.addActionListener(this);
		jmi_salvatag.setIcon(new ImageIcon(ClassLoader.getSystemResource("./Risorse/salva.gif")));
		tagMenu.add(jmi_salvatag);

		jmi_salvanometag = new JMenuItem("Salva nome file tag (.xml)");
		jmi_salvanometag.addActionListener(this);
		jmi_salvanometag.setIcon(new ImageIcon(ClassLoader.getSystemResource("./Risorse/salvanome.gif")));
		tagMenu.add(jmi_salvanometag);

		tagMenu.addSeparator();

		jmi_aggiunginodotag = new JMenuItem("Aggiungi nuovo nodo tag xml");
		jmi_aggiunginodotag.addActionListener(this);
		jmi_aggiunginodotag.setIcon(new ImageIcon(ClassLoader.getSystemResource("./Risorse/creaxml.gif")));
		tagMenu.add(jmi_aggiunginodotag);

		jmi_modificanodotag = new JMenuItem("Modifica nodo tag xml");
		jmi_modificanodotag.addActionListener(this);
		jmi_modificanodotag.setIcon(new ImageIcon(ClassLoader.getSystemResource("./Risorse/tag.gif")));
		tagMenu.add(jmi_modificanodotag);

		jmi_eliminanodotag = new JMenuItem("Elimina nodo tag xml");
		jmi_eliminanodotag.addActionListener(this);
		jmi_eliminanodotag.setIcon(new ImageIcon(ClassLoader.getSystemResource("./Risorse/cancella.gif")));
		tagMenu.add(jmi_eliminanodotag);

		tagMenu.addSeparator();

		jmi_aggiungiattributonodotag = new JMenuItem("Aggiungi attributo nodo tag xml");
		jmi_aggiungiattributonodotag.addActionListener(this);
		jmi_aggiungiattributonodotag.setIcon(new ImageIcon(ClassLoader.getSystemResource("./Risorse/creaxml.gif")));
		tagMenu.add(jmi_aggiungiattributonodotag);

		jmi_modificaattributonodotag = new JMenuItem("Modifica attributo nodo tag xml");
		jmi_modificaattributonodotag.addActionListener(this);
		jmi_modificaattributonodotag.setIcon(new ImageIcon(ClassLoader.getSystemResource("./Risorse/tag.gif")));
		tagMenu.add(jmi_modificaattributonodotag);

		jmi_eliminaattributonodotag = new JMenuItem("Elimina attributo nodo tag xml");
		jmi_eliminaattributonodotag.addActionListener(this);
		jmi_eliminaattributonodotag.setIcon(new ImageIcon(ClassLoader.getSystemResource("./Risorse/cancella.gif")));
		tagMenu.add(jmi_eliminaattributonodotag);

		tagMenu.addSeparator();

		jmi_cancellaSelezioneJTable = new JMenuItem("Deseleziona celle attributi e valori");
		jmi_cancellaSelezioneJTable.addActionListener(this);
		jmi_cancellaSelezioneJTable.setIcon(new ImageIcon(ClassLoader.getSystemResource("./Risorse/cancellatable.gif")));
		tagMenu.add(jmi_cancellaSelezioneJTable);

		// definizione menu impostazioniMenu

		JMenu impostazioniMenu = new JMenu("Impostazioni");

		jmi_wordwrap = new JCheckBoxMenuItem("A capo automatico");
		jmi_wordwrap.addActionListener(this);
		impostazioniMenu.add(jmi_wordwrap);

		jmi_carattere = new JMenuItem("Tipo carattere");
		jmi_carattere.addActionListener(this);
		impostazioniMenu.add(jmi_carattere);

		jmi_coloretesto = new JMenuItem("Colore testo");
		jmi_coloretesto.addActionListener(this);
		impostazioniMenu.add(jmi_coloretesto);

		jmi_coloresfondo = new JMenuItem("Colore sfondo");
		jmi_coloresfondo.addActionListener(this);
		impostazioniMenu.add(jmi_coloresfondo);

		jmi_directory = new JMenuItem("Directory di lavoro");
		jmi_directory.addActionListener(this);
		impostazioniMenu.add(jmi_directory);

		jmi_filexmltag = new JMenuItem("File tag di default (.xml)");
		jmi_filexmltag.addActionListener(this);
		impostazioniMenu.add(jmi_filexmltag);

		jmi_dimensionefinestra = new JMenuItem("Dimensione finestra");
		jmi_dimensionefinestra.addActionListener(this);
		impostazioniMenu.add(jmi_dimensionefinestra);

		// definisce un nuovo menu' e il codice per sottomenu' di definizione dell'aspetto della finestra

		JMenu jmi_aspetto_menu = new JMenu("Aspetto finestra");
		jmi_aspetto_menu.addActionListener(this);
		impostazioniMenu.add(jmi_aspetto_menu);

		jmi_aspetto1 = new JRadioButtonMenuItem("Metal");
		jmi_aspetto1.addActionListener(this);
		jmi_aspetto_menu.add(jmi_aspetto1);

		jmi_aspetto2 = new JRadioButtonMenuItem("System");
		jmi_aspetto2.addActionListener(this);
		jmi_aspetto_menu.add(jmi_aspetto2);

		jmi_aspetto3 = new JRadioButtonMenuItem("Motif");
		jmi_aspetto3.addActionListener(this);
		jmi_aspetto_menu.add(jmi_aspetto3);

		jmi_aspetto4 = new JRadioButtonMenuItem("Windows");
		jmi_aspetto4.addActionListener(this);
		jmi_aspetto_menu.add(jmi_aspetto4);

		jmi_aspetto5 = new JRadioButtonMenuItem("Mac");
		jmi_aspetto5.addActionListener(this);
		jmi_aspetto_menu.add(jmi_aspetto5);

		ButtonGroup group = new ButtonGroup();
        	group.add(jmi_aspetto1);
        	group.add(jmi_aspetto2);
        	group.add(jmi_aspetto3);
        	group.add(jmi_aspetto4);
        	group.add(jmi_aspetto5);

        	if (lookandfeelCorrente.equals("Metal"))
        		jmi_aspetto1.setSelected(true);

        	if (lookandfeelCorrente.equals("System"))
        		jmi_aspetto2.setSelected(true);

        	if (lookandfeelCorrente.equals("Motif"))
        		jmi_aspetto3.setSelected(true);

        	if (lookandfeelCorrente.equals("Windows"))
        		jmi_aspetto4.setSelected(true);

        	if (lookandfeelCorrente.equals("Mac"))
        		jmi_aspetto5.setSelected(true);

        	impostazioniMenu.addSeparator();

        	jmi_dimensionipredefinite = new JMenuItem("Ripristina impostazioni predefinite");
		jmi_dimensionipredefinite.addActionListener(this);
		impostazioniMenu.add(jmi_dimensionipredefinite);

		// definizione menu helpMenu

		JMenu helpMenu = new JMenu("?");

		jmi_javahelpcontesto = new JMenuItem("Guida contestuale");
		//jmi_javahelpcontesto.addActionListener(this);
		jmi_javahelpcontesto.addActionListener(new CSH.DisplayHelpAfterTracking(hb));
		jmi_javahelpcontesto.setIcon(new ImageIcon(ClassLoader.getSystemResource("./Risorse/helpcntx.gif")));
		helpMenu.add(jmi_javahelpcontesto);

		jmi_javahelp = new JMenuItem("Guida del programma");
		//jmi_javahelp.addActionListener(this);
		helpMenu.add(jmi_javahelp);
		CSH.setHelpIDString(jmi_javahelp, "top");
		jmi_javahelp.addActionListener(new CSH.DisplayHelpFromSource(hb));

		jmi_guida = new JMenuItem("Guida testuale");
		jmi_guida.addActionListener(this);
		helpMenu.add(jmi_guida);

		jmi_info = new JMenuItem("Informazioni sul programma");
		jmi_info.addActionListener(this);
		helpMenu.add(jmi_info);

		// aggiunge i singoli menu alla barra dei menu

		menuBar.add(fileMenu);
		menuBar.add(modificaMenu);
		menuBar.add(ricercaMenu);
		menuBar.add(xmlMenu);
		menuBar.add(tagMenu);
		menuBar.add(impostazioniMenu);
		menuBar.add(helpMenu);

		return menuBar; // ritorna la barra dei menu dopo tutti gli aggiornamenti
	}


	// **** crea la ToolBar dell'editor ****

	private JToolBar crea_Barra()
	{
		JToolBar barraComandi = new JToolBar("Barra comandi");
		CSH.setHelpIDString(barraComandi, "oggettoJToolBar");

		nuovo = new JButton(new ImageIcon(ClassLoader.getSystemResource("./Risorse/nuovo.gif")));
		barraComandi.add(nuovo);
		nuovo.addActionListener(this);
		nuovo.setToolTipText("Nuovo file");

		apri = new JButton(new ImageIcon(ClassLoader.getSystemResource("./Risorse/apri.gif")));
		barraComandi.add(apri);
		apri.addActionListener(this);
		apri.setToolTipText("Apri il file");

		salva = new JButton(new ImageIcon(ClassLoader.getSystemResource("./Risorse/salva.gif")));
		barraComandi.add(salva);
		salva.addActionListener(this);
		salva.setToolTipText("Salva il file");

		stampa = new JButton(new ImageIcon(ClassLoader.getSystemResource("./Risorse/stampa.gif")));
		barraComandi.add(stampa);
		stampa.addActionListener(this);
		stampa.setToolTipText("Stampa il file (modalita' grafica)");

		barraComandi.addSeparator();

		annulla = new JButton(new ImageIcon(ClassLoader.getSystemResource("./Risorse/annulla.gif")));
		barraComandi.add(annulla);
		annulla.addActionListener(this);
		annulla.setToolTipText("Annulla l'ultima operazione");

		ripristina = new JButton(new ImageIcon(ClassLoader.getSystemResource("./Risorse/ripristina.gif")));
		barraComandi.add(ripristina);
		ripristina.addActionListener(this);
		ripristina.setToolTipText("Ripristina l'ultima operazione");

		barraComandi.addSeparator();

		taglia = new JButton(new ImageIcon(ClassLoader.getSystemResource("./Risorse/taglia.gif")));
		barraComandi.add(taglia);
		taglia.addActionListener(this);
		taglia.setToolTipText("Taglia il contenuto selezionato");

		copia = new JButton(new ImageIcon(ClassLoader.getSystemResource("./Risorse/copia.gif")));
		barraComandi.add(copia);
		copia.addActionListener(this);
		copia.setToolTipText("Copia il contenuto selezionato");

		incolla = new JButton(new ImageIcon(ClassLoader.getSystemResource("./Risorse/incolla.gif")));
		barraComandi.add(incolla);
		incolla.addActionListener(this);
		incolla.setToolTipText("Incolla il contenuto selezionato");

		barraComandi.addSeparator();

		memolinea = new JButton(new ImageIcon(ClassLoader.getSystemResource("./Risorse/memlinea.gif")));
		barraComandi.add(memolinea);
		memolinea.addActionListener(this);
		memolinea.setToolTipText("Memorizza il numero di linea del cursore");

		vaimemolinea = new JButton(new ImageIcon(ClassLoader.getSystemResource("./Risorse/vailinea.gif")));
		barraComandi.add(vaimemolinea);
		vaimemolinea.addActionListener(this);
		vaimemolinea.setToolTipText("Sposta il cursore al numero di linea memorizzato");

		barraComandi.addSeparator();

		trova = new JButton(new ImageIcon(ClassLoader.getSystemResource("./Risorse/trova.gif")));
		barraComandi.add(trova);
		trova.addActionListener(this);
		trova.setToolTipText("Trova il testo nel documento");

		trovasuccessivo = new JButton(new ImageIcon(ClassLoader.getSystemResource("./Risorse/trovas.gif")));
		barraComandi.add(trovasuccessivo);
		trovasuccessivo.addActionListener(this);
		trovasuccessivo.setToolTipText("Ripete la ricerca del testo nel documento");

		sostituisci = new JButton(new ImageIcon(ClassLoader.getSystemResource("./Risorse/sostituisci.gif")));
		barraComandi.add(sostituisci);
		sostituisci.addActionListener(this);
		sostituisci.setToolTipText("Trova il testo nel documento e lo sostituisce");

		barraComandi.addSeparator();

		creaxml = new JButton(new ImageIcon(ClassLoader.getSystemResource("./Risorse/creaxml.gif")));
		barraComandi.add(creaxml);
		creaxml.addActionListener(this);
		creaxml.setToolTipText("Crea il documento xml");

		benformatoxml = new JButton(new ImageIcon(ClassLoader.getSystemResource("./Risorse/benformato.gif")));
		barraComandi.add(benformatoxml);
		benformatoxml.addActionListener(this);
		benformatoxml.setToolTipText("Salva il file xml e verifica che sia ben formato");

		aggiornaxml = new JButton(new ImageIcon(ClassLoader.getSystemResource("./Risorse/aggiorna.gif")));
		barraComandi.add(aggiornaxml);
		aggiornaxml.addActionListener(this);
		aggiornaxml.setToolTipText("Salva il file xml, verifica che sia ben formato e aggiorna l'albero xml relativo al documento");

		aggiungitagxml = new JButton(new ImageIcon(ClassLoader.getSystemResource("./Risorse/tag.gif")));
		barraComandi.add(aggiungitagxml);
		aggiungitagxml.addActionListener(this);
		aggiungitagxml.setToolTipText("Aggiunge al documento il tag xml relativo al nodo selezionato");

		barraComandi.addSeparator();

		helpcontesto = new JButton(new ImageIcon(ClassLoader.getSystemResource("./Risorse/helpcntx.gif")));
		barraComandi.add(helpcontesto);
		helpcontesto.addActionListener(new CSH.DisplayHelpAfterTracking(hb));
		helpcontesto.setToolTipText("Apre la guida in linea relativa all'oggetto selezionato");

		return barraComandi;
	}


	// **** creazione della barra di stato dell'editor ****

	private JPanel crea_StatusBar()
	{
		stato = new JLabel("Pronto."); // label che evidenzia lo stato e le operazioni del programma
		colonnerighe = new JLabel("Riga: " + 0 + " Colonna: " + 0 + "  "); // label per i numeri di colonna e riga
		JPanel pannelloLabel = new JPanel(new BorderLayout(3,3)); // pannello per contenere le due label
		pannelloLabel.add(stato, BorderLayout.WEST); // posizionamento della label di stato
		pannelloLabel.add(colonnerighe, BorderLayout.EAST); // posizionamento della label colonnerighe

		return pannelloLabel;
	}


	// **** creazione pannello laterale dell'editor ****

	private JPanel crea_PannelloAlberoXml()
	{
		// crea un nodo generico che non ha ne' padre, ne' figli, ma che consente di avere figli
		DefaultMutableTreeNode nodoRadice = new DefaultMutableTreeNode("Albero xml vuoto");
		modelloAlbero = new DefaultTreeModel(nodoRadice); // crea un albero, passandovi la radice, in cui ogni nodo ha dei figli
		// restituisce un'istanza di JTree che mostra il nodo radice e l'albero e' creato usando uno specifico modello dati
		alberoXml = new JTree(modelloAlbero);
		// imposta la selezione dell'albero a SINGLE_TREE_SELECTION: consente di selezionare solo un nodo dell'albero per volta
		alberoXml.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
		alberoXml.setShowsRootHandles(true); // gli handles sono mostrati
		alberoXml.setEditable(false); // imposta l'albero JTree come non editabile
		CSH.setHelpIDString(alberoXml, "oggettoAlberoXml");

	     RenderAlbero render = new RenderAlbero(); // definisce un nuovo oggetto render di tipo RenderAlbero
	     alberoXml.setCellRenderer(render); // applica all'albero JTree il render con le associazioni di icone e tooltip relative

		JScrollPane scorreAlbero = new JScrollPane(); // viene creato uno scrollpane per l'albero JTree che visualizza il documento Xml
		scorreAlbero.getViewport().add(alberoXml); // l'albero e' posizionato in uno scrollpane
		scorreAlbero.setToolTipText("Rappresentazione ad albero del documento xml");

		/* modellotavolaAttributiXml e' un oggetto di tipo TavolaAttributi con campi nodoCorrente e
		   attributiCorrenti ed implementa TableModel, interfaccia usata da JTable per interrogare
		   un modello di dati ti tipo tabulare. Sono sufficienti due righe di codice per permettere
		   alla JTable di visualizzare qualsiasi modello dati che implementi TableModel */

		modellotavolaAttributiXml = new TavolaAttributi(); // nuovo oggetto TavolaAttributi che estende AbstractTableModel
		tavolaAttributiXml = new JTable(modellotavolaAttributiXml); // assegna alla JTable l'interfaccia specifica appena creata

		JScrollPane scorreTavola = new JScrollPane(tavolaAttributiXml);
		scorreTavola.getViewport().setBackground(tavolaAttributiXml.getBackground());

		JSplitPane splitAlbero = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, scorreAlbero, scorreTavola);
		splitAlbero.setDividerLocation((Integer.parseInt(finestraX))-250);
		//splitAlbero.setDividerSize(30);
		splitAlbero.setContinuousLayout(true);
		splitAlbero.setOneTouchExpandable(true); // fornisce simbolo per rapida esapansione-chiusura splitPane
		getContentPane().add(splitAlbero, BorderLayout.CENTER);

		JPanel pannelloAlberoXml = new JPanel(new BorderLayout());
		pannelloAlberoXml.add(splitAlbero, BorderLayout.CENTER);

		/* Se si e' interessati nel sapere quando la selezione cambia, bisogna implementare l'interfaccia
		   TreeSelectionListener ed aggiungere l'istanza usando il metodo addTreeSelectionListener.
		   valueChanged sara' invocato (una volta sola) quando cambia la selezione ovvero quando l'utente
		   clicca due volte consecutive sullo stesso nodo */

		// e' un listener che e' modificato quando cambia la selezione in un TreeSelectionModel*/
		TreeSelectionListener listenerAlbero = new TreeSelectionListener() // per rilevare quando l'utente seleziona un nodo nell'albero
		{
			public void valueChanged(TreeSelectionEvent e) // richiamato ogni qualvolta cambia la selezione di un valore
			{
				Node nodoselezionato = restituisciNodoSelezionato();

				// aggiorna l'albero con il nodo attualmente selezionato
				aggiornaAttributiNodoInTavola(nodoselezionato); // il valore null e' accettato
			}
		};
		alberoXml.addTreeSelectionListener(listenerAlbero); // assegna all'albero Xml il listener definito per eventi TreeSelection

		return (pannelloAlberoXml);
    	}


    	// **** carica l'attuale nodo selezionato e lo restituisce come istanza di XmlNodo ****

	public XmlNodo restituisciNodoSelezionatoAlbero()
	{
		TreePath path = alberoXml.getSelectionPath(); // restituisce il path del nodo selezionato grazie a getSelectionPath
		if (path == null) // restituisce null se il path ottenuto e' null
			return null;
		Object oggetto = path.getLastPathComponent(); // in oggetto restituisce l'ultimo componente che ha determinato questo path
		if (!(oggetto instanceof XmlNodo)) // se l'oggetto non e' istanza di XmlNodo ritorna null
			return null;
		return (XmlNodo)oggetto; // se l'oggetto che ha determinato l'ultimo path e' di tipo XmlNodo, restituiscilo con un casting
	}


	// **** restituisce l'attuale nodo selezionato richiamando restituisciNodoSelezionatoAlbero ****

	public Node restituisciNodoSelezionato()
	{
		XmlNodo nodoAlberoXml = restituisciNodoSelezionatoAlbero(); // in nodoAlberoXml viene messo il nodo attualmente selezionato
		if (nodoAlberoXml == null)
			return null;
		return nodoAlberoXml.restituisciXmlNodo(); // restituisce da un oggetto XmlNodo il nodo di tipo Node
	}


	// **** aggiorna l'albero xml con il nodo attualmente selezionato ****

	public void aggiornaAttributiNodoInTavola(Node nodo)
	{
		/* modellotavolaAttributiXml e' un oggetto di tipo TavolaAttributi con campi nodoCorrente e
		   attributiCorrenti ed implementa TableModel, interfaccia usata da JTable per interrogare
		   un modello di dati ti tipo tabulare */

		modellotavolaAttributiXml.assegnaNodo(nodo);

		/* TableModelEvent e' usato per notificare i listener che il modello della tavola e' cambiato
		   Tutte le righe contenenti dati nella tavola sono cambiate, i listener devono perdere ogni stato
		   basato sulle precedenti righe e richiedere al TableModel il nuovo numero di linee e di valori */

		tavolaAttributiXml.tableChanged(new TableModelEvent(modellotavolaAttributiXml));
	}


	// **** metodo invocato per creare i menu' Popup che appaiono premendo il pulsante destro del mouse sulla JTextArea ****

	private void creaMenuPopup()
	{
	   JMenuItem menuItem; // definisce un oggetto di tipo JMenuItem per la definizione degli item dei menu' popup

	   popupAreaTesto = new JPopupMenu();

	   menuItem = new JMenuItem("Tag testo");
	   menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_K, ActionEvent.CTRL_MASK));
	   menuItem.setIcon(new ImageIcon(ClassLoader.getSystemResource("./Risorse/tag.gif")));
        menuItem.addActionListener(this);
        menuItem.setActionCommand("tag testo popup");
        popupAreaTesto.add(menuItem);

        menuItem = new JMenuItem("Seleziona paragrafo tag");
	   menuItem.setIcon(new ImageIcon(ClassLoader.getSystemResource("./Risorse/selblocco.gif")));
        menuItem.addActionListener(this);
        menuItem.setActionCommand("seleziona tag testo popup");
        popupAreaTesto.add(menuItem);

        menuItem = new JMenuItem("Elimina tag testo");
	   menuItem.setIcon(new ImageIcon(ClassLoader.getSystemResource("./Risorse/canctag.gif")));
        menuItem.addActionListener(this);
        menuItem.setActionCommand("elimina tag testo popup");
        popupAreaTesto.add(menuItem);

        popupAreaTesto.addSeparator();

        menuItem = new JMenuItem("Taglia");
        menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_X, ActionEvent.CTRL_MASK));
        menuItem.setIcon(new ImageIcon(ClassLoader.getSystemResource("./Risorse/taglia.gif")));
        menuItem.addActionListener(this);
        menuItem.setActionCommand("taglia testo popup");
        popupAreaTesto.add(menuItem);

        menuItem = new JMenuItem("Copia");
        menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_C, ActionEvent.CTRL_MASK));
        menuItem.setIcon(new ImageIcon(ClassLoader.getSystemResource("./Risorse/copia.gif")));
        menuItem.addActionListener(this);
        menuItem.setActionCommand("copia testo popup");
        popupAreaTesto.add(menuItem);

        menuItem = new JMenuItem("Incolla");
        menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_V, ActionEvent.CTRL_MASK));
        menuItem.setIcon(new ImageIcon(ClassLoader.getSystemResource("./Risorse/incolla.gif")));
        menuItem.addActionListener(this);
        menuItem.setActionCommand("incolla testo popup");
        popupAreaTesto.add(menuItem);

        popupAreaTesto.setOpaque(true);
    	   popupAreaTesto.setLightWeightPopupEnabled(true);

        // aggiunge al componente JTextArea il listener per fare comparire il menu' popup
        MouseListener popupListener = new PopupListener(popupAreaTesto);
        areaTesto.addMouseListener(popupListener);
	}


    	// **** imposta le proprieta' dell'area di testo dell'editor ****

    	private void impostaProprietaAreaTesto(JTextArea areaTesto)
	{
		// sets the preferences of each new and opened file.
		//areaTesto.setSelectedTextColor(Color.green);
		//areaTesto.setSelectionColor(Color.yellow);

		areaTesto.setBackground(coloreSfondoDefault);
		areaTesto.setForeground(coloreTestoDefault);
		areaTesto.setFont(carattereDefault);

		areaTesto.setLineWrap(wordWrapDefault.booleanValue()); // imposta se e' attivo o meno il word-wrap nell'area testo
		areaTesto.setWrapStyleWord(wordWrapDefault.booleanValue()); // imposta lo stile di word-wrapping
		jmi_wordwrap.setState(wordWrapDefault.booleanValue()); // modifica la selezione nel menu'
	}


	// **** creazione area di testo dell'editor ****

    	private JScrollPane crea_AreaTesto() // JScrollPane
    	{
		//areaTesto = new JTextArea(); // definizione dell'oggetto area di testo
		areaTesto = new UndoableTextArea();
		areaTesto.addCaretListener(this); // rileva i movimenti del cursore per il calcolo di riga e colonna
		impostaProprietaAreaTesto(areaTesto);
		JScrollPane areaScorrimento = new JScrollPane(areaTesto); // vista scrollabile per JTextArea
		creaMenuPopup(); // crea i menu' popup relativi all'areaTesto
		CSH.setHelpIDString(areaScorrimento, "oggettoJTextArea");

		return areaScorrimento;
	}


	// **** rimuove il carattere  che e' presente caricando i file ****

	private void rimuoviCarattere()
	{
		boolean modificato = false;

		String testo = areaTesto.getText();
		int posizioneCarattere = testo.indexOf("") ;
        	while ( posizioneCarattere >= 0 )
        	{
            	testo = testo.substring(0, posizioneCarattere ) + testo.substring(posizioneCarattere + "1".length() ) ;
               posizioneCarattere = testo.indexOf("", posizioneCarattere);
               modificato = true;
          }
		if (modificato == true)
		{
          	areaTesto.setText(testo);
			areaTesto.setCaretPosition(0);
		}
	}


	// **** creazione split pane centrale dell'editor ****

	private JSplitPane crea_SplitPaneCentrale()
	{
		splitPaneCentrale = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
		splitPaneCentrale.setContinuousLayout(true);
		splitPaneCentrale.setOneTouchExpandable(true); // fornisce simbolo per rapida esapansione-chiusura splitPane
        	splitPaneCentrale.setDividerLocation(Integer.parseInt(divisoreVerticale)); // impostazione posizione delimitatore verticale
        	splitPaneCentrale.setLeftComponent(crea_AreaTesto());
        	splitPaneCentrale.setRightComponent(crea_PannelloAlberoXml());

		splitPaneCentrale2 = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
		splitPaneCentrale2.setContinuousLayout(true);
		splitPaneCentrale2.setOneTouchExpandable(true); // fornisce simbolo per rapida esapansione-chiusura splitPane
        	splitPaneCentrale2.setDividerLocation(Integer.parseInt(divisoreOrizzontale)); // impostazione posizione delimitatore orizzontale
        	splitPaneCentrale2.setTopComponent(splitPaneCentrale);
        	// splitPaneCentrale2.setBottomComponent(crea_PannelloAlberoXml()); linea di prova
        	oggettoXmlTag = new XmlTag(this); // crea l'oggetto
        	CSH.setHelpIDString(oggettoXmlTag, "oggettoXmlTag");

        	splitPaneCentrale2.setBottomComponent(oggettoXmlTag);

        	return splitPaneCentrale2;
	}


	// **** dichiarazione di un costruttore per la classe ****

	private XMLStudio() // costruttore per la classe XMLStudio relativa all'editor
	{
		// inizializzazioni dell'editor
		creaJavaHelp();

		carica_ImpostazioniDefault(); // carica le proprieta' di default dell'editor scelte dall'utente

		initLookAndFeel(lookandfeelCorrente); // richiama il metodo per l'impostazione del look e feel dell'interfaccia grafica

		screenSize = Toolkit.getDefaultToolkit().getScreenSize(); // rilevazione dimensioni dello schermo
		//this.setSize(screenSize); imposta la dimensione della finestra dell'applicazione a tutto schermo

		//GraphicsDevice device = GraphicsEnvironment.getLocalGraphicsEnvironment().getDefaultScreenDevice(); // altro metodo per impostare a tutto schermo l'applicazione
      	//device.setFullScreenWindow(this);

		Dimension size = this.getSize(); // centra il JFrame al centro dello schermo
		screenSize.height = screenSize.height/2;
		screenSize.width = screenSize.width/2;
		size.height = size.height/2;
		size.width = size.width/2;
		this.setLocation(screenSize.height - size.height, screenSize.width - size.width);

		//GraphicsDevice device = GraphicsEnvironment.getLocalGraphicsEnvironment().getDefaultScreenDevice();
      	//device.setFullScreenWindow(this);

		ultimiFile = new StoriaFile(); // crea un oggetto di tipo StoriaFile

		setTitle("XMLStudio"); // impostazione del titolo
		setJMenuBar(crea_MenuBar()); // creazione della barra dei menu

		Container contenitoreComponenti = getContentPane(); // ritorna l'oggetto ContentPane per il frame corrente
		contenitoreComponenti.setLayout(new BorderLayout()); // impostazione del layout del ContentPane del Frame

		contenitoreComponenti.add(crea_Barra(), BorderLayout.NORTH); // creazione e posizionamento ToolBar dell'editor
		contenitoreComponenti.add(crea_SplitPaneCentrale(), BorderLayout.CENTER); // inserimento dell'area di testo
		contenitoreComponenti.add(crea_StatusBar(), BorderLayout.SOUTH); // creazione e posizionamento StatusBar dell'editor

		// stato.setText("Dimensione schermo " + screenSize.toString());
		// stato.setText(carattereDefault.toString());

		this.setIconImage(Toolkit.getDefaultToolkit().createImage(getClass ().getResource("./Risorse/icona.gif")));
		azioneSalvaPulsante(false); // disabilita il pulsante per il salvataggio del file

		addWindowListener(this);  // per la gestione dell'evento WindowClosing della finestra dell'editor
	}


	// **** il metodo windowClosing gestisce la chiusura della finestra ****

	public void windowClosing(WindowEvent event)
	{
		azioneUscita();
	}


     // **** sette metodi relativi alla finestra dell'editor, implementabili se necessario ****

	public void windowIconified(WindowEvent event)
	{
	}

	public void windowDeiconified(WindowEvent event)
	{
	}

	public void windowClosed(WindowEvent event)
	{
	}

	public void windowDeactivated(WindowEvent event)
	{
	}

	public void windowActivated(WindowEvent event)
	{
	}

	public void windowOpened(WindowEvent event)
	{
	}


	// **** rileva il movimento del cursore su areaTesto ****

	public void caretUpdate(CaretEvent evento)
	{
		try
		{
			areaTesto.requestFocus(); // richiama il focus sulla JTextArea areaTesto - disatt
			int colonna = evento.getDot() - areaTesto.getLineStartOffset(areaTesto.getLineOfOffset(evento.getDot())) + 1;
			riga = areaTesto.getLineOfOffset(evento.getDot()) + 1;
			colonnerighe.setText("Riga: " + riga + " Colonna: " + colonna + "  ");
			stato.setText("Pronto."); // indica l'avvenuto caricamento
			azioneSalvaPulsante(true); // abilita il pulsante per il salvataggio del file
			areaTesto.requestFocus(); // richiama il focus sulla JTextArea areaTesto
		}
		catch(Exception e)
		{
			System.err.println("Errore di rilevamento movimento cursore.");
		}
	}


	// **** metodi invocati quando avviene un action event ****

	public void actionPerformed(ActionEvent e)
	{
		String evento = e.getActionCommand();

		if (evento.equals("Nuovo") || e.getSource() == nuovo)
			azioneNuovoFile();
		if (evento.equals("Apri..") || e.getSource() == apri)
			azioneApriFile();
		if (evento.equals("Chiudi"))
			azioneChiudiFile();
		if (evento.equals("Salva") || e.getSource() == salva)
			azioneSalvaFile();
		if (evento.equals("Salva come..."))
			azioneSalvaFileNome();
		if (evento.equals("Stampa (Windows)"))
			azioneStampaWindows();
		if (evento.equals("Stampa (passando stringa)"))
			azioneStampaStringa();
		if (evento.equals("Stampa (grafica)") || e.getSource() == stampa)
			azioneStampaGrafica();

		if (evento.equals("Uscita"))
			azioneUscita();

		if (evento.equals(ultimiFile.primo))
			azioneApriUltimoFile(ultimiFile.primo);
		if (evento.equals(ultimiFile.secondo))
			azioneApriUltimoFile(ultimiFile.secondo);
		if (evento.equals(ultimiFile.terzo))
			azioneApriUltimoFile(ultimiFile.terzo);
		if (evento.equals(ultimiFile.quarto))
			azioneApriUltimoFile(ultimiFile.quarto);

		if (evento.equals("Annulla") || e.getSource() == annulla)
			azioneAnnulla();
		if (evento.equals("Ripristina") || e.getSource() == ripristina)
			azioneRipristina();
		if (evento.equals("Taglia") || e.getSource() == taglia)
			azioneTaglia();
		if (evento.equals("Copia") || e.getSource() == copia)
			azioneCopia();
		if (evento.equals("Incolla") || e.getSource() == incolla)
			azioneIncolla();
		if (evento.equals("Seleziona tutto"))
			azioneSelezionaTutto();

		if (evento.equals("Trova") || e.getSource() == trova)
			azioneTrova();
		if (evento.equals("Trova tag"))
			azioneTrovaTag();
		if (evento.equals("Trova successivo") || e.getSource() == trovasuccessivo)
			azioneTrovaSuccessivo();
		if (evento.equals("Sostituisci") || e.getSource() == sostituisci)
			azioneSostituisci();
		if (evento.equals("Vai a linea"))
			azioneVaiALinea();
		if (evento.equals("Memorizza numero linea") || e.getSource() == memolinea)
			azioneMemorizzaLinea();
		if (evento.equals("Vai a numero linea") || e.getSource() == vaimemolinea)
			azioneVaiAMemorizzaLinea();

		if (evento.equals("Crea documento xml") || e.getSource() == creaxml)
			azioneCreaXml();
		if (evento.equals("Verifica documento xml ben formato") || e.getSource() == benformatoxml)
			azioneBenFormatoXml();
		if (evento.equals("Verifica documento xml valido"))
			azioneValidoXml();
		if (evento.equals("Aggiorna albero xml") || e.getSource() == aggiornaxml)
			azioneCreaAlberoXml();
		if (evento.equals("Aggiungi tag xml") || e.getSource() == aggiungitagxml)
			azioneAggiungiTagXml();
		if (evento.equals("Elimina tag xml"))
			azioneRimuoviTagXml();
		if (evento.equals("Seleziona paragrafo tag xml"))
			azioneSelezionaTagXml();
		if (evento.equals("Sostituisci &"))
			azioneSostituisciE();
		if (evento.equals("Sostituisci <"))
			azioneSostituisciMinore();
		if (evento.equals("Sostituisci >"))
			azioneSostituisciMaggiore();
		if (evento.equals("Sostituisci '"))
			azioneSostituisciApostrofo();
		if (evento.equals("Sostituisci \""))
			azioneSostituisciQuota();
		if (evento.equals("Ripristina caratteri speciali"))
			azioneRipristinaCaratteriSpeciali();
		if (evento.equals("Crea dtd da documento xml"))
			azioneCreaDtd();
		if (evento.equals("Crea dtd e documento xml valido"))
		     azioneCreaDocumentoXmlValido();
		if (evento.equals("Crea css associato al file xml"))
			azioneCreaCss();

		if (evento.equals("Crea nuovo file tag (.xml)"))
			oggettoXmlTag.azioneNuovoDocumentoXmlTag();
		if (evento.equals("Apre file tag (.xml)"))
			oggettoXmlTag.azioneApriDocumentoXmlTag();
		if (evento.equals("Deseleziona celle attributi e valori"))
			oggettoXmlTag.azioneCancellaSelezioneJTable();
		if (evento.equals("Salva file tag (.xml)"))
			oggettoXmlTag.azioneSalvaDocumentoXmlTag(false);
		if (evento.equals("Salva nome file tag (.xml)"))
			oggettoXmlTag.azioneSalvaDocumentoXmlTag(true);
		if (evento.equals("Aggiungi nuovo nodo tag xml"))
			oggettoXmlTag.azioneAggiungiNodoXmlTag();
		if (evento.equals("Modifica nodo tag xml"))
			oggettoXmlTag.azioneEditaNodoXmlTag();
		if (evento.equals("Elimina nodo tag xml"))
			oggettoXmlTag.azioneCancellaNodo();
		if (evento.equals("Aggiungi attributo nodo tag xml"))
			oggettoXmlTag.azioneAggiungiAttributoNodo();
		if (evento.equals("Modifica attributo nodo tag xml"))
			oggettoXmlTag.azioneModificaAttributoNodo();
		if (evento.equals("Elimina attributo nodo tag xml"))
			oggettoXmlTag.azioneEliminaAttributoNodo();

		if (evento.equals("A capo automatico"))
			azioneWordWrap();
		if (evento.equals("Tipo carattere"))
			azioneTipoCarattere();
		if (evento.equals("Colore sfondo"))
			azioneTipoColoreSfondo();
		if (evento.equals("Colore testo"))
			azioneTipoColoreTesto();
		if (evento.equals("Directory di lavoro"))
			azioneImpostaDirectory();
		if (evento.equals("File tag di default (.xml)"))
			azioneImpostaFileXmlTag();

		if (evento.equals("Dimensione finestra"))
			azioneDimensioneFinestra();
		if (evento.equals("Ripristina impostazioni predefinite"))
			ripristinaImpostazioniDefault();

		if (evento.equals("Metal"))
			azioneImpostaLookMetal();
		if (evento.equals("System"))
			azioneImpostaLookSystem();
		if (evento.equals("Windows"))
			azioneImpostaLookWindows();
		if (evento.equals("Motif"))
			azioneImpostaLookMotif();
		if (evento.equals("Mac"))
			azioneImpostaLookMac();

		if (evento.equals("Guida testuale"))
			azioneGuida();
		if (evento.equals("Informazioni sul programma"))
			azioneAboutBox();

		// gestione eventi relativi ai menu' popup
		if (evento.equals("tag testo popup"))
			azioneAggiungiTagXml();
		if (evento.equals("elimina tag testo popup"))
			azioneRimuoviTagXml();
		if (evento.equals("seleziona tag testo popup"))
			azioneSelezionaTagXml();
		if (evento.equals("taglia testo popup"))
			azioneTaglia();
		if (evento.equals("copia testo popup"))
			azioneCopia();
		if (evento.equals("incolla testo popup"))
			azioneIncolla();
	}


	// **** metodo che modifica il cursore dell'applicazione in clessidra ****

	public void cursoreClessidra()
	{
		getContentPane().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR)); // modifica il cursore in clessidra
		areaTesto.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR)); // modifica il cursore in clessidra
		splitPaneCentrale.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR)); // modifica il cursore in clessidra
		splitPaneCentrale2.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR)); // modifica il cursore in clessidra
		oggettoXmlTag.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR)); // modifica il cursore in clessidra
	}


	// **** metodo che modifica il cursore dell'applicazione in normale ****

	public void cursoreNormale()
	{
		getContentPane().setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR)); // ripristina il cursore normale
		areaTesto.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR)); // ripristina il cursore normale
		splitPaneCentrale.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR)); // modifica il cursore in normale
		splitPaneCentrale2.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR)); // modifica il cursore in normale
		oggettoXmlTag.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR)); // modifica il cursore in normale
	}


	// **** metodi legati all'esecuzione di eventi da parte dell'utente ****

	// **** azioneNuovoFile e' richiamato per creare un nuovo file vuoto ****

	private void azioneNuovoFile()
	{
		azioneRicordaSalvaFile();
		areaTesto.setVisible(true);
		areaTesto.setText("");
		setTitle("XMLStudio"); // imposta il nuovo titolo nell'editor
		fileCorrente = new File("Vuoto");
		stato.setText("Nuovo file.");
		areaTesto.cancelladatiUndoManager(); // rimuove i dati presenti nell'UndoManager
		azioneCreaAlberoXmlVuoto();  // reinizializza il viewer dell'albero Xml
		memorizzaLinea = null; // azzera il numero di linea sul quale spostarsi se richiesto
		azioneSalvaPulsante(false); // disabilita il pulsante per il salvataggio del file
	}


	// **** azioneChiudiFile e' richiamato per chiudere il file correntemente in uso ****

	private void azioneChiudiFile()
	{
		azioneRicordaSalvaFile();
		areaTesto.setText(null); // cancella il testo contenuto nell'areaTesto
		setTitle("XMLStudio"); // imposta il nuovo titolo nell'editor
		stato.setText("Pronto.");
		areaTesto.setVisible(false); // disattiva l'areaTesto (la rende 'grigia')
		areaTesto.cancelladatiUndoManager(); // rimuove i dati presenti nell'UndoManager
		fileCorrente = new File("Vuoto");
		azioneCreaAlberoXmlVuoto(); // reinizializza il viewer dell'albero Xml
		memorizzaLinea = null; // azzera il numero di linea sul quale spostarsi se richiesto
		azioneSalvaPulsante(false); // disabilita il pulsante per il salvataggio del file
	}


	// **** azioneApriFile e' richiamato per aprire il file ****

	private void azioneApriFile()
	{
	  	Thread runner = new Thread() // utilizza le thread per consentire la modifica del cursore in clessidra
	  	{
	     	public void run()
			{
				azioneRicordaSalvaFile();
				cursoreClessidra();
				areaTesto.rimuoviUndoManager(); // cessa l'esecuzione dell'UndoManager

				JFileChooser fileChooser = new JFileChooser(directoryCorrente); // crea l'oggetto per l'apertura del file sulla directory corrente

				fileChooser.setApproveButtonText("Apri"); // imposta il valore del bottone di esecuzione
				fileChooser.setDialogTitle("Apri file txt/xml"); // imposta il titolo della finestra Dialog

				// imposta ed aggiunge i filtri opportuni
				fileChooser.resetChoosableFileFilters(); // reinizializza l'oggetto fileChooser
				FiltroFile txtFiltro = new FiltroFile("txt", "File documenti *.TXT"); // imposta il filtro per i file .txt
				FiltroFile xmlFiltro = new FiltroFile("xml", "File xml *.XML"); // imposta il filtro per i file .xml
				fileChooser.addChoosableFileFilter(txtFiltro); // aggiunge il filtro alla finestra di apertura
				fileChooser.addChoosableFileFilter(xmlFiltro); // aggiunge il filtro alla finestra di apertura
				fileChooser.setFileFilter(txtFiltro); // imposta il filtro txtFiltro come filtro di default

				int ritornoValore = fileChooser.showOpenDialog(XMLStudio.this); // mostra la finestra dialog per l'apertura del file

		    		if (ritornoValore == JFileChooser.APPROVE_OPTION)
    				{
      				try
      				{
        					File f = fileChooser.getSelectedFile(); // associa all'oggetto f il file selezionato
        					if (f == null || !f.isFile()) // se f e' nullo o non e' un file allora esci dal metodo
							return;
        					fileCorrente = f; // in fileCorrente salva i dati relativi al file appena salvato per successivo uso
        					Reader in = new FileReader(f); // crea l'oggetto in per leggere il file
        					areaTesto.read(in, null); // riporta il contenuto dell'oggetto in su areaTesto
        					rimuoviCarattere(); // rimuove un carattere che non permette l'elaborazione
        					areaTesto.setVisible(true); // attiva l'areaTesto (la rende 'grigia')
        					setTitle("XMLStudio - " + f.getName()); // imposta il nuovo titolo nell'editor
        					stato.setText("File " + f.getName() + " caricato."); // indica l'avvenuto caricamento
        					areaTesto.creaUndoManager(); // reinizializza l'UndoManager
        					fileGiaSalvato = false; // il fileCorrente appena aperto non e' ancora stato salvato

						String nomefile = fileCorrente.getName();
						String estensione = nomefile.substring(nomefile.length()-4);
						if (estensione.equals(".txt") == true) // se il file e' testuale proponi la trasformazione in xml
						{
							ritornoValore = JOptionPane.showConfirmDialog(getContentPane(), "Si desidera creare il documento Xml?", "Domanda", JOptionPane.YES_NO_OPTION);
							if (ritornoValore == JOptionPane.YES_OPTION)
								azioneCreaXml();
							else
								ultimiFile.salvaStoriaFile(fileCorrente); // il file caricato e' testuale ma non si vuole convertirlo in .xml
								setJMenuBar(crea_MenuBar()); // creazione della barra dei menu
						}
						else
						{
							ultimiFile.salvaStoriaFile(fileCorrente); // il file caricato non ha estensione .txt
							setJMenuBar(crea_MenuBar()); // creazione della barra dei menu
						}
						azioneCreaAlberoXmlVuoto(); // reinizializza il viewer dell'albero Xml
						memorizzaLinea = null; // azzera il numero di linea sul quale spostarsi se richiesto
						azioneSalvaPulsante(false); // disabilita il pulsante per il salvataggio del file
      				}
      				catch(Exception e)
      				{
	      				stato.setText("Errore di apertura del file " + fileCorrente.getName() + " " + e);
	      				System.err.println("Errore di apertura del file " + fileCorrente.getName() + " " + e);
	      				cursoreNormale();
	     			}
	     			finally
	     			{
						cursoreNormale();
					}
    				}
    				cursoreNormale();
    			}
    		};
		runner.start();
		//cursoreNormale();
	}


    	// **** azioneApriUltimoFile e' richiamato per aprire il file pi in alto nella history del menu File dell'editor ****

	private void azioneApriUltimoFile(final String stringaultimofile)
	{
		Thread runner = new Thread() // utilizza le thread per consentire la modifica del cursore in clessidra
	  	{
	     	public void run()
			{
				azioneRicordaSalvaFile();
				cursoreClessidra();
				areaTesto.rimuoviUndoManager(); // cessa l'esecuzione dell'UndoManager
				try
    				{
	    				File f = new File(stringaultimofile); // associa all'oggetto f il file selezionato
	    				if (f.exists())
	    				{
    						fileCorrente = f; // in fileCorrente salva i dati relativi al file appena salvato per successivo uso
    						Reader in = new FileReader(f); // crea l'oggetto in per leggere il file
    						areaTesto.read(in, null); // riporta il contenuto dell'oggetto in su areaTesto
    						rimuoviCarattere();  // rimuove un carattere che non permette l'elaborazione
    						areaTesto.setVisible(true); // attiva l'areaTesto (la rende 'grigia')
    						setTitle("XMLStudio - " + f.getName()); // imposta il nuovo titolo nell'editor
    						stato.setText("File " + f.getName() + " caricato."); // indica l'avvenuto caricamento
    						areaTesto.creaUndoManager(); // reinizializza l'UndoManager
    						fileGiaSalvato = false; // il fileCorrente appena aperto non e' ancora stato salvato
    						ultimiFile.salvaStoriaFile(f); // aggiorna l'elenco dei file aperti di recente
    						setJMenuBar(crea_MenuBar()); // creazione della barra dei menu
    						azioneCreaAlberoXmlVuoto(); // reinizializza il viewer dell'albero Xml
    						memorizzaLinea = null; // azzera il numero di linea sul quale spostarsi se richiesto
    						azioneSalvaPulsante(false); // disabilita il pulsante per il salvataggio del file
    					}
    					else
    					{
	    					stato.setText("Probabilmente e' stato cancellato un file relativo ad ultimi file aperti.");
    					}
    				}
    				catch(Exception e)
    				{
     				stato.setText("Errore di apertura dell'ultimo file." + e);
      				System.err.println("Errore di apertura dell'ultimo file." + e);
     			}
    				finally
    				{
	    				cursoreNormale();
				}
			}
		};
		runner.start();
	}


    	// **** azioneSalvaFileNome  richiamato per salvare il file con un nome specifico ****

    	private void azioneSalvaFileNome()
    	{
	    	boolean salvareFile = false;

	    	JFileChooser fileChooser = new JFileChooser(directoryCorrente); // crea l'oggetto per l'apertura del file sulla directory corrente
		// imposta la corrente directory di lavoro, esempio /Tesi/Tesi XMLStudio oppure "."

		fileChooser.setApproveButtonText("Salva"); // imposta il valore del bottone di esecuzione
		fileChooser.setDialogTitle("Salva file xml"); // imposta il titolo della finestra Dialog

		// imposta ed aggiunge i filtri opportuni
		fileChooser.resetChoosableFileFilters(); // reinizializza l'oggetto fileChooser
		FiltroFile txtFiltro = new FiltroFile("txt", "File documenti *.TXT"); // imposta il filtro per i file .txt
		FiltroFile xmlFiltro = new FiltroFile("xml", "File xml *.XML"); // imposta il filtro per i file .xml
		fileChooser.addChoosableFileFilter(txtFiltro); // aggiunge il filtro alla finestra di apertura
		fileChooser.addChoosableFileFilter(xmlFiltro); // aggiunge il filtro alla finestra di apertura
		fileChooser.setFileFilter(xmlFiltro); // imposta il filtro xmlFiltro come filtro di default

     	int ritornoValore = fileChooser.showSaveDialog(getContentPane()); // mostra la finestra dialog per il salvataggio del file

     	if (ritornoValore == JFileChooser.APPROVE_OPTION)
     	{
   			try
    			{
	      		File f = fileChooser.getSelectedFile(); // associa all'oggetto f il file selezionato
		 		if (f.exists()) // il file selezionato esiste gia' nella directory corrente
		 		{
		 		     ritornoValore = JOptionPane.showConfirmDialog(getContentPane(), "Si desidera sovrascrivere il file " + f.getName() + "?", "Domanda", JOptionPane.YES_NO_OPTION);
					if (ritornoValore == JOptionPane.YES_OPTION)
						salvareFile = true; // e' stato selezionato di sovrascrivere il file
     				if (ritornoValore == JOptionPane.NO_OPTION)
						salvareFile = false; // e' stato selezionato di non sovrascrivere il file
				}
     			else
     			{
	     			salvareFile = true; // il file non esiste nella directory corrente e viene salvato
     			}

     			if (salvareFile == true) // gestisce il salvataggio del file
     			{
	     			fileCorrente = f; // in fileCorrente salva i dati relativi al fine appena salvato per successivo uso
	     			fileGiaSalvato = true; // il fileCorrente appena aperto e' stato salvato
     				Writer out = new FileWriter(f); // crea l'oggetto out per salvare il file
     				areaTesto.write(out); // riporta il contenuto di areaTesto sull'oggetto out
     				setTitle("XMLStudio - " + f.getName()); // imposta il nuovo titolo nell'editor
     				stato.setText("File " + f.getName() + " salvato."); // indica l'avvenuto caricamento
     				ultimiFile.salvaStoriaFile(fileCorrente); // il file caricato non ha estensione .txt
					setJMenuBar(crea_MenuBar()); // creazione della barra dei menu
					azioneSalvaPulsante(false); // disabilita il pulsante per il salvataggio del file
     			}
     			else // non salva il file
     			{
	     			stato.setText("File " + f.getName() + " non salvato."); // indica l'avvenuto caricamento
     			}
      		}
      		catch(Exception e)
      		{
		     	stato.setText("Errore di salvataggio del file " + fileCorrente.getName() + ".");
	      		System.err.println("Errore di salvataggio del file " + fileCorrente.getName() + ".");
	      	}
  		}
  	}


  	// **** metodo che abilita e disabilita l'icona per il salvataggio dei file ****

  	private void azioneSalvaPulsante(boolean abilitato)
  	{
	  	jmi_salva.setEnabled(abilitato);
	  	salva.setEnabled(abilitato);
	}


  	// **** azioneSalvaFile  richiamato per salvare il file ****

  	private void azioneSalvaFile()
  	{
	  	if (fileGiaSalvato == false) // il fileCorrente appena aperto non e' ancora stato salvato
	  	{
		  	azioneSalvaFileNome(); // al file non e' ancora stato assegnato un nome
	 	}
	 	else
	 	{
			if (fileCorrente.isFile() == true)
			{
	  			try
	  			{
		     		Writer out = new FileWriter(fileCorrente); // crea l'oggetto out per salvare il file
     				areaTesto.write(out); // riporta il contenuto di areaTesto sull'oggetto out
     				setTitle("XMLStudio - " + fileCorrente.getName()); // imposta il nuovo titolo nell'editor
     				stato.setText("File " + fileCorrente.getName() + " salvato."); // indica l'avvenuto caricamento
     				azioneSalvaPulsante(false); // disabilita il pulsante per il salvataggio del file
      			}
      			catch(Exception e)
      			{
			     	stato.setText("Errore di salvataggio del file " + fileCorrente.getName() + ".");
      				System.err.println("Errore di salvataggio del file " + fileCorrente.getName() + ".");
	     		}
     		}
     		else
     		{
		     	azioneSalvaFileNome(); // al file non e' ancora stato assegnato un nome
     		}
     	}
	}


	// **** chiede all'utente se desidera salvare il file corrente aperto e visualizzato nella JTextArea ****

	private void azioneRicordaSalvaFile()
	{
		if ((areaTesto.isVisible() == true) && (!fileCorrente.getName().equals("Vuoto")))
		{
			int ritornoValore = JOptionPane.showConfirmDialog(getContentPane(), "Si desidera salvare il file corrente " + fileCorrente.getName() + "?", "Domanda", JOptionPane.YES_NO_OPTION);
			if (ritornoValore == JOptionPane.YES_OPTION)
				azioneSalvaFile(); // salva il file
		}
	}


	// **** azioneStampaWindows consente di stampare il documento visibile nella JTextArea in ambiente Windows ****

	private void azioneStampaWindows()
	{
		// getContentPane().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR)); // dovrebbe cambiare il cursorse in clessidra
		if (!areaTesto.getText().equals("")) // se vi e' del testo in areaTesto
		{
			try
			{
				stato.setText("Stampa del documento in ambiente Windows.");
				PrintWriter out = new PrintWriter(new BufferedWriter(new FileWriter("lpt1"))); //lpt1 in Windows consente di inviare il testo alla stampante
				out.print(areaTesto.getText());
				out.flush();
				out.close();
			}
			catch ( IOException e)
			{
				stato.setText("Errore di stampa su carta - Stampa Windows.");
	      		System.err.println("Errore di stampa su carta - Stampa Windows.");
			}
		}
		else
		{
			JOptionPane.showInternalMessageDialog(getContentPane(), "Non vi e' testo da stampare.", "Informazione", JOptionPane.INFORMATION_MESSAGE);
		}
		// getContentPane().setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR)); // dovrebbe cambiare il cursorse in clessidra
	}


	// **** azioneStampaStringa consente di stampare il documento visibile nella JTextArea in ambiente Unix ****

	private void azioneStampaStringa()
	{
		// getContentPane().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR)); // dovrebbe cambiare il cursorse in clessidra
		if (!areaTesto.getText().equals("")) // se vi e' del testo in areaTesto
		{
			try
			{
				stato.setText("Stampa del documento passando la stringa appropriata.");
				CreaDialog cd = new CreaDialog(this, "Inserire la stringa per la stampa", true, "Stampa (Es. lpt1, lp, /dev/lp)", "Ok"); // crea una finestra Dialog
				parola = cd.getString(); // in parola viene inserito la stringa digitata
				if (parola != null)// se la stringa non e' nulla
				{
					PrintWriter out = new PrintWriter(new BufferedWriter(new FileWriter(parola))); //lpt1 in Windows consente di inviare il testo alla stampante
					out.print(areaTesto.getText());
					out.flush();
					out.close();
				}
			}
			catch ( IOException e)
			{
				stato.setText("Errore di stampa su carta - StampaStringa.");
	      		System.err.println("Errore di stampa su carta - StampaStringa.");
			}
		}
		else
		{
			JOptionPane.showInternalMessageDialog(getContentPane(), "Non vi e' testo da stampare.", "Informazione", JOptionPane.INFORMATION_MESSAGE);
		}
		// getContentPane().setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR)); // dovrebbe cambiare il cursorse in clessidra
	}


	// **** azioneStampaGrafica invia alla stampante il documento contenuto nella JTextArea ****

	private void azioneStampaGrafica()
	{
		if (!areaTesto.getText().equals("")) // se vi e' del testo in areaTesto
		{
			stato.setText("Stampa del documento in modalita' grafica.");
			PrintJob jobStampa = getToolkit().getPrintJob(this, "Stampa documento", null); // attiva un PrintJob per la corrente istanza di java.awt.Toolkit
      		if (jobStampa != null)
      		{
	        		Graphics oggettoGrafico = jobStampa.getGraphics(); // richiede al PrintJob un oggetto grafico per stampare la pagina
        			if (oggettoGrafico != null)
        			{
	          		String testo = areaTesto.getText(); // in testo viene inserito il testo della JTextArea
          			StampaGrafica oggettoStampaGrafica = new StampaGrafica(jobStampa, oggettoGrafico, testo, fileCorrente.getName());
	          		oggettoGrafico.dispose(); // rilascio di tutte le risorse che usavano questo contesto grafico
        			}
        			jobStampa.end(); // termina il PrintJob
			}
		}
		else
		{
			JOptionPane.showInternalMessageDialog(getContentPane(), "Non vi e' testo da stampare.", "Informazione", JOptionPane.INFORMATION_MESSAGE);
		}
	}


	// **** azioneUscita  richiamato quando si esce dal programma ****

	private void azioneUscita()
	{
		// avvisa l'utente se si desidera salvare il file corrente aperto nell'editor
		azioneRicordaSalvaFile();

		// avvisa l'utente se si desidera salvare il file corrente relativo ai tag xml
		oggettoXmlTag.azioneRicordaSalva();

		System.out.println("Grazie per aver usato XMLStudio.");
		System.exit(0);
	}


	// **** azioneAnnulla, azioneRipristina realizzano le specifiche funzionalit di undo e redo dell'editor ****

	private void azioneAnnulla()
	{
		areaTesto.annulla();
	}


	// **** azioneAnnulla, azioneRipristina realizzano le specifiche funzionalit di undo e redo dell'editor ****

	private void azioneRipristina()
	{
		areaTesto.ripristina();
	}


	// **** azioneTaglia, azioneCopia, azioneIncolla realizzano le specifiche funzionalit nell'editor ****

	private void azioneTaglia()
	{
		areaTesto.cut();
	}


	// **** copia in memoria una porzione di testo selezionata nell'areaTesto ****

	private void azioneCopia()
	{
		areaTesto.copy();
	}


	// **** incolla una porzione di testo dalla memoria ****

	private void azioneIncolla()
	{
		areaTesto.paste();
	}


	// **** seleziona tutto il testo incluso in areaTesto ****

	private void azioneSelezionaTutto()
	{
		areaTesto.selectAll();
		areaTesto.requestFocus();
		stato.setText("Testo interamente selezionato.");
	}


	// **** azioneTrovaStringa individua del testo (in genere un carattere &, >, <, ', " all'interno del documento xml ****

	private boolean azioneTrovaStringa(String parola) // gli passa un carattere che rende il documento invalido e non ben formato
	{
		int posizione, // posizione della parola individuata
		    posizioneiniziale, // posizione della fine della stringa <documento> dalla quale bisogna iniziare a cercare
		    posizionefinale; // posizione dell'inizio della stringa </documento> prima della quale bisogna cercare

		if (parola != null) // se la stringa non e' nulla
		{
			String testoCompleto = areaTesto.getText().toLowerCase(); // in testoCompleto inserisce il testo di areaTesto
			posizioneiniziale = testoCompleto.indexOf("<documento>") + 5; // ricerca in testoCompleto la stringa parola a partire da posizioneIniz
			posizionefinale = testoCompleto.indexOf("</documento>");

			posizione = testoCompleto.indexOf(parola, posizioneiniziale + 1); // ricerca in testoCompleto la stringa parola a partire da salvaPosizione + 1

			if ((posizione < posizionefinale -1) && (posizione != -1))// || (posizione == posizioneiniziale) || (posizione == posizionefinale))// restituisce -1 se non individua la stringa parola in testoCompleto
			{
				return true; // non ha trovato un carattere &, >, <, ', " o altra stringa nel documento xml
			}
			else
			{
				return false; // ha trovato un carattere &, >, <, ', " o altra stringa nel documento xml
			}
		}
		return false;
	}


	// **** azioneTrovaCaratteriNonValidi individua nel testo xml dei caratteri che non validi ****

	private String azioneTrovaCaratteriNonValidi()
	{
		String stringacompleta = null, // in stringacompleta verra' incluso l'eventuale sequenza di caratteri che rendono non valido e non ben formato il documento
			  stringacaratterinonvalidi = "";

		if (azioneTrovaStringa("&"))
			stringacaratterinonvalidi = "& ";
		if (azioneTrovaStringa("<"))
			stringacaratterinonvalidi = stringacaratterinonvalidi + "< ";
		if (azioneTrovaStringa(">"))
			stringacaratterinonvalidi = stringacaratterinonvalidi + "> ";
		if (azioneTrovaStringa("'"))
			stringacaratterinonvalidi = stringacaratterinonvalidi + "' ";
		if (azioneTrovaStringa("\""))
			stringacaratterinonvalidi = stringacaratterinonvalidi + "\" ";
		if (stringacaratterinonvalidi.equals(null) == false)
			stringacompleta = "I caratteri non validi sono: " + stringacaratterinonvalidi;

		return stringacompleta;
	}


	// **** azioneTrova individua del testo all'interno dell'editor ****

	private void azioneTrova()
	{
		CreaDialog cd = new CreaDialog(this, "Trova", true, "Trova", "Ok"); // crea una finestra Dialog
		parola = cd.getString(); // in parola viene inserito la stringa digitata

		if (parola != null)// se la stringa non e' nulla
		{
			parola = cd.getString().toLowerCase(); // in parola viene inserito la stringa digitata
			String testoCompleto = areaTesto.getText().toLowerCase(); // in testoCompleto inserisce il testo di areaTesto

			int posizione = testoCompleto.indexOf(parola); // restituisce la posizione in cui trova in testoCompleto la prima occorrenza di parola

			if (posizione == -1) // restituisce -1 se non individua la stringa parola in testoCompleto
			{
				Toolkit tk = Toolkit.getDefaultToolkit();
				tk.beep(); // emette un suono beep
				stato.setText("Vocabolo " + parola + " non trovato: superata la fine del file.");
			}
			else
			{
				areaTesto.select(posizione, posizione + parola.length()); // seleziona il testo individuato tra le specifiche locazioni
				areaTesto.requestFocus();
				stato.setText("Parola " + parola + " individuata.");
			}
		}
	}


	// **** azioneTrova individua un tag di testo all'interno dell'editor ****

	private void azioneTrovaTag()
	{
		CreaDialog cd = new CreaDialog(this, "Trova tag", true, "Trova tag", "Ok"); // crea una finestra Dialog
		parola = cd.getString(); // in parola viene inserito la stringa digitata

		if (parola != null)// se la stringa non e' nulla
		{
			parola = "<" + cd.getString().toLowerCase(); // in parola viene inserito la stringa digitata
			String testoCompleto = areaTesto.getText().toLowerCase(); // in testoCompleto inserisce il testo di areaTesto

			int posizione = testoCompleto.indexOf(parola); // restituisce la posizione in cui trova in testoCompleto la prima occorrenza di parola

			if (posizione == -1) // restituisce -1 se non individua la stringa parola in testoCompleto
			{
				Toolkit tk = Toolkit.getDefaultToolkit();
				tk.beep(); // emette un suono beep
				stato.setText("Vocabolo " + parola + " non trovato: superata la fine del file.");
			}
			else
			{
				areaTesto.select(posizione, posizione + parola.length()); // seleziona il testo individuato tra le specifiche locazioni
				areaTesto.requestFocus();
				stato.setText("Parola " + parola + " individuata.");
			}
		}
	}


	// **** azioneTrovaSuccessivo ripete la ricerca della parola introdotta in Trova ****

	private void azioneTrovaSuccessivo()
	{
		if (parola == null)
		{
			azioneTrova();
			return;
		}

		String testoCompleto = areaTesto.getText().toLowerCase(); // in testoCompleto inserisce il testo di areaTesto
		int posizioneInizio = areaTesto.getSelectionEnd(); // in posizioneInizio inserisce il valore finale evidenziato dalla precedente ricerca
		int posizione = testoCompleto.indexOf(parola, posizioneInizio); // ricerca in testoCompleto la stringa parola a partire da posizioneInizio

		if (posizione == -1) // restituisce -1 se non individua la stringa parola in testoCompleto
		{
			Toolkit tk = Toolkit.getDefaultToolkit();
			tk.beep(); // emette un suono beep
			stato.setText("Vocabolo " + parola + " non trovato: superata la fine del file.");
		}
		else
		{
			areaTesto.select(posizione, posizione + parola.length()); // seleziona il testo individuato tra le specifiche locazioni
			areaTesto.requestFocus();
			stato.setText("Parola " + parola + " individuata.");
		}
	}


	// **** azioneSostituisciE trova e sostituisce le occorrenze di & nel documento xml per renderlo ben formato ****

	private void azioneSostituisciE()
	{
		Object[] options = {"Manuale", "Automatica"};
		int ritornoValore =  JOptionPane.showOptionDialog(getContentPane(), "Scegliere un'opzione, sostituzione di tipo:", "Informazione", JOptionPane.DEFAULT_OPTION, JOptionPane.INFORMATION_MESSAGE, null, options, options[0]);
		if (ritornoValore == JOptionPane.YES_OPTION)
		{
			azioneSostituisciManuale("&", "&amp;"); // sostituzione manuale
		}
		else
		{
			azioneSostituisciAutomatico("&", "&amp;"); // sostituzione automatica
		}
	}


	// **** azioneSostituisciMinore trova e sostituisce le occorrenze di < nel documento xml per renderlo ben formato ****

	private void azioneSostituisciMinore()
	{
		Object[] options = {"Manuale", "Automatica"};
		int ritornoValore =  JOptionPane.showOptionDialog(getContentPane(), "Scegliere un'opzione, sostituzione di tipo:", "Informazione", JOptionPane.DEFAULT_OPTION, JOptionPane.INFORMATION_MESSAGE, null, options, options[0]);
		if (ritornoValore == JOptionPane.YES_OPTION)
		{
			azioneSostituisciManuale("<", "&lt;"); // sostituzione manuale
		}
		else
		{
			azioneSostituisciAutomatico("<", "&lt;"); // sostituzione automatica
		}
	}


	// **** azioneSostituisciMaggiore trova e sostituisce le occorrenze di > nel documento xml per renderlo ben formato ****

	private void azioneSostituisciMaggiore()
	{
		Object[] options = {"Manuale", "Automatica"};
		int ritornoValore =  JOptionPane.showOptionDialog(getContentPane(), "Scegliere un'opzione, sostituzione di tipo:", "Informazione", JOptionPane.DEFAULT_OPTION, JOptionPane.INFORMATION_MESSAGE, null, options, options[0]);
		if (ritornoValore == JOptionPane.YES_OPTION)
		{
			azioneSostituisciManuale(">", "&gt;"); // sostituzione manuale
		}
		else
		{
			azioneSostituisciAutomatico(">", "&gt;"); // sostituzione automatica
		}
	}


	// **** azioneSostituisciApostrofo trova e sostituisce le occorrenze di ' nel documento xml per renderlo ben formato ****

	private void azioneSostituisciApostrofo()
	{
		Object[] options = {"Manuale", "Automatica"};
		int ritornoValore =  JOptionPane.showOptionDialog(getContentPane(), "Scegliere un'opzione, sostituzione di tipo:", "Informazione", JOptionPane.DEFAULT_OPTION, JOptionPane.INFORMATION_MESSAGE, null, options, options[0]);
		if (ritornoValore == JOptionPane.YES_OPTION)
		{
			azioneSostituisciManuale("'", "&apos;"); // sostituzione manuale
		}
		else
		{
			azioneSostituisciAutomatico("'", "&apos;"); // sostituzione automatica
		}
	}


	// **** azioneSostituisciQuota trova e sostituisce le occorrenze di " nel documento xml per renderlo ben formato ****

	private void azioneSostituisciQuota()
	{
		Object[] options = {"Manuale", "Automatica"};
		int ritornoValore =  JOptionPane.showOptionDialog(getContentPane(), "Scegliere un'opzione, sostituzione di tipo:", "Informazione", JOptionPane.DEFAULT_OPTION, JOptionPane.INFORMATION_MESSAGE, null, options, options[0]);
		if (ritornoValore == JOptionPane.YES_OPTION)
		{
			azioneSostituisciManuale("\"", "&quot;"); // sostituzione manuale
		}
		else
		{
			azioneSostituisciAutomatico("\"", "&quot;"); // sostituzione automatica
		}
	}


	// **** i caratteri speciali &, <, >, ', " vengono ripristinati (dalla sequenza di escape) ****

	private void azioneRipristinaCaratteriSpeciali()
	{
		try
		{
			azioneSostituisciAutomatico("&amp;", "&"); // sostituzione automatica
			azioneSostituisciAutomatico("&lt;", "<"); // sostituzione automatica
			azioneSostituisciAutomatico("&gt;", ">"); // sostituzione automatica
			azioneSostituisciAutomatico("&apos;", "'"); // sostituzione automatica
			azioneSostituisciAutomatico("&quot;", "\""); // sostituzione automatica
			stato.setText("Caratteri speciali ripristinati: documento xml non piu' ben formato.");
		}
		catch (Exception e)
		{
			stato.setText("Eccezione rilevata nel ripristino caratteri speciali");
		}
	}


	// **** azioneSostituisciAutomatico trova e sostituisce del testo all'interno dell'editor ****

	private void azioneSostituisciAutomatico(String parola, String parolaSostituisci)
	{
		String testoCompleto;
		int posizione, salvaposizione, posizionefinale = -1;
		boolean flagxml;

		cursoreClessidra();
		String nomefile = fileCorrente.getName();
		String estensione = nomefile.substring(nomefile.length()-4);
		if (estensione.equals(".xml") == true) // l'estensione del file corrente e' .txt
		{
			flagxml = true;
		}
		else
		{
			flagxml = false;
		}

		if ((parola.equals("<") == true) || (parola.equals(">") == true)) // caso speciale a causa dei tag <documento> e </documento>
		{
			testoCompleto = areaTesto.getText().toLowerCase(); // in testoCompleto inserisce il testo di areaTesto
			if (parola.equals("<"))
				posizione = testoCompleto.indexOf("<documento>"); // ricerca in testoCompleto la stringa parola a partire da posizioneIniz
			else
				posizione = testoCompleto.indexOf("<documento>") + 11; // ricerca in testoCompleto la stringa parola a partire da posizioneIniz
			salvaposizione = posizione;
			posizione = testoCompleto.indexOf(parola, salvaposizione + 1); // ricerca in testoCompleto la stringa parola a partire da salvaPosizione + 1
			salvaposizione = posizione;
		}
		else
		{
			testoCompleto = areaTesto.getText().toLowerCase(); // in testoCompleto inserisce il testo di areaTesto
			posizione = testoCompleto.indexOf(parola); // ricerca in testoCompleto la stringa parola a partire da posizioneInizio
			salvaposizione = posizione;
		}

		if ((parola != null) && (parolaSostituisci != null)) // se le stringhe non sono nulle
		{
			do
			{
				if (posizione == -1) // restituisce -1 se non individua la stringa parola in testoCompleto
				{
					Toolkit tk = Toolkit.getDefaultToolkit();
					tk.beep(); // emette un suono beep
					stato.setText("Vocabolo " + parola + " non trovato: superata la fine del file.");
				}
				else
				{
					areaTesto.select(posizione, posizione + parola.length()); // seleziona il testo individuato tra le specifiche locazioni
					areaTesto.requestFocus();

					//areaTesto.replaceRange(parolaSostituisci, posizione, posizione + parola.length()); // seleziona il testo individuato tra le specifiche locazioni
					areaTesto.replaceSelection(parolaSostituisci);
					areaTesto.requestFocus();
					stato.setText("Parola " + parola + " sostituita con " + parolaSostituisci + ".");
					//posizione = testoCompleto.indexOf(parola); // ricerca in testoCompleto la stringa parola a partire da posizioneInizio
					testoCompleto = areaTesto.getText().toLowerCase(); // in testoCompleto inserisce il testo di areaTesto
					posizione = testoCompleto.indexOf(parola, salvaposizione + 1); // ricerca in testoCompleto la stringa parola a partire da posizioneIniz
					salvaposizione = posizione;

					if (flagxml) // se il file e' xml deve terminare cercando la stringa </documento> altrimenti deve terminare con la fine file
					{
						posizionefinale = testoCompleto.indexOf("</documento>"); // ricerca in testoCompleto la stringa parola
					}
					else
					{
						posizionefinale = testoCompleto.length();
					}
				}
			} while ((posizione != -1) && (posizione < posizionefinale));
		}
		cursoreNormale();
	}


	// **** azioneSostituisciManuale trova e sostituisce una per una le parole nel testo ****

	private void azioneSostituisciManuale(String parola, String parolaSostituisci)
	{
		boolean esci = false;

		String testoCompleto;
		int posizione, salvaposizione, posizionefinale = -1 ;
		boolean flagxml;

		String nomefile = fileCorrente.getName();
		String estensione = nomefile.substring(nomefile.length()-4);
		if (estensione.equals(".xml") == true) // l'estensione del file corrente e' .txt
		{
			flagxml = true;
		}
		else
		{
			flagxml = false;
		}

		if ((parola.equals("<") == true) || (parola.equals(">") == true)) // caso speciale a causa dei tag <documento> e </documento>
		{
			testoCompleto = areaTesto.getText().toLowerCase(); // in testoCompleto inserisce il testo di areaTesto
			if (parola.equals("<"))
				posizione = testoCompleto.indexOf("<documento>"); // ricerca in testoCompleto la stringa parola a partire da posizioneIniz
			else
				posizione = testoCompleto.indexOf("<documento>") + 11; // ricerca in testoCompleto la stringa parola a partire da posizioneIniz
			salvaposizione = posizione;
			posizione = testoCompleto.indexOf(parola, salvaposizione + 1); // ricerca in testoCompleto la stringa parola a partire da salvaPosizione + 1
			salvaposizione = posizione;
		}
		else
		{
			testoCompleto = areaTesto.getText().toLowerCase(); // in testoCompleto inserisce il testo di areaTesto
			posizione = testoCompleto.indexOf(parola); // ricerca in testoCompleto la stringa parola a partire da posizioneInizio
			salvaposizione = posizione;
		}

		if ((parola != null) && (parolaSostituisci != null)) // se le stringhe non sono nulle
		{
			do
			{
				if (posizione == -1) // restituisce -1 se non individua la stringa parola in testoCompleto
				{
					Toolkit tk = Toolkit.getDefaultToolkit();
					tk.beep(); // emette un suono beep
					stato.setText("Vocabolo " + parola + " non trovato: superata la fine del file.");
				}
				else
				{
					areaTesto.select(posizione, posizione + parola.length()); // seleziona il testo individuato tra le specifiche locazioni
					areaTesto.requestFocus();

					int ritornoValore = JOptionPane.showConfirmDialog(getContentPane(), "Si desidera sostituire " + parola + " con " + parolaSostituisci + "?", "Domanda", JOptionPane.YES_NO_CANCEL_OPTION, JOptionPane.INFORMATION_MESSAGE);
					if (ritornoValore == JOptionPane.YES_OPTION)
					{
						//areaTesto.replaceRange(parolaSostituisci, posizione, posizione + parola.length()); // seleziona il testo individuato tra le specifiche locazioni
						areaTesto.replaceSelection(parolaSostituisci);
						areaTesto.requestFocus();
						stato.setText("Parola " + parola + " sostituita con " + parolaSostituisci + ".");
						//posizione = testoCompleto.indexOf(parola); // ricerca in testoCompleto la stringa parola a partire da posizioneInizio
						testoCompleto = areaTesto.getText().toLowerCase(); // in testoCompleto inserisce il testo di areaTesto
						posizione = testoCompleto.indexOf(parola, salvaposizione + 1); // ricerca in testoCompleto la stringa parola a partire da posizioneIniz
						salvaposizione = posizione;
					}
					if (ritornoValore == JOptionPane.NO_OPTION)
					{
						stato.setText("Occorrenza non sostituita.");
						posizione = testoCompleto.indexOf(parola, areaTesto.getSelectionEnd()); // ricerca in testoCompleto la stringa parola a partire da posizioneIniz
						salvaposizione = posizione;
					}
					if (ritornoValore == JOptionPane.CANCEL_OPTION)
						esci = true;
					if (flagxml) // se il file e' xml deve terminare cercando la stringa </documento> altrimenti deve terminare con la fine file
						posizionefinale = testoCompleto.indexOf("</documento>"); // ricerca in testoCompleto la stringa parola
					else
						posizionefinale = testoCompleto.length();
				}
			} while ((posizione != -1) && (esci == false) && (posizione < posizionefinale));
		}
	}


	// **** azioneSostituisci trova e sostituisce del testo all'interno dell'editor ****

	private void azioneSostituisci()
	{
		CreaDialog cd = new CreaDialog(this, "Sostituisci", true, "Trova       ", "Sostituisci", "Ok", "Annulla"); // crea una finestra Dialog
		parola = cd.getString();
		String parolaSostituisci = cd.getString2();

		if ((parola != null) && (parolaSostituisci != null)) // se la stringa non e' nulla
		{
			parola = cd.getString().toLowerCase(); // in parola viene inserito la stringa digitata da cercare
			parolaSostituisci = cd.getString2().toLowerCase(); // in parolaSostituisci viene inserita la stringa digitata da sostituire

	 		Object[] options = {"Manuale", "Automatica"};
			int ritornoValore =  JOptionPane.showOptionDialog(getContentPane(), "Scegliere un'opzione, sostituzione di tipo:", "Informazione", JOptionPane.DEFAULT_OPTION, JOptionPane.INFORMATION_MESSAGE, null, options, options[0]);
			if (ritornoValore == JOptionPane.YES_OPTION)
			{
				azioneSostituisciManuale(parola, parolaSostituisci); // sostituzione manuale delle parole
			}
			else
			{
				azioneSostituisciAutomatico(parola, parolaSostituisci); // sostituzione automatica delle parole
			}
		}
	}


	// **** posiziona il cursore ad un numero di riga del documento identificato da linea ****

	private void vaiALinea(String linea)
	{
		if(linea != null) // se la stringa non e' nulla
		{
			try
			{
				int valoreLinea = Integer.parseInt(linea); // conversione di linea in valore intero
				if(valoreLinea >= 1)
				{
					areaTesto.select(areaTesto.getLineStartOffset(valoreLinea - 1), areaTesto.getLineEndOffset(valoreLinea - 1) - 1);
					areaTesto.requestFocus(); // posiziona il focus su questo oggetto
					stato.setText("Cursore spostato alla linea " + linea + ".");
				}
				else
					stato.setText(linea + " non e' un numero di linea valido.");
			}
			catch(Exception e)
			{
				stato.setText(linea + " non e' un numero di linea valido.");
			}

		}
	}


	// **** azioneVaiALinea sposta il cursore alla linea specificata ****

	private void azioneVaiALinea()
	{
		CreaDialog cd = new CreaDialog(this, "Vai alla linea numero", true, "1 - " + areaTesto.getLineCount(), "Vai"); // crea una finestra Dialog
		String linea = cd.getString(); // in linea viene inserito la stringa linea
		if(linea != null) // se la stringa non e' nulla
		{
			vaiALinea(linea);
		}
		cd.dispose(); // chiude la finestra Dialog
	}


	// **** azioneMemorizzaLinea salva in memorizzaLinea il numero di linea del cursore nel documento attivo ****

	private void azioneMemorizzaLinea()
	{
		memorizzaLinea = String.valueOf(riga); // memorizza il valore di riga relativo alla posizione del cursore
		if(memorizzaLinea != null) // se la stringa non e' nulla
		{
			stato.setText("Memorizzato numero linea " + memorizzaLinea + ".");
		}
	}


	// **** azioneVaiAMemorizzaLinea sposta il cursore al numero di linea salvato in memorizzaLinea ****

	private void azioneVaiAMemorizzaLinea()
	{
		if(memorizzaLinea != null) // se la stringa non e' nulla
		{
			vaiALinea(memorizzaLinea);
		}
		else
		{
			JOptionPane.showInternalMessageDialog(getContentPane(), "E' prima necessario memorizzare un numero di linea al quale spostarsi.", "Informazione", JOptionPane.INFORMATION_MESSAGE);
		}
	}


	// **** metodo usato da azioneCreaXml per aggiungere le intestazioni xml al file testuale aperto ****

	private void aggiungiIntestazioniXml()
	{
		areaTesto.insert("<?xml version=\"1.0\" encoding=\"iso-8859-1\" standalone=\"yes\"?>\n<documento>\n", 0);
		areaTesto.append("</documento>");
		stato.setText("File Xml creato, salvare il file per proseguire.");
		azioneSalvaFileNome(); // salva il file corrente assegnandogli un nome
		ultimiFile.salvaStoriaFile(fileCorrente); // aggiorna l'elenco dei file aperti di recente
		setJMenuBar(crea_MenuBar()); // creazione della barra dei menu
	}


	// **** crea il file xml aggiungendo le intestazioni opportune ****

	private void azioneCreaXml()
	{
		if (fileCorrente.exists())
		{
			// se il file caricato e' .txt, aggiungi i tag xml all'inizio e alla fine del documento
			String nomefile = fileCorrente.getName();
			String estensione = nomefile.substring(nomefile.length()-4);
			if (estensione.equals(".txt") == true) // l'estensione del file corrente e' .txt
			{
				aggiungiIntestazioniXml();
			}
			if (estensione.equals(".xml") == true) // l'estensione del file corrente e' .xml
			{
				//JOptionPane.showInternalMessageDialog(getContentPane(), "Il file aperto e' gia' un file Xml.", "Informazione", JOptionPane.INFORMATION_MESSAGE);
				int ritornoValore = JOptionPane.showConfirmDialog(getContentPane(), "Il file aperto e' gia' xml. Si desidera comunque aggiungere le intestazioni?", "Domanda", JOptionPane.YES_NO_OPTION);
				if (ritornoValore == JOptionPane.YES_OPTION)
				{
					aggiungiIntestazioniXml();
     			}
			}
		}
		else
		{
			// JOptionPane.showInternalMessageDialog(getContentPane(), "E' prima necessario aprire un documento testuale.", "Informazione", JOptionPane.INFORMATION_MESSAGE);
			int ritornoValore = JOptionPane.showConfirmDialog(getContentPane(), "E' prima necessario aprire un documento testuale. Si desidera comunque aggiungere le intestazioni?", "Domanda", JOptionPane.YES_NO_OPTION);
			if (ritornoValore == JOptionPane.YES_OPTION)
			{
				aggiungiIntestazioniXml();
     		}
		}
	}


  	// **** azioneImpostaDirectory  richiamato per impostare la directory corrente di lavoro ****

  	private void azioneImpostaDirectory()
  	{
	  	int ritornoValore = JOptionPane.showConfirmDialog(getContentPane(), "Si desidera impostare la directory del programma anche come directory dei documenti?", "Domanda", JOptionPane.YES_NO_OPTION);
		if (ritornoValore == JOptionPane.YES_OPTION)
		{
			directoryCorrente = ".";
     	}
     	else
		{
			directoryCorrente = JOptionPane.showInputDialog("Inserire un nuovo path di directory dei documenti");
		  	if (directoryCorrente == null) directoryCorrente = ".";
     	}
     	salva_ImpostazioniDefault();
     	stato.setText("Impostazione del nuovo path di lavoro: " + directoryCorrente);
	}


	// **** azioneImpostaFileXmlTag  richiamato per impostare il file xml che contiene i tag da aggiungere al documento ****

  	private void azioneImpostaFileXmlTag()
  	{
	  	int ritornoValore = JOptionPane.showConfirmDialog(getContentPane(), "Si desidera impostare XMLStudio.xml come file di default per i tag?", "Domanda", JOptionPane.YES_NO_OPTION);
		if (ritornoValore == JOptionPane.YES_OPTION)
		{
			fileXmlTagCorrente = "./Tag/XMLStudio.xml";
     	}
     	else
		{
			fileXmlTagCorrente = JOptionPane.showInputDialog("Inserire un nuovo file (xml) di default per i tag");
		  	if (fileXmlTagCorrente == null)
		  	{
		  		fileXmlTagCorrente = "./Tag/XMLStudio.xml";
	  		}
	  		else
	  		{
				try
				{
			  		File f = new File(fileXmlTagCorrente); // associa all'oggetto f il file selezionato
					if (f.exists())
					{
			  			String nomefile = f.getName();
						String estensione = nomefile.substring(nomefile.length()-4);
						if (estensione.equals(".xml") == false) // se il file non e' xml segnala che deve esserlo
						{
							JOptionPane.showInternalMessageDialog(getContentPane(), "Il file di default per i tag deve essere xml.", "Informazione", JOptionPane.INFORMATION_MESSAGE);
							fileXmlTagCorrente = "./Tag/XMLStudio.xml";
						}
					}
					else
					{
						JOptionPane.showInternalMessageDialog(getContentPane(), "Attenzione, il file digitato e' inesistente.", "Informazione", JOptionPane.INFORMATION_MESSAGE);
						fileXmlTagCorrente = "./Tag/XMLStudio.xml";
					}
				}
				catch (Exception e)
				{
					//JOptionPane.showInternalMessageDialog(getContentPane(), "Il file di default per i tag deve essere xml. Rilevata eccezione: " + e, "Attenzione", JOptionPane.ERROR_MESSAGE);
					JOptionPaneEsteso mostraEccezione = new JOptionPaneEsteso(this, "Informazione", "Eccezione rilevata: " + e + ". Il file di default per i tag deve essere xml.");
				}
			}
     	}
     	salva_ImpostazioniDefault();
     	stato.setText("Impostazione del file di default per tag: " + fileXmlTagCorrente);
	}


	// **** azioneDimensioneFinestra e' richiamato per memorizzare le nuove dimensioni della finestra dell'editor ****

	private void azioneDimensioneFinestra()
	{
		CreaDialog cd = new CreaDialog(this, "Imposta la dimensione finestra", true, "X (800)", "Y (600)", "Pannello orizzontale (350)", "Pannello verticale (630)", "Ok", "Annulla"); // crea una finestra Dialog

		finestraX = cd.getString();
		finestraY = cd.getString2();
		divisoreOrizzontale = cd.getString3();
		divisoreVerticale = cd.getString4();

		if ((finestraX != null) && (finestraY != null) && (divisoreOrizzontale != null) && (divisoreVerticale != null)) // se le stringhe  non sono nulle
		{
			salva_ImpostazioniDefault();
			stato.setText("Nuove dimensioni finestra selezionate: " + finestraX + " " + finestraY + " " + divisoreOrizzontale + " " + divisoreVerticale);

			azioneRiavviaEditor();
		}
	}


	// **** azioneTipoColoreSfondo e' richiamato per modificare il colore dello sfondo dell'editor ****

	private void azioneTipoColoreSfondo()
	{
		coloreSfondoDefault = JColorChooser.showDialog(this, "Colore sfondo", Color.white);
		impostaProprietaAreaTesto(areaTesto);
		salva_ImpostazioniDefault();
		stato.setText("Impostazione del nuovo colore dello sfondo.");
	}


	// **** azioneTipoColoreTesto e' richiamato per modificare il colore dello sfondo dell'editor ****

	private void azioneTipoColoreTesto()
	{
		coloreTestoDefault = JColorChooser.showDialog(this, "Colore sfondo", Color.black);
		impostaProprietaAreaTesto(areaTesto);
		salva_ImpostazioniDefault();
		stato.setText("Impostazione del nuovo colore del testo.");
	}


	// **** azioneTipoCarattere e' richiamato per cambiare il tipo di carattere nell'editor ****

	private void azioneTipoCarattere()
	{
		stato.setText("Scelta del nuovo carattere di default.");
		ImpostaFont scegliFont = new ImpostaFont(this); // crea una dialog Window passando il frame corrente come owner
		if (scegliFont.returnFont() !=null)
		{
			areaTesto.setFont(scegliFont.returnFont());
			carattereDefault = scegliFont.returnFont();
			stato.setText("Nuovo carattere di default selezionato: " + carattereDefault.toString());
			salva_ImpostazioniDefault();
		}
		else
			stato.setText("Non e' stato selezionato alcun font.");
	}


	// **** imposta nell'area testo l'attivazione o meno della funzione di "A capo automatico" (detto Word-Wrap) ****

	private void azioneWordWrap()
	{
		if (wordWrapDefault.booleanValue() == true) // modifica l'impostazione del word-wrap nell'area testo
		{
			wordWrapDefault = new Boolean("False");
		}
		else
		{
			wordWrapDefault = new Boolean("True");
		}
		areaTesto.setLineWrap(wordWrapDefault.booleanValue()); // imposta se e' attivo o meno il word-wrap nell'area testo
		areaTesto.setWrapStyleWord(wordWrapDefault.booleanValue()); // imposta lo stile di word-wrapping
		jmi_wordwrap.setState(wordWrapDefault.booleanValue()); // modifica la selezione nel menu'
		stato.setText("Impostata proprieta' a capo automatico.");
		salva_ImpostazioniDefault();
	}


	// **** azioneImpostaLookMetal e' richiamato per cambiare il lookandfeel della finestra in "Metal" ****

	private void azioneImpostaLookMetal()
	{
		lookandfeelCorrente = "Metal";
		initLookAndFeel(lookandfeelCorrente);
		salva_ImpostazioniDefault();
		stato.setText("Nuovo lookandfeel impostato: Metal, riavviare l'applicazione.");
		azioneRiavviaEditor();
	}


	// **** azioneImpostaLookWindows e' richiamato per cambiare il lookandfeel della finestra in "Windows" ****

	private void azioneImpostaLookWindows()
	{
		lookandfeelCorrente = "Windows";
		initLookAndFeel(lookandfeelCorrente);
		salva_ImpostazioniDefault();
		stato.setText("Nuovo lookandfeel impostato: Windows, riavviare l'applicazione.");
		azioneRiavviaEditor();
	}


	// **** azioneImpostaLookSystem e' richiamato per cambiare il lookandfeel della finestra in "System" ****

	private void azioneImpostaLookSystem()
	{
		lookandfeelCorrente = "System";
		initLookAndFeel(lookandfeelCorrente);
		salva_ImpostazioniDefault();
		stato.setText("Nuovo lookandfeel impostato: System, riavviare l'applicazione.");
		azioneRiavviaEditor();
	}


	// **** azioneImpostaLookMotif e' richiamato per cambiare il lookandfeel della finestra in "Motif" ****

	private void azioneImpostaLookMotif()
	{
		lookandfeelCorrente = "Motif";
		initLookAndFeel(lookandfeelCorrente);
		salva_ImpostazioniDefault();
		stato.setText("Nuovo lookandfeel impostato: Motif, riavviare l'applicazione.");
		azioneRiavviaEditor();
	}


	// **** azioneImpostaLookMac e' richiamato per cambiare il lookandfeel della finestra in "Mac" ****

	private void azioneImpostaLookMac()
	{
		lookandfeelCorrente = "Mac";
		initLookAndFeel(lookandfeelCorrente);
     	salva_ImpostazioniDefault();
		stato.setText("Nuovo lookandfeel impostato: Mac, riavviare l'applicazione.");
		azioneRiavviaEditor();
	}


	// **** azioneGuida mostra una breve guida testuale con informazioni relative all'installazione ****

	private void azioneGuida()
	{
		Guida finestraguida = new Guida(this); // crea una dialog Window passando il frame corrente come owner
	}


	// **** azioneAboutBox e' richiamato per mostrare la finestra "About" relativa al programma ****

	private void azioneAboutBox()
	{
		AboutBox finestraAboutBox = new AboutBox(this);
	}


	// **** azioneRiavviaEditor distrugge l'attuale oggetto editor e ne crea uno nuovo con le  impostazioni correnti ****

	private void azioneRiavviaEditor()
	{
		int ritornoValore = JOptionPane.showConfirmDialog(getContentPane(), "Si desidera riavviare l'editor per rendere effettive le nuove impostazioni?", "Domanda", JOptionPane.YES_NO_OPTION);
		if (ritornoValore == JOptionPane.YES_OPTION)
		{
			dispose();
			XMLStudio XMLStudio = new XMLStudio();
			XMLStudio.setVisible(true);
		}
	}


	// **** segnala se il documento xml attivo e' o meno ben formato secondo le specifiche xml  ****

    	private boolean azioneBenFormatoXml()
    	{
	    	boolean benformato = false;

	    	if (fileCorrente.exists())
		{
			cursoreClessidra();
			// se il file caricato non e' .xml, segnala che non puoi verificare che il documento e' ben formato
			String nomefile = fileCorrente.getName();
			String estensione = nomefile.substring(nomefile.length()-4);
			if (estensione.equals(".xml") == false) // l'estensione del file corrente non e' .xml
			{
				JOptionPane.showInternalMessageDialog(getContentPane(), "Il file aperto non e' .xml, per cui  non e' possibile verificare che il documento sia ben formato.", "Informazione", JOptionPane.INFORMATION_MESSAGE);
			}
			else
			{
	    			try
	    			{
		    			int ritornoValore = JOptionPane.showConfirmDialog(getContentPane(), "Si desidera sovrascrivere il file " + fileCorrente.getName() + "?", "Domanda", JOptionPane.YES_NO_OPTION);
					if (ritornoValore == JOptionPane.YES_OPTION)
						fileGiaSalvato = true; // e' stato selezionato di sovrascrivere il file
     				if (ritornoValore == JOptionPane.NO_OPTION)
						fileGiaSalvato = false; // e' stato selezionato di non sovrascrivere il file

					azioneSalvaFile(); // prima di creare l'albero xml e' necessario salvare il file

					// DocumentBuilderFactory definisce API che consentono ad applicazioni di ottenere un parser che produce oggetti DOM ad albero da documenti XML
				    	DocumentBuilderFactory docBuilderFactory = DocumentBuilderFactory.newInstance(); // crea una nuova istanza di DocumentBuilderFactory
				    	// docBuilderFactory.setValidating(false); // estensione di XMLStudio: per verificare anche la validita', impostarlo su true
				    	// DocumentBuilder e' usato per ottenere un'istanza di documento DOM da un documento XML
					DocumentBuilder docBuilder = docBuilderFactory.newDocumentBuilder(); // crea una nuova istanza di DocumentBuilder usando i parametri definiti

					try
					{
						documentoXml = docBuilder.parse(fileCorrente); // parsifica il contenuto di un dato file come un documento XML e restituisce un nuovo oggetto documento DOM
						JOptionPane.showInternalMessageDialog(getContentPane(), "Il documento " + fileCorrente.getName() + " e' ben formato: e' possibile creare l'albero di parsing.", "Informazione", JOptionPane.INFORMATION_MESSAGE);
						stato.setText("Il documento " + fileCorrente.getName() + " e' ben formato.");
						benformato = true;
					}
					catch(SAXException e)
					{
						JOptionPaneEsteso mostraEccezione = new JOptionPaneEsteso(this, "Informazione", "Il documento " + fileCorrente.getName() + " non e' xml ben formato: " + e + " Si consiglia di apportare le modifiche per renderlo ben formato. " + azioneTrovaCaratteriNonValidi());
						// JOptionPane.showInternalMessageDialog(getContentPane(), "Il documento" + fileCorrente.getName() + " non e' xml ben formato: si consiglia di apportare le modifiche per renderlo ben formato.", "Attenzione", JOptionPane.ERROR_MESSAGE);
						stato.setText("Il documento " + fileCorrente.getName() + " non e' ben formato.");
					}
				}
				catch (Exception e)
 				{
					JOptionPaneEsteso mostraEccezione = new JOptionPaneEsteso(this, "Informazione", "Eccezione rilevata nella verifica che il documento XML sia ben formato: " + e);
					// e.printStackTrace();
				}
			}
			cursoreNormale();
		}
		else // se fileCorrente non esiste
		{
			JOptionPane.showInternalMessageDialog(getContentPane(), "Per verificare che un documento xml sia ben formato, e' prima necessario caricare un file .xml.", "Informazione", JOptionPane.INFORMATION_MESSAGE);
		}
		return benformato; // se non ha gia' restituito return in altri sottomenu
	}


	// **** segnala se il documento xml attivo e' o meno valido secondo le specifiche xml (rispetta l'eventuale dtd incluso) ****

    	private boolean azioneValidoXml()
    	{
	    	boolean valido = true; // per la rilevazione di eccezioni durante il controllo di validita' del documento xml
	    	MyErrorHandler errori = null; // per la rilevazione di errori durante il controllo di validita' del documento xml

	    	if (fileCorrente.exists())
		{
			cursoreClessidra();
			// se il file caricato non e' .xml, segnala che non puoi verificare che il documento e' ben formato
			String nomefile = fileCorrente.getName();
			String estensione = nomefile.substring(nomefile.length()-4);
			if (estensione.equals(".xml") == false) // l'estensione del file corrente non e' .xml
			{
				JOptionPane.showInternalMessageDialog(getContentPane(), "Il file aperto non e' .xml, per cui  non e' possibile verificare che il documento sia ben formato.", "Informazione", JOptionPane.INFORMATION_MESSAGE);
			}
			else
			{
	    			try
	    			{
		    			int ritornoValore = JOptionPane.showConfirmDialog(getContentPane(), "Si desidera sovrascrivere il file " + fileCorrente.getName() + "?", "Domanda", JOptionPane.YES_NO_OPTION);
					if (ritornoValore == JOptionPane.YES_OPTION)
						fileGiaSalvato = true; // e' stato selezionato di sovrascrivere il file
     				if (ritornoValore == JOptionPane.NO_OPTION)
						fileGiaSalvato = false; // e' stato selezionato di non sovrascrivere il file

					azioneSalvaFile(); // prima di creare l'albero xml e' necessario salvare il file

					// DocumentBuilderFactory definisce API che consentono ad applicazioni di ottenere un parser che produce oggetti DOM ad albero da documenti XML
				    	DocumentBuilderFactory docBuilderFactory = DocumentBuilderFactory.newInstance(); // crea una nuova istanza di DocumentBuilderFactory
				    	docBuilderFactory.setValidating(true); // estensione di XMLStudio: per verificare anche la validita', impostarlo su true
				    	// DocumentBuilder e' usato per ottenere un'istanza di documento DOM da un documento XML
					DocumentBuilder docBuilder = docBuilderFactory.newDocumentBuilder(); // crea una nuova istanza di DocumentBuilder usando i parametri definiti
					errori = new MyErrorHandler(this);
					docBuilder.setErrorHandler(errori); // per la rilevazione di errori relativi alla validita' del documento

					try
					{
						documentoXml = docBuilder.parse(fileCorrente); // parsifica il contenuto di un dato file come un documento XML e restituisce un nuovo oggetto documento DOM
					}
					catch(SAXException e)
					{
						valido = false;
						JOptionPaneEsteso mostraEccezione = new JOptionPaneEsteso(this, "Informazione", "Il documento " + fileCorrente.getName() + " non e' valido: " + e + " Si consiglia di apportare le modifiche per renderlo ben formato. " + azioneTrovaCaratteriNonValidi());
						// JOptionPane.showInternalMessageDialog(getContentPane(), "Il documento" + fileCorrente.getName() + " non e' xml valido: si consiglia di apportare le modifiche per renderlo ben formato.", "Attenzione", JOptionPane.ERROR_MESSAGE);
					}
				}
				catch (Exception e)
 				{
	 				valido = false;
					JOptionPaneEsteso mostraEccezione = new JOptionPaneEsteso(this, "Informazione", "Eccezione rilevata nella verifica che il documento XML sia valido: " + e);
					// e.printStackTrace();
				}
			}
			cursoreNormale();
		}
		else // se fileCorrente non esiste
		{
			JOptionPane.showInternalMessageDialog(getContentPane(), "Per verificare che un documento xml sia ben formato, e' prima necessario caricare un file .xml.", "Informazione", JOptionPane.INFORMATION_MESSAGE);
		}

		if (((errori.valoreValido()) == true) && (valido == true)) // indica all'utente l'esito dell'analisi: se il documento e' o meno valido
		{
			stato.setText("Il documento " + fileCorrente.getName() + " e' valido.");
			JOptionPane.showInternalMessageDialog(getContentPane(), "Il documento " + fileCorrente.getName() + " e' valido: e' possibile creare l'albero di parsing.", "Informazione", JOptionPane.INFORMATION_MESSAGE);
			valido = true;
		}
		else
		{
			stato.setText("Il documento " + fileCorrente.getName() + " non e' valido.");
			JOptionPane.showInternalMessageDialog(getContentPane(), "Il documento " + fileCorrente.getName() + " non e' valido.", "Informazione", JOptionPane.INFORMATION_MESSAGE);
			valido = false;
		}
		return valido; // se non ha gia' restituito return in altri sottomenu
	}


	// **** crea o aggiorna l'albero che rappresenta l'attuale documento Xml ****

    	private void azioneCreaAlberoXmlVuoto()
    	{
	    	// System.out.println("azioneCreaAlberoXml: " + fileCorrente.getName());
	    	try
	    	{
	  		DefaultMutableTreeNode nodoRadice = new DefaultMutableTreeNode("Albero xml vuoto");
 			modelloAlbero = new DefaultTreeModel(nodoRadice); // crea un albero, passandovi la radice, in cui ogni nodo ha dei figli
	 		alberoXml.setModel(modelloAlbero);
 			alberoXml.treeDidChange(); // impostato quando il JTree e' cambiato a tal punto da necessitare il ridimensionamento
 			espandiAlbero(alberoXml); // espande l'albero JTree legato al documento Xml affinche' sia visibile nel viewer
 			aggiornaAttributiNodoInTavola(null); // imposta a null gli attributi nella tabella dei nodi
 			// stato.setText("Albero relativo al documento " + fileCorrente.getName() + " non e' aggiornato.");
		}
		catch (Exception e)
 		{
	 		// System.out.println("Eccezione rilevata nella creazione dell'albero: " + e);
			JOptionPaneEsteso mostraEccezione = new JOptionPaneEsteso(this, "Informazione", "Eccezione rilevata nella creazione dell'albero: " + e + ". Il documento xml analizzato non e' ben formato.");
			// e.printStackTrace();
		}
	}


	// **** crea o aggiorna l'albero che rappresenta l'attuale documento Xml ****

    	private void azioneCreaAlberoXml()
    	{
	    	//System.out.println("azioneCreaAlberoXml: " + fileCorrente.getName());
	    	if (fileCorrente.exists())
		{
			// se il file caricato non e' .xml, segnala che non puoi creare l'albero xml
			String nomefile = fileCorrente.getName();
			String estensione = nomefile.substring(nomefile.length()-4);
			if (estensione.equals(".xml") == false) // l'estensione del file corrente non e' .xml
			{
				JOptionPane.showInternalMessageDialog(getContentPane(), "Il file aperto non e' .xml, per cui  non e' possibile creare l'albero xml.", "Informazione", JOptionPane.INFORMATION_MESSAGE);
			}
			else
			{
	    			try
	    			{
		    			int ritornoValore = JOptionPane.showConfirmDialog(getContentPane(), "Si desidera sovrascrivere il file " + fileCorrente.getName() + "?", "Domanda", JOptionPane.YES_NO_OPTION);
					if (ritornoValore == JOptionPane.YES_OPTION)
						fileGiaSalvato = true; // e' stato selezionato di sovrascrivere il file
     				if (ritornoValore == JOptionPane.NO_OPTION)
						fileGiaSalvato = false; // e' stato selezionato di non sovrascrivere il file

					azioneSalvaFile(); // prima di creare l'albero xml e' necessario salvare il file

					// DocumentBuilderFactory definisce API che consentono ad applicazioni di ottenere un parser che produce oggetti DOM ad albero da documenti XML
				    	DocumentBuilderFactory docBuilderFactory = DocumentBuilderFactory.newInstance(); // crea una nuova istanza di DocumentBuilderFactory
				    	// DocumentBuilder e' usato per ottenere un'istanza di documento DOM da un documento XML
					DocumentBuilder docBuilder = docBuilderFactory.newDocumentBuilder(); // crea una nuova istanza di DocumentBuilder usando i parametri definiti

					try
					{
						cursoreClessidra();
						documentoXml = docBuilder.parse(fileCorrente); // parsifica il contenuto di un dato file come un documento XML e restituisce un nuovo oggetto documento DOM
						// System.out.println(fileCorrente.getName() + " e' xml ben formato.");

						Element nodoRadice = documentoXml.getDocumentElement();  // individua il nodo radice dal documento Xml
						nodoRadice.normalize(); // assicura che la vista del documento DOM sia la stessa come se fosse salvato e caricato

						DefaultMutableTreeNode topAlbero = creaAlberoNodi(nodoRadice); // passando nodoRadice crea l'albero

						modelloAlbero.setRoot(topAlbero); // imposta topAlbero come radice
						alberoXml.treeDidChange(); // impostato quando il JTree e' cambiato a tal punto da necessitare il ridimensionamento
						espandiAlbero(alberoXml); // espande l'albero JTree legato al documento Xml affinche' sia visibile nel viewer
						aggiornaAttributiNodoInTavola(null); // imposta a null gli attributi nella tabella dei nodi

						stato.setText("Creato l'albero relativo al documento ben formato " + fileCorrente.getName());
						cursoreNormale();
					}
					catch(SAXException e)
					{
						// System.out.println(fileCorrente.getName() + " non e' xml ben formato.");
						// JOptionPane.showInternalMessageDialog(getContentPane(), "Il documento " + fileCorrente.getName() + " non e' xml ben formato: albero non creato.", "Attenzione", JOptionPane.ERROR_MESSAGE);
						JOptionPaneEsteso mostraEccezione = new JOptionPaneEsteso(this, "Informazione", "Il documento " + fileCorrente.getName() + " non e' xml ben formato: " + e + " Si consiglia di apportare le modifiche per renderlo ben formato.");
						stato.setText("Il documento " + fileCorrente.getName() + " non e' xml ben formato.");
						cursoreNormale();
					}
				}
				catch (Exception e)
 				{
	 				// System.out.println("Eccezione rilevata nella creazione dell'albero: " + e);
					JOptionPaneEsteso mostraEccezione = new JOptionPaneEsteso(this, "Informazione", "Eccezione rilevata nella creazione dell'albero: " + e + ". Il documento xml analizzato non e' ben formato.");
					cursoreNormale();
					// e.printStackTrace();
				}
			}
		}
		else // se fileCorrente non esiste
		{
			JOptionPane.showInternalMessageDialog(getContentPane(), "Per creare l'albero xml e' prima necessario caricare un file .xml.", "Informazione", JOptionPane.INFORMATION_MESSAGE);
		}
	}


	// **** dato il nodo radice crea l'oggetto alberoNodi di tipo XmlNodo che estende DefaultMutableTreeNode ****

	private DefaultMutableTreeNode creaAlberoNodi(Node nodoRadice) // dato un nodo crea la gerarchia di nodi ad alber
	{
		if (!possoVisualizzareNodo(nodoRadice)) // se non puo' visualizzare la radice, ritorna null
		{
			stato.setText("Analisi del file xml: non posso visualizzare la radice.");
			return null;
		}

		XmlNodo alberoNodi = new XmlNodo(nodoRadice); // crea un oggetto treeNode di tipo XmlNodo per rappresentare la radice

		NodeList listaNodi = nodoRadice.getChildNodes(); // listaNodi e' una collezione ordinata di nodi: rappresenta tutti i nodi figli del nodo radice

		// per ogni nodo figlio viene creato un albero di nodi richiamando ricorsivamente creaAlberoNodi
		for (int k = 0; k < listaNodi.getLength(); k++) // getLength restituisce il numero di nodi nella lista
		{
			Node nodo = listaNodi.item(k); // ritorna il k-esimo nodo selezionato
			DefaultMutableTreeNode nodoFiglio = creaAlberoNodi(nodo); // chiamata ricorsiva del metodo passandogli il nodo e assumendolo come figlio
			if (nodoFiglio != null)
				alberoNodi.add(nodoFiglio); // aggiunge all'oggetto treeNode il nodo figlio esaminato
		}
		return alberoNodi; // alla fine restituisci l'oggetto treeNode creato
	}


	// **** metodo responsabile di espandere ogni nodo a partire dalla radice affinche' sia visibile l'albero nel viewer ****

	private static void espandiAlbero(JTree alberoJTree) // riceve come parametro alberoXml da espandere
	{
	    // oggettoRadice e' nodo dell'albero in un JTree
	    // getModel restituisce il modello di tipo TreeModel usato per i dati
	    // getRoot restituisce il nodo radice
	    TreeNode nodoRadice = (TreeNode)alberoJTree.getModel().getRoot(); // individua il nodo radice di questo albero
	    TreePath path = new TreePath(nodoRadice); // individua il path della radice di questo albero
	    for (int k = 0; k < nodoRadice.getChildCount(); k++) // per ogni nodo figlio della radice fai
	    {
			TreeNode nodoFiglio = (TreeNode)nodoRadice.getChildAt(k); // individua il k-esimo nodo figlio della radice
			espandiAlbero(alberoJTree, path, nodoFiglio); // espandi l'albero dati il Jtree, il path della radice e il nodoFiglio
	    }
	}


	// **** metodo responsabile di espandere ogni nodo affinche' sia visibile l'albero nel viewer ****

	private static void espandiAlbero(JTree alberoJTree, TreePath path, TreeNode nodo)
	{
		if (path == null || nodo == null) // se il path del nodo radice e' nullo o il nodo radice e' nullo
			return;				    // esci dal metodo
		alberoJTree.expandPath(path); // dato il path, assicura che il nodo identificato dal path specificato sia espanso e visibile
		TreePath nuovoPath = path.pathByAddingChild(nodo); // restituisce un nuovo path contenente tutti gli elementi del nodo padre e del nodo figlio
		for (int k = 0; k < nodo.getChildCount(); k++) // per ogni nodo figlio di nodo (che e' gia' un nodo interno dell'albero) fai
		{
			TreeNode nodoFiglio = (TreeNode)nodo.getChildAt(k); // individua il k-esimo nodo figlio del nodo considerato
			if (nodoFiglio != null) // se nodoFiglio individuato non e' null allora espandi quella porzione dell'albero
			{
				espandiAlbero(alberoJTree, nuovoPath, nodoFiglio); // alberoJTree, il path del nodo padre, il nodo figlio
			}
		}
	}


	// **** verifica se il nodo (radice) e' un elemento o un nodo testuale ****

	private boolean possoVisualizzareNodo(Node nodocorrente) // se il nodo non e' elemento o testuale non viene visualizzato nell'albero
	{
		switch (nodocorrente.getNodeType())  // estrapola il tipo del nodo attualmente esaminato
		{
			case Node.ELEMENT_NODE: // il nodo e' di tipo elemento nel file xml
				return true;
			case Node.TEXT_NODE: // il nodo e' di tipo testuale
				String testonodo = nodocorrente.getNodeValue().trim(); // essendo il nodo testuale, preleva il suo valore ovvero del testo
				return !(testonodo.equals("") || testonodo.equals("\n") || testonodo.equals("\r\n")); // ritorna true o false dopo i confronti
		}
		return false;
	}


	// **** restituisce il valore della variabile fileXmlTagCorrente ****

	public String getfileXmlTagCorrente()
	{
		return fileXmlTagCorrente; // restituisce il valore della variabile fileXmlTagCorrente
	}


	// **** metodo che consente l'inserimento dei tag xml all'interno del documento attivo ****

	private void azioneAggiungiTagXml()
	{
		Node nodoSelezionato; // nodo attualmente selezionato
		String testoNodo = null, // contiene il testo del nodo selezionato
		       testoAttributi = null, // contiene la sequenza di attributi relativi al nodo selezionato
		       testoTagXml1 = null, // contiene il testo relativo al tag comprendente anche gli attributi
		       testoTagXml2 = null, // contiene il testo relativo alla chiusura del tag
		       testoTagXmlCompleto = null; // contiene la selezione, i due tag xml ed eventuali attributi

		try
		{
			nodoSelezionato = oggettoXmlTag.restituisciNodoSelezionato(); // in nodoselezionato viene posto il nodo selezionato nell'oggetto XmlTag

			switch (nodoSelezionato.getNodeType())  // estrapola il tipo del nodo dal nodo attualmente esaminato
			{
				case Node.ELEMENT_NODE: // il nodo e' di tipo elemento nel file xml
				{
					testoNodo = nodoSelezionato.getNodeName();

					int[] rigaSelezionata = oggettoXmlTag.tavolaAttributiXmlTag.getSelectedRows(); // aggiunge in testoAttributi eventuali attributi presenti nel nodo
					for (int i=0; i<rigaSelezionata.length; i++)
					{
						if (testoAttributi == null)
							testoAttributi = " " + (oggettoXmlTag.tavolaAttributiXmlTag.getValueAt(rigaSelezionata[i], 0)).toString() + "=\"" + (oggettoXmlTag.tavolaAttributiXmlTag.getValueAt(rigaSelezionata[i], 1)).toString() + "\"";
						else
							testoAttributi = testoAttributi + " " + (oggettoXmlTag.tavolaAttributiXmlTag.getValueAt(rigaSelezionata[i], 0)).toString() + "=\"" + (oggettoXmlTag.tavolaAttributiXmlTag.getValueAt(rigaSelezionata[i], 1)).toString() + "\"";
					}

					if (testoAttributi == null)
					{
						testoTagXml1 = "<" + testoNodo + ">";
						testoTagXml2 = "</" + testoNodo + ">";
					}
					else
					{
						testoTagXml1 = "<" + testoNodo + testoAttributi + ">";
						testoTagXml2 = "</" + testoNodo + ">";
					}
					break;
				}
				case Node.TEXT_NODE: // il nodo e' di tipo testuale
				{
					testoNodo = nodoSelezionato.getNodeValue(); // essendo il nodo testuale, preleva il suo valore ovvero del testo
					testoTagXml1 = "<" + testoNodo + ">";
					testoTagXml2 = "</" + testoNodo + ">";
					break;
				}

			}
			testoTagXmlCompleto = testoTagXml1 + (areaTesto.getSelectedText()) + testoTagXml2;
			areaTesto.replaceSelection(testoTagXmlCompleto);
			stato.setText("Inserito tag " + testoTagXml1);
		}
		catch (Exception e)
		{
			JOptionPaneEsteso mostraEccezione = new JOptionPaneEsteso(this, "Informazione", "Eccezione rilevata nell'aggiunta del tag xml: " + e + ". Probabile errore da correggere nel file xml oppure nodo xml non selezionato.");
			stato.setText("Rilevata eccezione nell'aggiunta del tag xml.");
		}

	}


	// **** metodo che consente la rimozione dei tag xml dal testo selezionato ****

	private void azioneRimuoviTagXml()
	{
		String testoSelezionato;
		int i,j;

		testoSelezionato = areaTesto.getSelectedText();
		if (testoSelezionato == null)
		{
			JOptionPane.showInternalMessageDialog(getContentPane(), "Per rimuovere i tag e' necessario selezionare un paragrafo che li includa come estremi.", "Informazione", JOptionPane.ERROR_MESSAGE);
          	stato.setText("Testo non selezionato.");
		}
		else
		{
			if ((testoSelezionato.indexOf('<') != 0) || (testoSelezionato.lastIndexOf(">") != (testoSelezionato.length()-1)))
			{
				JOptionPane.showInternalMessageDialog(getContentPane(), "E' necessario selezionare un paragrafo che includa esattamente i tag come estremi.", "Informazione", JOptionPane.ERROR_MESSAGE);
          		stato.setText("Errore di selezione del testo.");
 			}
 			else
 			{
	 			i = testoSelezionato.indexOf('>');
				j = testoSelezionato.lastIndexOf("</");
				areaTesto.replaceSelection(testoSelezionato.substring(i+1, j));
 				stato.setText("Etichette rimosse dal paragrafo.");
 			}
		}
	}


	// **** restituisce true se la stringa testo termina con contiene ****

	private boolean selezioneContieneFine(String testo, String contiene)
	{
		if (testo != null)
			return testo.endsWith(contiene);
		else
			return false;
	}


	// **** restituisce true se la stringa testo inizia con contiene ****

	private boolean selezioneContieneInizio(String testo, String contiene)
	{
		if (testo != null)
			return testo.startsWith(contiene);
		else
			return false;
	}


	// **** metodo che consente la selezione di un paragrafo compreso tra due tag xml ****

	private void azioneSelezionaTagXml()
	{
		String testo; // include la porzione di testo da esaminare
		boolean uscita = false; // consente di uscire dal ciclo while
		int posizioneDot = areaTesto.getCaret().getDot(); // rileva la posizione corrente del cursore
		int salvaPosizioneDotInizio, salvaPosizioneDotFine, salvaPosizioneDotCentrale;
		String testoSelezionato, stringaTag, salvaStringaTag;
		int contatore = 0;

		cursoreClessidra();
		areaTesto.requestFocus(); // richiama il focus sulla JTextArea areaTesto
		salvaPosizioneDotCentrale = posizioneDot; // salva la posizione dove l'utente ha posizionato il cursore

		while (!uscita)
		{
			areaTesto.select(areaTesto.getCaret().getMark(), areaTesto.getCaret().getDot());
			testo = areaTesto.getSelectedText();

			if ((selezioneContieneFine(testo, ">") == true) || (selezioneContieneFine(testo, " ") == true)) // verifica se e' stato raggiunto l'elemento >
			{
				uscita = true;
			}
			else
			{
				posizioneDot = posizioneDot + 1; // sposta il cursore avanti di una posizione
				areaTesto.getCaret().moveDot(posizioneDot); // realizza lo spostamento del cursore
			}
		}
		salvaPosizioneDotFine = posizioneDot; // individuato l'estremo destro della selezione

 		areaTesto.getCaret().setDot(salvaPosizioneDotCentrale);
 		posizioneDot = salvaPosizioneDotCentrale;
 		uscita = false;
 		while (!uscita)
 		{
 			areaTesto.select(areaTesto.getCaret().getDot(), areaTesto.getCaret().getMark());
 			testo = areaTesto.getSelectedText();
 			if ((selezioneContieneInizio(testo, "<") == true) || (selezioneContieneInizio(testo, " ") == true))// verifica se e' stato raggiunto l'elemento <
 			{
 				uscita = true;
 				if (selezioneContieneInizio(testo, " ") == true)
 				{
	 				cursoreNormale();
	 				areaTesto.select(0, 0); // non selezionare il testo
	 				stato.setText("Selezione non avvenuta.");
			 		JOptionPane.showInternalMessageDialog(getContentPane(), "Errore di selezione: posizionare il cursore sul tag di inizio o di fine e non sugli attributi.", "Informazione", JOptionPane.ERROR_MESSAGE);
			 		return;
 				}
 			}
 			else
 			{
 				posizioneDot = posizioneDot - 1; // sposta il cursore indietro di una posizione
 				areaTesto.getCaret().moveDot(posizioneDot); // realizza lo spostamento del cursore
 			}
 		}
 		salvaPosizioneDotInizio = posizioneDot; // individuato l'estremo sinistro della selezione
 		//areaTesto.getCaret().setSelectionVisible(false); // rendi invisibile la selezione effettuata
 		//areaTesto.getCaret().setVisible(false); // rendi invisibile la selezione effettuata
 		areaTesto.select(salvaPosizioneDotInizio, salvaPosizioneDotFine); // seleziona il testo

 		testoSelezionato = areaTesto.getSelectedText();

//  		if (testoSelezionato.indexOf("<") == testoSelezionato.lastIndexOf("<"))
//  		{
	 		stato.setText("Individuato il tag " + testoSelezionato);

	 		if (selezioneContieneInizio(testoSelezionato, "</") == false)
 			{
	 			// andare avanti per trovare </tag>
	 			salvaStringaTag = testoSelezionato.substring(1, (testoSelezionato.length()-1));
	 			stringaTag = "</" + salvaStringaTag + ">"; // contiene il tag ad esempio applet estrapolato da <applet> al quale sono aggiunti </ e >

	 			uscita = false;
 				while (!uscita)
 				{
	 				posizioneDot = posizioneDot + 1; // sposta il cursore avanti di una posizione
 					areaTesto.getCaret().moveDot(posizioneDot); // realizza lo spostamento del cursore
 					areaTesto.select(salvaPosizioneDotInizio, areaTesto.getCaret().getDot());
 					testo = areaTesto.getSelectedText();

 					if (selezioneContieneFine(testo, stringaTag) == true) // verifica se e' stato raggiunto l'elemento <
 					{
	 					if (contatore>0) contatore--; // ha trovato </tag>
 						if (contatore == 0)	uscita = true;
 					}
 					else
 					{
	 					if (selezioneContieneFine(testo, "<"+salvaStringaTag) == true) contatore++; // ha trovato <tag
 					}
 				}
 				areaTesto.select(salvaPosizioneDotInizio, posizioneDot); // seleziona il testo
 				areaTesto.getCaret().setSelectionVisible(true); // rendi visibile la selezione effettuata
 				stato.setText("Testo selezionato compreso tra le etichette <" + salvaStringaTag + "> ... </" + salvaStringaTag + ">");
 			}
 			else
 			{
		 		//tornare indietro per trovare <tag>
	 			salvaStringaTag = testoSelezionato.substring(2, (testoSelezionato.length()-1));
	 			stringaTag = "<" + salvaStringaTag + ">"; // contiene il tag ad esempio applet estrapolato da <applet> al quale sono aggiunti < e >

	 			uscita = false;
 				while (!uscita)
 				{
	 				posizioneDot = posizioneDot - 1; // sposta il cursore indietro di una posizione
 					areaTesto.getCaret().moveDot(posizioneDot); // realizza lo spostamento del curs
 					areaTesto.select(areaTesto.getCaret().getDot(), salvaPosizioneDotFine);
 					testo = areaTesto.getSelectedText();

 					if (selezioneContieneInizio(testo, "<"+salvaStringaTag) == true) // verifica se e' stato raggiunto l'elemento <
 					{
 						if (contatore == 0)	uscita = true;
 						if (contatore>0) contatore--; // ha trovato <tag
 					}
 					else
 					{
	 					if (selezioneContieneInizio(testo, "</"+salvaStringaTag+">") == true) contatore++; // ha trovato </tag
 					}
 				}
 				areaTesto.select(posizioneDot, salvaPosizioneDotFine); // seleziona il testo
 				areaTesto.getCaret().setSelectionVisible(true); // rendi visibile la selezione effettuata
 				stato.setText("Testo selezionato compreso tra le etichette <" + salvaStringaTag + "> ... </" + salvaStringaTag + ">");
 			}

//  		}
//  		else
//  		{
// 	 		areaTesto.select(0, 0); // non selezionare il testo
// 	 		stato.setText("Selezione non avvenuta.");
// 	 		JOptionPane.showInternalMessageDialog(getContentPane(), "Errore di selezione del testo: posizionare il cursore sul tag di inizio o fine.", "Informazione", JOptionPane.ERROR_MESSAGE);
//  		}

 		cursoreNormale();
	}


	// **** crea un documento .dtd a partire dal file documento .xml attivo ****

	private boolean azioneCreaDtd()
	{
		boolean creadtd = false;

		// determina il nome del file DTD
		String nomefilexml = fileCorrente.getPath(); // nome del file xml caricato
		String estensione = nomefilexml.substring(nomefilexml.length()-4); // estrae l'estensione del file
		if (estensione.equals(".xml") == true) // verifica che l'estensione del file caricato sia xml
		{
			String nomefile = nomefilexml.substring(0, nomefilexml.length()-4); // nomefile contiene il path e il nome del file DTD
			String nomefileDTD = nomefile + ".dtd";

			if (azioneBenFormatoXml()) // verifica che il documento Xml caricato sia ben formato
			{
				try
				{
					DTDGenerator oggettoCreaDTD = new DTDGenerator(); // o getAbsolutePath()
					oggettoCreaDTD.run(fileCorrente.getPath());
          			oggettoCreaDTD.printDTD(nomefileDTD);
          			JOptionPane.showInternalMessageDialog(getContentPane(), "Il file DTD " + nomefileDTD + " e' stato creato.", "Informazione", JOptionPane.INFORMATION_MESSAGE);
          			stato.setText("File DTD " + nomefileDTD + " creato.");
          			creadtd = true;
          		}
          		catch (Exception e)
          		{
					JOptionPane.showInternalMessageDialog(getContentPane(), "Errore rilevato durante la creazione del file DTD " + nomefileDTD, "Informazione", JOptionPane.ERROR_MESSAGE);
          			stato.setText("File DTD non creato.");
          		}
          	}
          }
         	else
         	{
	         	JOptionPane.showInternalMessageDialog(getContentPane(), "Per creare il file DTD e' prima necessario aprire un file XML ben formato.", "Informazione", JOptionPane.INFORMATION_MESSAGE);
			stato.setText("File xml per creare il DTD non caricato.");
		}
		return creadtd;
	}


	// **** crea, a partire dal documento xml corrente, un documento xml valido (che include il dtd associato) ****

	private void azioneCreaDocumentoXmlValido()
	{
		if (azioneCreaDtd()) // crea il DTD associato al documento XML
		{
			try
			{
				String nomefilexml = fileCorrente.getName(); // nome del file xml caricato
				String nomefile = nomefilexml.substring(0, nomefilexml.length()-4); // nomefile contiene il path e il nome del file DTD
				String nomefileDTD = nomefile + ".dtd";

				String stringa = "<?xml version=\"1.0\" encoding=\"iso-8859-1\" standalone=\"no\"?>\n<!DOCTYPE documento SYSTEM \"./" + nomefileDTD + "\">\n<documento>\n";
				areaTesto.replaceRange(stringa, 0, areaTesto.getLineEndOffset(1));

				JOptionPane.showInternalMessageDialog(getContentPane(), "Il documento " + fileCorrente.getName() + " e' ora valido.", "Informazione", JOptionPane.INFORMATION_MESSAGE);
				stato.setText("Il documento " + fileCorrente.getName() + " e' valido.");
			}
			catch (Exception e)
			{
				JOptionPane.showInternalMessageDialog(getContentPane(), "Errore rilevato durante la creazione del documento valido " + fileCorrente.getName(), "Informazione", JOptionPane.ERROR_MESSAGE);
          		stato.setText("Documento xml valido " + fileCorrente.getName() + " non creato.");
			}
		}
	}


	// **** nella creazione del foglio di stile, selezionato un nodo, ne restituisce il testo ****

	private String restituisciNodoperCSS()
	{
		Node nodoSelezionato; // nodo attualmente selezionato
		String testoNodo = null; // contiene il testo del nodo selezionato
		boolean uscita = false; // permette da uscire dal ciclo while

		try
		{
			while (!uscita)
			{
				JOptionPane.showInternalMessageDialog(getContentPane(), "Selezionare un tag per la sua colorazione nel CSS.", "Informazione", JOptionPane.INFORMATION_MESSAGE);
				nodoSelezionato = oggettoXmlTag.restituisciNodoSelezionato(); // in nodoselezionato viene posto il nodo selezionato nell'oggetto XmlTag

				switch (nodoSelezionato.getNodeType()) // estrapola il tipo del nodo dal nodo attualmente esaminato
				{
					case Node.ELEMENT_NODE: // il nodo e' di tipo elemento nel file xml
					{
						testoNodo = nodoSelezionato.getNodeName();
						uscita = true;
						break;
					}
					case Node.TEXT_NODE: // il nodo e' di tipo testuale
					{
						// testoNodo = nodoSelezionato.getNodeValue(); // essendo il nodo testuale, preleva il suo valore ovvero del testo
						JOptionPane.showInternalMessageDialog(getContentPane(), "E' necessario selezionare il nodo e non la componente testuale.", "Informazione", JOptionPane.ERROR_MESSAGE);
						break;
					}
				}
			}
		} catch (Exception e)
		{
			JOptionPane.showInternalMessageDialog(getContentPane(), "Errore durante la selezione del nodo nella creazione del file CSS.", "Informazione", JOptionPane.ERROR_MESSAGE);
		}
		return testoNodo;
	}


	// **** crea il foglio di stile CSS ****

	private void elaboraSalvaCSS(File fileCss)
	{
		boolean uscita = false; // permette di uscire dal ciclo while
		int ritornoValore; // valore restituito per le domande
		String valoreRosso, valoreVerde, valoreBlu, valoreRGB, nodoInCSS; // stringhe relativi ai colori in formato RGB e al testo del nodo selezionato

		try
		{
			BufferedWriter out = new BufferedWriter(new FileWriter(fileCss)); // crea il fileCSS
			out.write("documento { color : #000000; font-family : Arial Black; }"); // inizia a scrivere l'intestazione di default nel file
			out.newLine(); // scrivi una nuova linea

	        	while (!uscita)
	        	{
		        	coloreTestoDefault = JColorChooser.showDialog(this, "Scegliere colore per testo individuato dal tag", Color.black);

		        	//String valore = int2hexstr(coloreTestoDefault.getRGB());
		        	valoreRosso = Integer.toHexString(coloreTestoDefault.getRed());
		        	if (valoreRosso.length() == 1)
		        		valoreRosso += "0";
		        	valoreVerde = Integer.toHexString(coloreTestoDefault.getGreen());
		        	if (valoreVerde.length() == 1)
		        		valoreVerde += "0";
		        	valoreBlu = Integer.toHexString(coloreTestoDefault.getBlue());
		        	if (valoreBlu.length() == 1)
		        		valoreBlu += "0";

		        	valoreRGB = valoreRosso + valoreVerde + valoreBlu;

		        	nodoInCSS = restituisciNodoperCSS();

		        	out.write(nodoInCSS + " { color : #" + valoreRGB + "; border : solid 1px; font-family : Arial Black; padding: 4px; width: 100%; }");
		        	out.newLine(); // scrivi una nuova linea

		        	ritornoValore = JOptionPane.showConfirmDialog(getContentPane(), "Si desidera memorizzare un altro colore per tag sul file " + fileCss.getName() + "?", "Domanda", JOptionPane.YES_NO_OPTION);
				if (ritornoValore == JOptionPane.NO_OPTION)
					uscita = true;
	          }
	          out.close( );
 		} catch (Exception e)
 		{
	 		JOptionPane.showInternalMessageDialog(getContentPane(), "Errore durante l'elaborazione dei colori nella creazione del file CSS.", "Informazione", JOptionPane.ERROR_MESSAGE);
 		}
	}


	// **** crea e salva il foglio di stile Css ****

	private void azioneCreaCss()
	{
	     boolean salvareFile = false;

	    	JFileChooser fileChooser = new JFileChooser(directoryCorrente); // crea l'oggetto per l'apertura del file sulla directory corrente
		// imposta la corrente directory di lavoro, esempio /Tesi/Tesi XMLStudio oppure "."

		fileChooser.setApproveButtonText("Salva"); // imposta il valore del bottone di esecuzione
		fileChooser.setDialogTitle("Salva file css"); // imposta il titolo della finestra Dialog

		// imposta ed aggiunge i filtri opportuni
		fileChooser.resetChoosableFileFilters(); // reinizializza l'oggetto fileChooser
		FiltroFile cssFiltro = new FiltroFile("css", "File css *.CSS"); // imposta il filtro per i file .css
		fileChooser.addChoosableFileFilter(cssFiltro); // aggiunge il filtro alla finestra di apertura
		fileChooser.setFileFilter(cssFiltro); // imposta il filtro xmlFiltro come filtro di default

     	int ritornoValore = fileChooser.showSaveDialog(getContentPane()); // mostra la finestra dialog per il salvataggio del file

     	if (ritornoValore == JFileChooser.APPROVE_OPTION)
     	{
   			try
    			{
	      		File f = fileChooser.getSelectedFile(); // associa all'oggetto f il file selezionato
		 		if (f.exists()) // il file selezionato esiste gia' nella directory corrente
		 		{
		 		     ritornoValore = JOptionPane.showConfirmDialog(getContentPane(), "Si desidera sovrascrivere il file " + f.getName() + "?", "Domanda", JOptionPane.YES_NO_OPTION);
					if (ritornoValore == JOptionPane.YES_OPTION)
						salvareFile = true; // e' stato selezionato di sovrascrivere il file
     				if (ritornoValore == JOptionPane.NO_OPTION)
						salvareFile = false; // e' stato selezionato di non sovrascrivere il file
				}
     			else
     			{
	     			salvareFile = true; // il file non esiste nella directory corrente e viene salvato
     			}

     			if (salvareFile == true) // gestisce il salvataggio del file
     			{
	     			elaboraSalvaCSS(f);

	     			aggiungiIntestazioneCSS(f.getName());

     				stato.setText("File " + f.getName() + " salvato e intestazione aggiornata."); // indica l'avvenuto caricamento

     				//Runtime.getRuntime().exec("cmd /c start "+fileCorrente.getPath());
     			}
     			else // non salva il file
     			{
	     			stato.setText("File " + f.getName() + " non salvato."); // indica l'avvenuto caricamento
     			}
      		}
      		catch(Exception e)
      		{
		     	stato.setText("Errore di salvataggio del file " + fileCorrente.getName() + ".");
	      		System.err.println("Errore di salvataggio del file " + fileCorrente.getName() + ".");
	      	}
  		}
	}


	// **** aggiunge l'intestazione relativa al file CSS elaborato ****

	private void aggiungiIntestazioneCSS(String nomeFileCSS)
	{
		String testoCSS = "<?xml-stylesheet type=\"text/css\" href=\"" + nomeFileCSS + "\"?>\n<documento>\n";

		String testo = areaTesto.getText();
		int posizione = testo.indexOf("<documento>"); // ricerca in testoCompleto la stringa parola a partire da posizioneIniz
		areaTesto.select(posizione, posizione + 11); // seleziona il testo individuato tra le specifiche locazioni
		areaTesto.replaceSelection(testoCSS);
	}


	// **** metodo main del programma ****

	public static void main(String[] args)
	{
		// verifica della versione delle librerie Swing installate sulla macchina

		try
		{
        		String versione = System.getProperty("java.version");

        		if (versione.compareTo("1.1.2") < 0)
        		{
	            	System.out.println("Attenzione: Le librerie Swing devono essere eseguite con una versione 1.1.2 o superiore di Virtual Machine per Java.");
        		}
        	} catch (Throwable t)
        	{
	        	System.out.println("Errore, eccezione rilevata: " + t);
          	t.printStackTrace();
        	}

		// creazione dell'oggetto editor XMLStudio

		XMLStudio XMLStudio = new XMLStudio();
		XMLStudio.setVisible(true);
	}
}