/* ******************************************************************************

     XMLStudio - Editor integrato per documenti XML
     Programma per la Tesi di Laurea di Paolo Guagliumi - pguagliumi@libero.it
     Docente relatore: Prof.ssa Paola Giannini - giannini@di.unito.it

     XmlNodo.java - Definizione classe che estende DefaultMutableTreeNode
    (C) 2002 Paolo Guagliumi - Il programma adotta la licenza GPL allegata

   ****************************************************************************** */


// **** importa i packages utili ****

import javax.swing.tree.*;
import javax.xml.parsers.*;
import org.w3c.dom.*;


// **** definizione classe per rappresentare i nodi Xml nel nostro viewer ****

/* questa classe estende DefaultMutableTreeNode per rappresentare i nodi Xml nel viewer
   che mostra l'albero del documento Xml parsificato. La principale personalizzazione
   della classe e' data dalla ridefinizione del metodo toString che restituisce una
   rappresentazione testuale del nodo in base al suo tipo elemento o testuale */

class XmlNodo extends DefaultMutableTreeNode
{
	// **** costruttore della classe XmlNodo ****

	public XmlNodo(Node node)
	{
		super(node); // l'oggetto node ha tutte le caratteristiche dei nodi DefaultMutableTreeNode
	}

	// **** restituisce da un oggetto XmlNodo il nodo di tipo Node ****

	public Node restituisciXmlNodo()
	{
		Object oggetto = getUserObject(); // restituisce l'oggetto Node memorizzato in questo nodo XmlNode
		if (oggetto instanceof Node)
			return (Node) oggetto; // casting di oggetto in Node
		return null; // altrimenti non restituisce nulla
	}

	// **** restituisce la stringa che individua il contenuto del nodo ****

	public String toString()
	{
		Node nodo = restituisciXmlNodo();
		if (nodo == null) // se l'oggetto e' vuoto
			return getUserObject().toString();
		StringBuffer sb = new StringBuffer();
		switch (nodo.getNodeType())
		{
			case Node.ELEMENT_NODE: // se il nodo e' un elemento includilo tra parentesi graffe
				sb.append('<');
				sb.append(nodo.getNodeName());
				sb.append('>');
				break;
			case Node.TEXT_NODE: // se il nodo e' testuale restituisci il suo valore
				sb.append(nodo.getNodeValue());
				break;
		}
		return sb.toString(); // restituisce la stringa che individua il contenuto del nodo
	}


	// **** aggiunge il nodo figlio al nodo padre ****

	public void aggiungiXmlNodo(XmlNodo nodoFiglio)
	throws Exception
	{
		Node nodoPadre = restituisciXmlNodo(); // restituisce l'oggetto di tipo node selezionato
		if (nodoPadre == null) // il nodo padre non dev'essere nullo
			throw new Exception("Nodo xml corrotto");
		nodoPadre.appendChild(nodoFiglio.restituisciXmlNodo()); // appende il nodo figlio al padre
		add(nodoFiglio);
	}


	// **** rimuove un determinato nodo ****

	public void rimuovi() throws Exception
	{
		Node nodoAttuale = restituisciXmlNodo(); // restituisce l'oggetto di tipo node selezionato
		if (nodoAttuale == null) // il nodo non dev'essere nullo
			throw new Exception("Nodo xml corrotto nella rimozione");
		Node nodoPadre = nodoAttuale.getParentNode(); // individua il nodo genitore del nodo attualmente selezionato
		if (nodoPadre == null) // se nodoPadre e' null allora si sta cercando di rimuovere la radice
			throw new Exception("Non e' possibile rimuovere il nodo radice");
		TreeNode nodoGenitore = getParent(); // restituisce il nodo padre del nodo selezionato
		if (!(nodoGenitore instanceof DefaultMutableTreeNode))
			throw new Exception("Non e' possibile rimuovere il nodo dell'albero");
		nodoPadre.removeChild(nodoAttuale);
		((DefaultMutableTreeNode)nodoGenitore).remove(this);
	}

	// **** modifica il tag di un determinato nodo ****

// 	public void cambia() throws Exception
// 	{
// 		Node nodoAttuale = restituisciXmlNodo(); // restituisce l'oggetto di tipo node selezionato
// 		if (nodoAttuale == null) // il nodo non dev'essere nullo
// 			throw new Exception("Nodo xml corrotto nella rimozione");
// 		//nodoAttuale.setPrefix("nuovoTagNodo");
//		//nodoAttuale.setNodeValue("nuovoTagNodo");
// 		System.out.println("sono qui:3 " + nodoAttuale.getNodeName() + "/" + nodoAttuale.getNodeValue());
// 	}
}