/* ******************************************************************************

     XMLStudio - Editor integrato per documenti XML
     Programma per la Tesi di Laurea di Paolo Guagliumi - pguagliumi@libero.it
     Docente relatore: Prof.ssa Paola Giannini - giannini@di.unito.it

     XmlTag.java - Definizione classe per l'implementazione dell'oggetto XmlTag
    (C) 2002 Paolo Guagliumi - Il programma adotta la licenza GPL allegata

    *  =================================================== *
    *  This program contains code from the book "Swing"    *
    *  2nd Edition by Matthew Robinson and Pavel Vorobiev  *
    *  http://www.spindoczine.com/sbe                      *
    *  =================================================== *

   ****************************************************************************** */


// **** importa i packages utili ****

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;

import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.event.*;

import javax.xml.parsers.*;
import org.w3c.dom.*;

/* **** questa classe consente di definire l'oggetto costituito da un JTree e da una JTable
        che permette l'inserimento dei tag all'interno dei documenti testuali **** */

class XmlTag extends JPanel implements ActionListener
{
	protected JButton nuovotag, apritag, salvatag, salvanometag, aggiunginodoxmltag, editanodoxmltag,
	cancellanodoxmltag, aggiungiattributonodoxmltag, modificaattributonodoxmltag, cancellaattributonodoxmltag,
	cancellaSelezioneJTable; // definizione pulsanti della ToolBar

	public static final String nomeClasseXmlTag = "Tag in formato Xml";

	protected Document documentoXmlTag; // interfaccia che rappresenta l'intero documento xml tag
	protected File fileXmlTagCorrente; // file attualmente visualizzato nell'albero dei tag
	protected JTree alberoXmlTag; // albero che mostra i tag selezionabili dall'utente
	protected DefaultTreeModel modelloAlberoXmlTag; // implementa l'interfaccia TreeModel

	protected DefaultTreeCellEditor editorCellaAlbero; // editor di celle dell'albero per consentire l'editing di nodi di testo
	protected Node nodoAttualmenteEditato = null; // rappresenta il nodo attualmente in fase di editing da parte dell'utente

     protected JTable tavolaAttributiXmlTag; // definisce la JTable per gli attributi dell'albero di parsing
     protected TavolaAttributi modelloTavolaAttributiXmlTag; // definisce un'estensione di AbstractTableModel per implementare la JTable

     XMLStudio oggettoXMLStudioGlobale;


	// **** crea la ToolBar relativa all'oggetto XmlTag ****

	public JToolBar creaToolBar()
	{
		JToolBar barraTag = new JToolBar();
		// barraTag.setFloatable(false);

		nuovotag = new JButton(new ImageIcon(ClassLoader.getSystemResource("./Risorse/nuovo.gif")));
		barraTag.add(nuovotag);
		nuovotag.addActionListener(this);
		nuovotag.setToolTipText("Crea un nuovo file xml relativo ai tag");

		apritag = new JButton(new ImageIcon(ClassLoader.getSystemResource("./Risorse/apri.gif")));
		barraTag.add(apritag);
		apritag.addActionListener(this);
		apritag.setToolTipText("Apre un nuovo file xml relativo ai tag");

		salvatag = new JButton(new ImageIcon(ClassLoader.getSystemResource("./Risorse/salva.gif")));
		barraTag.add(salvatag);
		salvatag.addActionListener(this);
		salvatag.setToolTipText("Salva il file xml relativo ai tag");

		salvanometag = new JButton(new ImageIcon(ClassLoader.getSystemResource("./Risorse/salvanome.gif")));
		barraTag.add(salvanometag);
		salvanometag.addActionListener(this);
		salvanometag.setToolTipText("Salva il file xml relativo ai tag indicandone il nome");

		barraTag.addSeparator();

		barraTag.add(new JLabel("Nodo "));
		aggiunginodoxmltag = new JButton(new ImageIcon(ClassLoader.getSystemResource("./Risorse/creaxml.gif")));
		barraTag.add(aggiunginodoxmltag);
		aggiunginodoxmltag.addActionListener(this);
		aggiunginodoxmltag.setToolTipText("Aggiungi un nuovo tag xml");

		editanodoxmltag = new JButton(new ImageIcon(ClassLoader.getSystemResource("./Risorse/tag.gif")));
		barraTag.add(editanodoxmltag);
		editanodoxmltag.addActionListener(this);
		editanodoxmltag.setToolTipText("Modifica il nodo tag xml");

		cancellanodoxmltag = new JButton(new ImageIcon(ClassLoader.getSystemResource("./Risorse/cancella.gif")));
		barraTag.add(cancellanodoxmltag);
		cancellanodoxmltag.addActionListener(this);
		cancellanodoxmltag.setToolTipText("Cancella il nodo tag xml");

		barraTag.addSeparator();

		barraTag.add(new JLabel("Attributi "));
		aggiungiattributonodoxmltag = new JButton(new ImageIcon(ClassLoader.getSystemResource("./Risorse/creaxml.gif")));
		barraTag.add(aggiungiattributonodoxmltag);
		aggiungiattributonodoxmltag.addActionListener(this);
		aggiungiattributonodoxmltag.setToolTipText("Aggiunge un nuovo attributo al nodo tag xml");

		modificaattributonodoxmltag = new JButton(new ImageIcon(ClassLoader.getSystemResource("./Risorse/tag.gif")));
		barraTag.add(modificaattributonodoxmltag);
		modificaattributonodoxmltag.addActionListener(this);
		modificaattributonodoxmltag.setToolTipText("Modifica il valore dell'attributo del nodo tag xml");

		cancellaattributonodoxmltag = new JButton(new ImageIcon(ClassLoader.getSystemResource("./Risorse/cancella.gif")));
		barraTag.add(cancellaattributonodoxmltag);
		cancellaattributonodoxmltag.addActionListener(this);
		cancellaattributonodoxmltag.setToolTipText("Cancella l'attributo del nodo tag xml");

		cancellaSelezioneJTable = new JButton(new ImageIcon(ClassLoader.getSystemResource("./Risorse/cancellatable.gif")));
		barraTag.add(cancellaSelezioneJTable);
		cancellaSelezioneJTable.addActionListener(this);
		cancellaSelezioneJTable.setToolTipText("Deseleziona le celle degli attributi e valori evidenziati");

		return barraTag;
	}


	// **** metodi invocati quando avviene un action event ****

	public void actionPerformed(ActionEvent e)
	{
		String evento = e.getActionCommand();

		if (e.getSource() == nuovotag)
			azioneNuovoDocumentoXmlTag();
		if (e.getSource() == apritag)
			azioneApriDocumentoXmlTag();
		if (e.getSource() == salvatag)
			azioneSalvaDocumentoXmlTag(false);
		if (e.getSource() == salvanometag)
			azioneSalvaDocumentoXmlTag(true);

		if (e.getSource() == aggiunginodoxmltag)
			azioneAggiungiNodoXmlTag();
		if (e.getSource() == editanodoxmltag)
			azioneEditaNodoXmlTag();
		if (e.getSource() == cancellanodoxmltag)
			azioneCancellaNodo();
		if (e.getSource() == aggiungiattributonodoxmltag)
			azioneAggiungiAttributoNodo();
		if (e.getSource() == modificaattributonodoxmltag)
			azioneModificaAttributoNodo();
		if (e.getSource() == cancellaattributonodoxmltag)
			azioneEliminaAttributoNodo();

		if (e.getSource() == cancellaSelezioneJTable)
			azioneCancellaSelezioneJTable();
	}


	// **** costruttore della classe ****

	public XmlTag(XMLStudio oggettoXMLStudio) //final String fileXmlTagCorrente
	{
 		// System.out.println("Valore fileXmlTagCorrente:" + oggettoXMLStudio.getfileXmlTagCorrente());

		this.setLayout(new BorderLayout()); // imposta il layout dell'oggetto XmlTag

		JToolBar barraTag = creaToolBar(); // crea la JToolBar relativa all'oggetto XmlTag
		this.add(barraTag, BorderLayout.NORTH); // posiziona la JToolBar

		// crea un nodo generico che non ha ne' padre, ne' figli, ma che consente di avere figli
		DefaultMutableTreeNode nodoRadiceXmlTag = new DefaultMutableTreeNode("Nessun tag xml");
		// crea un albero, passandovi la radice, in cui ogni nodo ha dei figli
		modelloAlberoXmlTag = new DefaultTreeModel(nodoRadiceXmlTag);
		// restituisce un'istanza di JTree che mostra il nodo radice e l'albero e' creato usando uno specifico modello dati
		alberoXmlTag = new JTree(modelloAlberoXmlTag);
		// imposta la selezione dell'albero a SINGLE_TREE_SELECTION: consente di selezionare solo un nodo dell'albero per volta
		alberoXmlTag.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
		alberoXmlTag.setShowsRootHandles(true); // gli handles sono mostrati
		//alberoXmlTag.setEditable(false); // imposta l'albero JTree come non editabile

		RenderAlbero render = new RenderAlbero(); // definisce un nuovo oggetto render di tipo RenderAlbero
	     alberoXmlTag.setCellRenderer(render); // applica all'albero JTree il render con le associazioni di icone e tooltip relative

	     // **** definizione di una sottoclasse editor di celle personalizzato per consentire l'editing dei nodi **** ooo
	     // costruisce un oggetto DefaultTreeCellEditor per un JTree utilizzando lo specifico renderer applicato all'alberoXmlTag

	     editorCellaAlbero = new DefaultTreeCellEditor(alberoXmlTag, render)
		{
			// il realeditor che gestisce l'editing ritorna true se l'editing puo' essere iniziato

			public boolean isCellEditable(EventObject event)
			{
				Node nodo = restituisciNodoSelezionato();
				if (nodo != null && nodo.getNodeType() == Node.TEXT_NODE)
					return super.isCellEditable(event);
				else
					return false;
			}

			// configura l'editor passandovi l'albero da editare, il valore della cella da editare, se la cella e' selezionata, se il nodo e' espanso, se il nodo e' foglia, l'indice di riga del nodo editato

			public Component getTreeCellEditorComponent(JTree albero, Object valore, boolean selezionato, boolean espanso, boolean foglia, int riga)
			{
				if (valore instanceof XmlNodo)
					nodoAttualmenteEditato = ((XmlNodo)valore).restituisciXmlNodo();
				return super.getTreeCellEditorComponent(albero, valore, selezionato, espanso, foglia, riga);
			}
		};

		editorCellaAlbero.addCellEditorListener(new XmlEditorRileva());
		alberoXmlTag.setCellEditor(editorCellaAlbero);
 		alberoXmlTag.setEditable(true);
		alberoXmlTag.setInvokesStopCellEditing(true);

		JScrollPane scorreAlberoXmlTag = new JScrollPane(); // viene creato uno scrollpane per l'albero JTree che visualizza il documento Xml Tag
		scorreAlberoXmlTag.getViewport().add(alberoXmlTag); // l'albero e' posizionato in uno scrollpane
		scorreAlberoXmlTag.setToolTipText("Rappresentazione ad albero del documento tag xml");

		/* modelloTavolaAttributiXmlTag e' un oggetto di tipo TavolaAttributi con campi nodoCorrente e
		   attributiCorrenti ed implementa TableModel, interfaccia usata da JTable per interrogare
		   un modello di dati ti tipo tabulare. Sono sufficienti due righe di codice per permettere
		   alla JTable di visualizzare qualsiasi modello dati che implementi TableModel */

		modelloTavolaAttributiXmlTag = new TavolaAttributi(); // nuovo oggetto TavolaAttributi che estende AbstractTableModel
		tavolaAttributiXmlTag = new JTable(modelloTavolaAttributiXmlTag); // assegna alla JTable l'interfaccia specifica appena creata

		JScrollPane scorreTavolaXmlTag = new JScrollPane(tavolaAttributiXmlTag); // consente lo scorrimento di tavolaAttributiXmlTag
		scorreTavolaXmlTag.getViewport().setBackground(tavolaAttributiXmlTag.getBackground()); // imposta il background

		JSplitPane splitAlberoXmlTag = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, scorreAlberoXmlTag, scorreTavolaXmlTag);
		splitAlberoXmlTag.setDividerLocation(500);
		//splitAlbero.setDividerSize(30);
		splitAlberoXmlTag.setContinuousLayout(true);
		splitAlberoXmlTag.setOneTouchExpandable(true); // fornisce simbolo per rapida esapansione-chiusura splitPane
		this.add(splitAlberoXmlTag, BorderLayout.CENTER);

		/* Se si e' interessati nel sapere quando la selezione cambia, bisogna implementare l'interfaccia
		   TreeSelectionListener ed aggiungere l'istanza usando il metodo addTreeSelectionListener.
		   valueChanged sara' invocato (una volta sola) quando cambia la selezione ovvero quando l'utente
		   clicca due volte consecutive sullo stesso nodo */

		// e' un listener che e' modificato quando cambia la selezione in un TreeSelectionModel*/
 		TreeSelectionListener listenerAlberoXmlTag = new TreeSelectionListener() // per rilevare quando l'utente seleziona un nodo nell'albero ooo
		{
			public void valueChanged(TreeSelectionEvent e) // richiamato ogni qualvolta cambia la selezione di un valore
			{
				Node nodoselezionatoXmlTag = restituisciNodoSelezionato();

				// aggiorna l'albero con il nodo attualmente selezionato
				aggiornaAttributiNodoInTavola(nodoselezionatoXmlTag); // aggiorna l'albero xml con il nodo attualmente selezionato
				abilitaPulsantiNodi(); // rileva se abilitare o disabilitare i pulsanti dei nodi
				abilitaPulsantiAttributi(); // rileva se abilitare o disabilitare i pulsanti degli attributi
			}
		};

		ListSelectionListener listenerSelezione = new ListSelectionListener() // questo listener riceve notifica quando cambia il valore della selezione di una cella
		{
			public void valueChanged(ListSelectionEvent e)
			{
				abilitaPulsantiAttributi(); // rileva se abilitare o disabilitare i pulsanti degli attributi
			}
		};
		tavolaAttributiXmlTag.getSelectionModel().addListSelectionListener(listenerSelezione); // aggiunge il listener alla JTable

		alberoXmlTag.addTreeSelectionListener(listenerAlberoXmlTag); // assegna all'alberoXmlTag il listener definito per eventi TreeSelection
		caricaDocumentoXmlTag(oggettoXMLStudio.getfileXmlTagCorrente()); // carica il file xml di default per i tag
		oggettoXMLStudioGlobale = oggettoXMLStudio;
	}


	// **** carica l'attuale nodo selezionato e lo restituisce come istanza di XmlNodo ****

	public XmlNodo restituisciNodoSelezionatoAlbero()
	{
		TreePath path = alberoXmlTag.getSelectionPath(); // restituisce il path del nodo selezionato grazie a getSelectionPath
		if (path == null) // restituisce null se il path ottenuto e' null
			return null;
		Object oggetto = path.getLastPathComponent(); // in oggetto restituisce l'ultimo componente che ha determinato questo path
		if (!(oggetto instanceof XmlNodo)) // se l'oggetto non e' istanza di XmlNodo ritorna null
			return null;
		return (XmlNodo)oggetto; // se l'oggetto che ha determinato l'ultimo path e' di tipo XmlNodo, restituiscilo con un casting
	}


	// **** restituisce l'attuale nodo selezionato richiamando restituisciNodoSelezionatoAlbero ****

	public Node restituisciNodoSelezionato()
	{
		XmlNodo nodoAlberoXmlTag = restituisciNodoSelezionatoAlbero(); // in nodoAlberoXml viene messo il nodo attualmente selezionato
		if (nodoAlberoXmlTag == null)
			return null;
		return nodoAlberoXmlTag.restituisciXmlNodo(); // restituisce da un oggetto XmlNodo il nodo di tipo Node
	}


	// **** aggiorna l'albero xml con il nodo attualmente selezionato ****

	public void aggiornaAttributiNodoInTavola(Node nodo)
	{
		/* modelloTavolaAttributiXmlTag e' un oggetto di tipo TavolaAttributi con campi nodoCorrente e
		   attributiCorrenti ed implementa TableModel, interfaccia usata da JTable per interrogare
		   un modello di dati ti tipo tabulare */

		modelloTavolaAttributiXmlTag.assegnaNodo(nodo);

		/* TableModelEvent e' usato per notificare i listener che il modello della tavola e' cambiato
		   Tutte le righe contenenti dati nella tavola sono cambiate, i listener devono perdere ogni stato
		   basato sulle precedenti righe e richiedere al TableModel il nuovo numero di linee e di valori */

		tavolaAttributiXmlTag.tableChanged(new TableModelEvent(modelloTavolaAttributiXmlTag));
	}


	// **** dato il nodo radice crea l'oggetto alberoNodi di tipo XmlNodo che estende DefaultMutableTreeNode ****

	private DefaultMutableTreeNode creaAlberoNodi(Node nodoRadice) // dato un nodo crea la gerarchia di nodi ad alber
	{
		if (!possoVisualizzareNodo(nodoRadice)) // se non puo' visualizzare la radice, ritorna null
		{
			// stato.setText("Analisi del file xml: non posso visualizzare la radice.");
			return null;
		}

		XmlNodo alberoNodi = new XmlNodo(nodoRadice); // crea un oggetto alberoNodi di tipo XmlNodo per rappresentare la radice

		NodeList listaNodi = nodoRadice.getChildNodes(); // listaNodi e' una collezione ordinata di nodi: rappresenta tutti i nodi figli del nodo radice

		// per ogni nodo figlio viene creato un albero di nodi richiamando ricorsivamente creaAlberoNodi
		for (int k = 0; k < listaNodi.getLength(); k++) // getLength restituisce il numero di nodi nella lista
		{
			Node nodo = listaNodi.item(k); // ritorna il k-esimo nodo selezionato
			DefaultMutableTreeNode nodoFiglio = creaAlberoNodi(nodo); // chiamata ricorsiva del metodo passandogli il nodo e assumendolo come figlio
			if (nodoFiglio != null)
				alberoNodi.add(nodoFiglio); // aggiunge all'oggetto alberoNodi il nodo figlio esaminato
		}
		return alberoNodi; // alla fine restituisci l'oggetto alberoNodi creato
	}


	// **** metodo responsabile di espandere ogni nodo a partire dalla radice affinche' sia visibile l'albero nel viewer ****

	private static void espandiAlbero(JTree alberoJTree) // riceve come parametro alberoXml da espandere
	{
	    // oggettoRadice e' nodo dell'albero in un JTree
	    // getModel restituisce il modello di tipo TreeModel usato per i dati
	    // getRoot restituisce il nodo radice
	    TreeNode nodoRadice = (TreeNode)alberoJTree.getModel().getRoot(); // individua il nodo radice di questo albero
	    TreePath path = new TreePath(nodoRadice); // individua il path della radice di questo albero
	    for (int k = 0; k < nodoRadice.getChildCount(); k++) // per ogni nodo figlio della radice fai
	    {
			TreeNode nodoFiglio = (TreeNode)nodoRadice.getChildAt(k); // individua il k-esimo nodo figlio della radice
			espandiAlbero(alberoJTree, path, nodoFiglio); // espandi l'albero dati il Jtree, il path della radice e il nodoFiglio
	    }
	}


	// **** metodo responsabile di espandere ogni nodo affinche' sia visibile l'albero nel viewer ****

	private static void espandiAlbero(JTree alberoJTree, TreePath path, TreeNode nodo)
	{
		if (path == null || nodo == null) // se il path del nodo radice e' nullo o il nodo radice e' nullo
			return;				    // esci dal metodo
		alberoJTree.expandPath(path); // dato il path, assicura che il nodo identificato dal path specificato sia espanso e visibile
		TreePath nuovoPath = path.pathByAddingChild(nodo); // restituisce un nuovo path contenente tutti gli elementi del nodo padre e del nodo figlio
		for (int k = 0; k < nodo.getChildCount(); k++) // per ogni nodo figlio di nodo (che e' gia' un nodo interno dell'albero) fai
		{
			TreeNode nodoFiglio = (TreeNode)nodo.getChildAt(k); // individua il k-esimo nodo figlio del nodo considerato
			if (nodoFiglio != null) // se nodoFiglio individuato non e' null allora espandi quella porzione dell'albero
			{
				espandiAlbero(alberoJTree, nuovoPath, nodoFiglio); // alberoJTree, il path del nodo padre, il nodo figlio
			}
		}
	}


	// **** verifica se il nodo (radice) e' un elemento o un nodo testuale ****

	private boolean possoVisualizzareNodo(Node nodocorrente) // se il nodo non e' elemento o testuale non viene visualizzato nell'albero
	{
		switch (nodocorrente.getNodeType())  // estrapola il tipo del nodo attualmente esaminato
		{
			case Node.ELEMENT_NODE: // il nodo e' di tipo elemento nel file xml
				return true;
			case Node.TEXT_NODE: // il nodo e' di tipo testuale
				String testonodo = nodocorrente.getNodeValue().trim(); // essendo il nodo testuale, preleva il suo valore ovvero del testo
				return !(testonodo.equals("") || testonodo.equals("\n") || testonodo.equals("\r\n")); // ritorna true o false dopo i confronti
		}
		return false;
	}


	// **** metodo per la creazione di un nuovo documento xml che contiene i tag da inserire ****

	protected void azioneNuovoDocumentoXmlTag()
	{
		azioneRicordaSalva(); // chiede all'utente se salvare il file tag (xml) attualmente caricato

		String nomeNodoTag = (String)JOptionPane.showInputDialog(this, "Inserire il nome del nodo radice del nuovo documento tag (.xml)", "Inserimento nome nodo", JOptionPane.PLAIN_MESSAGE);
		if (nomeNodoTag == null) // se e' stato premuto il pulsante annulla, esci
			return;
		if (nomeXmlValido(nomeNodoTag))
		{
			oggettoXMLStudioGlobale.cursoreClessidra(); // imposta il cursore clessidra in tutta l'applicazione
			try
			{
				// DocumentBuilderFactory definisce API che consentono ad applicazioni di ottenere un parser che produce oggetti DOM ad albero da documenti XML
				DocumentBuilderFactory docBuilderFactory = DocumentBuilderFactory.newInstance(); // crea una nuova istanza di DocumentBuilderFactory
				// DocumentBuilder e' usato per ottenere un'istanza di documento DOM da un documento XML
				DocumentBuilder docBuilder = docBuilderFactory.newDocumentBuilder(); // crea una nuova istanza di DocumentBuilder usando i parametri definiti
				documentoXmlTag = docBuilder.newDocument(); // ottiene una nuova istanza di un oggetto document DOM per la costruzione dell'albero DOM

				Element nodoRadice = documentoXmlTag.createElement(nomeNodoTag); // crea un elemento del tipo specificato dall'input
				nodoRadice.normalize(); // assicura che la vista del documento DOM sia la stessa come se fosse salvato e caricato
				documentoXmlTag.appendChild(nodoRadice); // aggiunge il nodoRadice alla fine della lista dei nodi figli di questo nodo

				DefaultMutableTreeNode topAlbero = creaAlberoNodi(nodoRadice); // dato il nodo radice crea l'albero

				modelloAlberoXmlTag.setRoot(topAlbero); // imposta topAlbero come radice
				alberoXmlTag.treeDidChange(); // impostato quando il JTree e' cambiato a tal punto da necessitare il ridimensionamento
				espandiAlbero(alberoXmlTag); // espande l'albero JTree legato al documento Xml affinche' sia visibile nel viewer
				aggiornaAttributiNodoInTavola(null); // imposta a null gli attributi nella tabella dei nodi
			}
			catch (Exception e)
			{
				JOptionPaneEsteso mostraEccezione = new JOptionPaneEsteso(oggettoXMLStudioGlobale, "Informazione", "Eccezione rilevata nella crazione del nuovo documento tag (.xml): " + e);
				oggettoXMLStudioGlobale.stato.setText("Errore nella creazione del documento tag (.xml)");
				oggettoXMLStudioGlobale.cursoreNormale(); // imposta il cursore normale in tutta l'applicazione
			}
			finally
			{
				oggettoXMLStudioGlobale.cursoreNormale(); // imposta il cursore normale in tutta l'applicazione
			}
		}
		else
		{
			JOptionPane.showInternalMessageDialog(oggettoXMLStudioGlobale.getContentPane(), "E' stato inserito un nome di tag relativo al nodo radice non valido.", "Informazione", JOptionPane.ERROR_MESSAGE);
		}
	}

	// **** metodo per l'apertura di un documento xml che contiene i tag da inserire ****

	protected void azioneApriDocumentoXmlTag()
	{
		Thread runner = new Thread() // uso dei thread per consentire il cambio del cursore in clessidra durante il caricamento
		{
			public void run()
			{
				azioneRicordaSalva(); // chiede all'utente se salvare il file tag (xml) attualmente caricato
				//JFileChooser fileChooser = new JFileChooser(directoryCorrente); // crea un nuovo oggetto di tipo JFileChooser
				JFileChooser fileChooser = new JFileChooser("./Tag/"); // crea un nuovo oggetto di tipo JFileChooser
				fileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY); // consenti la selezione solo di file
				FiltroFile xmlFiltro = new FiltroFile("xml", "File xml *.XML"); // imposta il filtro per i file .xml (che rappresentano i tag)
				fileChooser.addChoosableFileFilter(xmlFiltro); // aggiunge il filtro alla finestra di apertura
				fileChooser.setFileFilter(xmlFiltro); // imposta il filtro xmlFiltro come filtro di default

				int ritornoValore = fileChooser.showOpenDialog(oggettoXMLStudioGlobale.getContentPane()); // mostra la finestra dialog per l'apertura del file

		    		if (ritornoValore == JFileChooser.APPROVE_OPTION)
    				{
					File f = fileChooser.getSelectedFile(); // associa all'oggetto f il file selezionato
					if (f == null || !f.isFile()) // se f e' nullo o non e' un file allora esci dal metodo
						return;

					oggettoXMLStudioGlobale.cursoreClessidra(); // imposta il cursore clessidra in tutta l'applicazione
					try
					{
						// DocumentBuilderFactory definisce API che consentono ad applicazioni di ottenere un parser che produce oggetti DOM ad albero da documenti XML
						DocumentBuilderFactory docBuilderFactory = DocumentBuilderFactory.newInstance(); // crea una nuova istanza di DocumentBuilderFactory
						// DocumentBuilder e' usato per ottenere un'istanza di documento DOM da un documento XML
						DocumentBuilder docBuilder = docBuilderFactory.newDocumentBuilder(); // crea una nuova istanza di DocumentBuilder usando i parametri definiti
						documentoXmlTag = docBuilder.parse(f); // parsifica il contenuto di un dato file come un documento XML e restituisce un nuovo oggetto documento DOM

						Element nodoRadice = documentoXmlTag.getDocumentElement(); // individua il nodo radice dal documento Xml
						nodoRadice.normalize(); // assicura che la vista del documento DOM sia la stessa come se fosse salvato e caricato

						DefaultMutableTreeNode nodoRadiceXmlTag = creaAlberoNodi(nodoRadice);

						modelloAlberoXmlTag.setRoot(nodoRadiceXmlTag); // imposta nodoRadiceXmlTag come radice
						alberoXmlTag.treeDidChange(); // impostato quando il JTree e' cambiato a tal punto da necessitare il ridimensionamento
						espandiAlbero(alberoXmlTag); // espande l'albero JTree legato al documento Xml affinche' sia visibile nel viewer
						aggiornaAttributiNodoInTavola(null); // imposta a null gli attributi nella tabella dei nodi
						fileXmlTagCorrente = f;
						oggettoXMLStudioGlobale.stato.setText("File " + f.getName() + " caricato."); // indica l'avvenuto caricamento
					}
					catch (Exception e)
					{
						JOptionPaneEsteso mostraEccezione = new JOptionPaneEsteso(oggettoXMLStudioGlobale, "Informazione", "Eccezione rilevata nella creazione dell'albero: " + e + ". Il documento xml analizzato non e' ben formato.");
						oggettoXMLStudioGlobale.stato.setText("Errore di apertura del file " + f.getName());
	      				System.err.println("Errore di apertura del file " + f.getName());
						oggettoXMLStudioGlobale.cursoreNormale(); // reimposta il cursore predefinito
					}
					finally
					{
						oggettoXMLStudioGlobale.cursoreNormale(); // reimposta il cursore predefinito
					}
				}
			}
		};
		runner.start();
	}


	// **** metodo per il caricamento automatico del file xml che contiene i tag da inserire ****

	protected void caricaDocumentoXmlTag(final String fileXmlTag)
	{
		Thread runner = new Thread() // uso dei thread per consentire il cambio del cursore in clessidra durante il caricamento
		{
			public void run()
			{
				if (fileXmlTag == null) // se non e' presente il file di default dei tag xml da caricare
					return;

				File f = new File(fileXmlTag); // associa all'oggetto f il file selezionato
				if (f == null || !f.isFile()) // se f e' nullo o non e' un file allora esci dal metodo
				{
					JOptionPane.showInternalMessageDialog(oggettoXMLStudioGlobale.getContentPane(), "Non e' stato caricato il file xml relativo ai tag.", "Informazione", JOptionPane.ERROR_MESSAGE);
					return;
				}

				oggettoXMLStudioGlobale.cursoreClessidra(); // imposta il cursore clessidra in tutta l'applicazione
				try
				{
					// DocumentBuilderFactory definisce API che consentono ad applicazioni di ottenere un parser che produce oggetti DOM ad albero da documenti XML
					DocumentBuilderFactory docBuilderFactory = DocumentBuilderFactory.newInstance(); // crea una nuova istanza di DocumentBuilderFactory
					// DocumentBuilder e' usato per ottenere un'istanza di documento DOM da un documento XML
					DocumentBuilder docBuilder = docBuilderFactory.newDocumentBuilder(); // crea una nuova istanza di DocumentBuilder usando i parametri definiti
					documentoXmlTag = docBuilder.parse(f); // parsifica il contenuto di un dato file come un documento XML e restituisce un nuovo oggetto documento DOM

					Element nodoRadice = documentoXmlTag.getDocumentElement(); // individua il nodo radice dal documento Xml
					nodoRadice.normalize(); // assicura che la vista del documento DOM sia la stessa come se fosse salvato e caricato

					DefaultMutableTreeNode nodoRadiceXmlTag = creaAlberoNodi(nodoRadice);

					modelloAlberoXmlTag.setRoot(nodoRadiceXmlTag); // imposta nodoRadiceXmlTag come radice
					alberoXmlTag.treeDidChange(); // impostato quando il JTree e' cambiato a tal punto da necessitare il ridimensionamento
					espandiAlbero(alberoXmlTag); // espande l'albero JTree legato al documento Xml affinche' sia visibile nel viewer
					aggiornaAttributiNodoInTavola(null); // imposta a null gli attributi nella tabella dei nodi
					fileXmlTagCorrente = f;
					oggettoXMLStudioGlobale.stato.setText("File " + f.getName() + " precaricato per tag xml."); // indica l'avvenuto caricamento
				}
				catch (Exception e)
				{
					JOptionPaneEsteso mostraEccezione = new JOptionPaneEsteso(oggettoXMLStudioGlobale, "Informazione", "Eccezione rilevata nella creazione dell'albero: " + e + ". Il documento xml analizzato non e' ben formato.");
					oggettoXMLStudioGlobale.cursoreNormale(); // reimposta il cursore predefinito
				}
				finally
				{
					oggettoXMLStudioGlobale.cursoreNormale(); // reimposta il cursore predefinito
				}
			}
		};
		runner.start();
	}


	// **** metodo per il salvataggio effettivo del file xml che contiene i tag da inserire ****

	protected void azioneSalvaGenerica(File filesalva)
	{
		boolean salvareFile = false;

		if (filesalva.exists()) // il file selezionato esiste gia' nella directory corrente
		{
		    	int ritornoValore = JOptionPane.showConfirmDialog(oggettoXMLStudioGlobale.getContentPane(), "Si desidera sovrascrivere il file " + filesalva.getName() + "?", "Domanda", JOptionPane.YES_NO_OPTION);
			if (ritornoValore == JOptionPane.YES_OPTION)
				salvareFile = true; // e' stato selezionato di sovrascrivere il file
			if (ritornoValore == JOptionPane.NO_OPTION)
				salvareFile = false; // e' stato selezionato di non sovrascrivere il file
		}
		else
		{
			salvareFile = true; // il file non esiste nella directory corrente e viene salvato
		}

		if (salvareFile == true) // gestisce il salvataggio del file
		{
			oggettoXMLStudioGlobale.cursoreClessidra(); // imposta il cursore clessidra in tutta l'applicazione
			fileXmlTagCorrente = filesalva; // in fileCorrente salva i dati relativi al fine appena salvato per successivo uso
			try // scrive su file l'albero relativo ai tag xml
 			{
 				FileWriter out = new FileWriter(filesalva);
 				XMLRoutine.writeDocumento(documentoXmlTag, out);
 				out.close();
 				CopiaCorretta copia = new CopiaCorretta(filesalva);
 				oggettoXMLStudioGlobale.stato.setText("File " + filesalva.getName() + " salvato."); // indica l'avvenuto salvataggio
 			}
 			catch (Exception e)
 			{
	 			JOptionPaneEsteso mostraEccezione = new JOptionPaneEsteso(oggettoXMLStudioGlobale, "Informazione", "Eccezione rilevata nel salvataggio del file " + fileXmlTagCorrente.getName() + ". Eccezione: " + e);
	 			oggettoXMLStudioGlobale.cursoreNormale(); // reimposta il cursore predefinito
	 			oggettoXMLStudioGlobale.stato.setText("Errore di salvataggio del file " + filesalva.getName() + "."); // indica l'avvenuto salvataggio
	 			// JOptionPane.showInternalMessageDialog(oggettoXMLStudioGlobale.getContentPane(), "Salvato e correntemente usato il file xml tag" + fileXmlTagCorrente.getName(), "Informazione", JOptionPane.INFORMATION_MESSAGE);
	 			// JOptionPane.showInternalMessageDialog(XmlTag.this, "Salvato e correntemente usato il file xml tag" + fileXmlTagCorrente.getName(), "Informazione", JOptionPane.INFORMATION_MESSAGE);
  			}
			oggettoXMLStudioGlobale.cursoreNormale(); // reimposta il cursore predefinito
		}
		else
		{
			oggettoXMLStudioGlobale.stato.setText("File " + filesalva.getName() + " non salvato."); // indica l'avvenuto salvataggio
		}
	}


	// **** metodo per il savaltaggio del file xml che contiene i tag da inserire ****

	protected void azioneSalvaDocumentoXmlTag(boolean salvaNome)
	{

		if (documentoXmlTag != null) // e' attualmente caricato un documento relativo a tag xml
 		{
			if (salvaNome == true) // si desidera salvare il file xml tag assegnandogli un nuovo nome
			{
				try
				{
					JFileChooser fileChooser = new JFileChooser("./Tag/"); // crea l'oggetto per l'apertura del file sulla directory corrente

					fileChooser.setApproveButtonText("Salva"); // imposta il valore del bottone di esecuzione
					fileChooser.setDialogTitle("Salva file xml tag"); // imposta il titolo della finestra Dialog

					// imposta ed aggiunge i filtri opportuni
					fileChooser.resetChoosableFileFilters(); // reinizializza l'oggetto fileChooser
					FiltroFile xmlFiltro = new FiltroFile("xml", "File xml *.XML"); // imposta il filtro per i file .xml
					fileChooser.addChoosableFileFilter(xmlFiltro); // aggiunge il filtro alla finestra di apertura
					fileChooser.setFileFilter(xmlFiltro); // imposta il filtro xmlFiltro come filtro di default

			     	int ritornoValore = fileChooser.showSaveDialog(oggettoXMLStudioGlobale.getContentPane()); // mostra la finestra dialog per il salvataggio del file

			     	if (ritornoValore == JFileChooser.APPROVE_OPTION)
			     	{
				     	File f = fileChooser.getSelectedFile(); // associa all'oggetto f il file selezionato
				     	azioneSalvaGenerica(f);
			 		}
		 		}
		 		catch (Exception e)
		 		{
					JOptionPaneEsteso mostraEccezione = new JOptionPaneEsteso(oggettoXMLStudioGlobale, "Informazione", "Eccezione rilevata nel salvataggio del file " + fileXmlTagCorrente.getName() + ". Eccezione: " + e);
		 		}
			}
			else // si desidera sovrascrivere il file xml tag senza assegnargli un nuovo nome
			{
				if (fileXmlTagCorrente.isFile() == true)
				{
	  				try
	  				{
		  				azioneSalvaGenerica(fileXmlTagCorrente);
      				}
      				catch(Exception e)
      				{
	      				JOptionPaneEsteso mostraEccezione = new JOptionPaneEsteso(oggettoXMLStudioGlobale, "Informazione", "Eccezione rilevata nel salvataggio del file " + fileXmlTagCorrente.getName() + ". Eccezione: " + e);
	     			}
     			}
     		}
		}
	}


	// **** metodo per ricordare di salvare il file xml tag attualmente aperto ****

	protected void azioneRicordaSalva()
	{
		if (oggettoXMLStudioGlobale.getfileXmlTagCorrente() != null)
		{
			int ritornoValore = JOptionPane.showConfirmDialog(oggettoXMLStudioGlobale.getContentPane(), "Si desidera salvare il file corrente " + fileXmlTagCorrente.getName() + " relativo ai tag xml?", "Domanda", JOptionPane.YES_NO_OPTION);
			if (ritornoValore == JOptionPane.YES_OPTION)
				azioneSalvaDocumentoXmlTag(false); // salva il file senza associare un nuovo nome
		}
	}


	// **** metodo per l'aggiunta di un nodo tag all'albero xml tag ****

	protected void azioneAggiungiNodoXmlTag()
	{
		if (documentoXmlTag == null) // se non fosse caricato nessun documento xml relativo ai tag
			return;

		XmlNodo nodoSelezionato = restituisciNodoSelezionatoAlbero(); // in nodoSelezionato vi e' il nodo attualmente selezionato
		if (nodoSelezionato == null) // se non e' selezionato nessun nodo
			return;

		Node nodoGenitoreNodoTag = nodoSelezionato.restituisciXmlNodo(); // e' effettivamente stato selezionato un nodo
		if (nodoGenitoreNodoTag == null) // se non e' null, prosegui
			return;

		String nomeTag = (String)JOptionPane.showInputDialog(this, "Inserire il nome del nuovo tag xml", "Inserimento tag", JOptionPane.PLAIN_MESSAGE);
		if (nomeTag == null) // se e' stato premuto il pulsante annulla, esci
			return;
		if (nomeXmlValido(nomeTag))
		{
			try
			{
				Element nuovoElementoTag = documentoXmlTag.createElement(nomeTag); // specificando un nomeTag crea un elemento per il documento xml
				XmlNodo nuovoTagNodo = new XmlNodo(nuovoElementoTag); // crea un nodo di tipo XmlNodo
				nodoSelezionato.aggiungiXmlNodo(nuovoTagNodo); // aggiunge il nodo figlio nuovoTagNodo al nodo preesistente padre

				modelloAlberoXmlTag.nodeStructureChanged(nodoSelezionato); // necessario per mostrare il nuovo nodo
				TreePath path = alberoXmlTag.getSelectionPath(); // in path vi e' il cammino del nodo attualmente selezionato
				if (path != null)
				{
					path = path.pathByAddingChild(nuovoTagNodo);
					alberoXmlTag.setSelectionPath(path); // seleziona il nodo specificato dal path indicato
					alberoXmlTag.scrollPathToVisible(path); // espande i cammini dei componenti sino all'attuale che e' mostrato
				}
			}
			catch (Exception e)
			{
				JOptionPaneEsteso mostraEccezione = new JOptionPaneEsteso(oggettoXMLStudioGlobale, "Informazione", "Eccezione rilevata nell'aggiunta del nodo tag xml: " + e);
			}
		}
		else
		{
			JOptionPane.showInternalMessageDialog(oggettoXMLStudioGlobale.getContentPane(), "E' stato inserito un nome di tag non valido.", "Informazione", JOptionPane.ERROR_MESSAGE);
		}
	}


	// **** metodo per la modifica (editing) di un nodo xml tag ****

	protected void azioneEditaNodoXmlTag()
	{
		TreePath path = alberoXmlTag.getSelectionPath(); // in path vi e' il cammino del nodo attualmente selezionato
		XmlNodo nodoAlberoPadre = restituisciNodoSelezionatoAlbero(); // in nodoAlberoPadreXml viene messo il nodo attualmente selezionato
		if (nodoAlberoPadre == null)
			return;
		Node nodo = nodoAlberoPadre.restituisciXmlNodo(); // restituisce il nodo corrente di tipo node dal nodo di tipo XmlNodo
		if (nodo == null)
			return;
		try
		{
			switch (nodo.getNodeType()) // estrapola il tipo del nodo attualmente esaminato
			{
				case Node.ELEMENT_NODE: // il nodo e' di tipo element

					// individua il nodo testuale figlio per editarlo
					for (int k=0; k < nodoAlberoPadre.getChildCount(); k++) // per ogni nodo figlio della radice fai
					{
						XmlNodo childNode = (XmlNodo)nodoAlberoPadre.getChildAt(k); // individua il k-esimo nodo figlio della radice
						Node nd = childNode.restituisciXmlNodo(); // restituisce il nodo corrente di tipo node dal nodo di tipo XmlNodo
						if (nd instanceof Text)
						{
							path = path.pathByAddingChild(childNode);
							alberoXmlTag.setSelectionPath(path); // seleziona il nodo specificato dal path indicato
							alberoXmlTag.scrollPathToVisible(path); // espande i cammini dei componenti sino all'attuale che e' mostrato
							alberoXmlTag.startEditingAtPath(path); // seleziona il nodo identificato dal path ed inizia il suo editing
							return; // esce
						}
					}
					// non ha individuato il nodo testuale figlio, allora crea un nuovo nodo testuale figlio
					Text testo = documentoXmlTag.createTextNode(""); // crea un nodo testuale con testo vuoto
					XmlNodo nodoFiglio = new XmlNodo(testo); // crea un nuovo nodo di tipo XmlNodo
					nodoAlberoPadre.aggiungiXmlNodo(nodoFiglio); // aggiunge il nodo figlio al nodo padre
					modelloAlberoXmlTag.nodeStructureChanged(nodoAlberoPadre);
					path = path.pathByAddingChild(nodoFiglio);
					alberoXmlTag.setSelectionPath(path); // seleziona il nodo specificato dal path indicato
					alberoXmlTag.scrollPathToVisible(path); // espande i cammini dei componenti sino all'attuale che e' mostrato
					alberoXmlTag.startEditingAtPath(path); // seleziona il nodo identificato dal path ed inizia il suo editing
					return; // esce
				case Node.TEXT_NODE: // il nodo e' di tipo testuale
					alberoXmlTag.startEditingAtPath(path); // seleziona il nodo identificato dal path ed inizia il suo editing
					return; // esce
			}
		}
		catch (Exception e)
		{
			JOptionPaneEsteso mostraEccezione = new JOptionPaneEsteso(oggettoXMLStudioGlobale, "Informazione", "Eccezione rilevata nella modifica del nodo tag xml: " + e);
		}
	}


	// **** metodo per la cancellazione di un nodo xml tag ****

	protected void azioneCancellaNodo()
	{
		TreePath path = alberoXmlTag.getSelectionPath(); // restituisce il path del nodo selezionato grazie a getSelectionPath
		XmlNodo nodoAlbero = restituisciNodoSelezionatoAlbero(); // in nodoAlbero viene messo il nodo attualmente selezionato
		if (nodoAlbero != null)
		{
			Node nodo = nodoAlbero.restituisciXmlNodo(); // restituisce il nodo corrente di tipo node dal nodo di tipo XmlNodo
			if (nodo != null)
			{
				int nomeAttributo = JOptionPane.showConfirmDialog(oggettoXMLStudioGlobale.getContentPane(), "Si desidera cancellare il nodo " + nodo.getNodeName() + "?", "Domanda", JOptionPane.YES_NO_OPTION);
				if (nomeAttributo == JOptionPane.YES_OPTION)
				{
					try
					{
						TreeNode nodoPadre = nodoAlbero.getParent(); // determina il nodo padre del nodo di tipo XmlNodo nodoAlbero
						nodoAlbero.rimuovi(); // rimuove il nodoAlbero e posiziona la selezione sul nodo padre
						modelloAlberoXmlTag.nodeStructureChanged(nodoPadre); // aggiorna la struttura dell'albero in base ai cambiamenti
					}
					catch (Exception e)
					{
						JOptionPaneEsteso mostraEccezione = new JOptionPaneEsteso(oggettoXMLStudioGlobale, "Informazione", "Eccezione rilevata nella cancellazione del nodo tag xml: " + e);
					}
				}
			}
		}
	}


	// **** dopo aver selezionato un nodo, dovrebbe modificare il suo nome nameSpace****

// 	protected void azioneModificaTagNodo()
// 	{
// 		TreePath path = alberoXmlTag.getSelectionPath(); // restituisce il path del nodo selezionato grazie a getSelectionPath
// 		XmlNodo nodoAlbero = restituisciNodoSelezionatoAlbero(); // in nodoAlbero viene messo il nodo attualmente selezionato
// 		System.out.println("sono qui:1");
// 		if (nodoAlbero != null)
// 		{
// 			System.out.println("sono qui:2");
// 			Node nodo = nodoAlbero.restituisciXmlNodo(); // restituisce il nodo corrente di tipo node dal nodo di tipo XmlNodo
// 			if (nodo != null)
// 			{
// 				try
// 				{
// 					nodoAlbero.cambia(); // ooo rimuove il nodoAlbero e posiziona la selezione sul nodo padre
// 					modelloAlberoXmlTag.nodeStructureChanged(nodoAlbero); // aggiorna la struttura dell'albero in base ai cambiamenti
// 					System.out.println("sono qui:4");
// 				}
// 				catch (Exception e)
// 				{
// 					JOptionPaneEsteso mostraEccezione = new JOptionPaneEsteso(oggettoXMLStudioGlobale, "Informazione", "Eccezione rilevata nella cancellazione del nodo tag xml: " + e);
// 				}
// 			}
// 		}
// 	}


	// **** aggiunge un attributo al nodo selezionato ****

	protected void azioneAggiungiAttributoNodo()
	{
		Node nodo = modelloTavolaAttributiXmlTag.restituisciNodoCorrente(); // restituisce il nodo esaminato
		if ((nodo instanceof Element) == true) // se siamo in presenza di un nodo di tipo element  (e non della porzione #text)
		{
			String nomeNuovoAttributo = (String)JOptionPane.showInputDialog(this, "Inserire un nuovo nome di attributo", "Inserimento attributo", JOptionPane.PLAIN_MESSAGE);
			if (nomeNuovoAttributo == null) // se e' stato premuto il pulsante annulla, esci
				return;
			if (nomeXmlValido(nomeNuovoAttributo) == true) // se il valore inserito e' un nome di attributo valido in xml
			{
				try
				{
					((Element)nodo).setAttribute(nomeNuovoAttributo, ""); // Element identifica un elemento xml e aggiunge un attributo senza valore associato ("")
					aggiornaAttributiNodoInTavola(nodo); // aggiorna l'albero xml con il nodo attualmente selezionato
					for (int k=0; k < modelloTavolaAttributiXmlTag.getRowCount(); k++) // cerca l'attributo e poi edita il suo valore alla riga relativa
					{
						if (modelloTavolaAttributiXmlTag.getValueAt(k, 0).equals(nomeNuovoAttributo)) // ha individuato il nome dell'attributo alla riga opportuna
						{
							tavolaAttributiXmlTag.requestFocus(); // si assegna il focus alla JTable in modo da consentire il diretto inserimento del valore
							tavolaAttributiXmlTag.editCellAt(k, 1); // edita la cella valor della JTable relativa all'attributo
							break; // se ha individuato l'attributo e quindi il valore, esce dal ciclo
						}
					}
				}
				catch (Exception e)
				{
					JOptionPaneEsteso mostraEccezione = new JOptionPaneEsteso(oggettoXMLStudioGlobale, "Informazione", "Eccezione rilevata nell'inserimento dell'attributo del nodo tag xml: " + e);
				}
			}
		}
	}


	// **** modifica il valore dell'attributo relativo al nodo selezionato ****

	protected void azioneModificaAttributoNodo()
	{
		// e' possibile richiamare questo metodo solo dopo che un attributo della JTable e' stato selezionato

		tavolaAttributiXmlTag.requestFocus(); // si assegna il focus alla JTable in modo da consentire il diretto inserimento del valore
		int riga = tavolaAttributiXmlTag.getSelectedRow(); // restituisce l'indice relativo alla riga selezionata
		if (riga >= 0) // se e' stata selezionata una riga
			tavolaAttributiXmlTag.editCellAt(riga, 1); // modifica il valore dell'attributo relativo a riga
	}


	// **** elimina un attributo dal nodo selezionato ****

	protected void azioneEliminaAttributoNodo()
	{
		// e' possibile richiamare questo metodo solo dopo che un attributo della JTable e' stato selezionato

		int rigaAttributoSelezionato = tavolaAttributiXmlTag.getSelectedRow(); // restituisce l'indice relativo alla riga selezionata
		if (rigaAttributoSelezionato >= 0) // se e' stata selezionata una riga della JTable relativa ad un attributo e ad un valore
		{
			Node nodoEliminare = restituisciNodoSelezionato(); // in nodoEliminare e' presente il nodo da eliminare
			if (nodoEliminare instanceof Element) // se siamo effettivamente in presenza di un nodo
			{
				String nomeAttributo = (String)tavolaAttributiXmlTag.getValueAt(rigaAttributoSelezionato, 0);
				int ritornoValore = JOptionPane.showConfirmDialog(oggettoXMLStudioGlobale.getContentPane(), "Si desidera eliminare l'attributo " + nomeAttributo + " ?", "Eliminazione attributo", JOptionPane.YES_NO_OPTION);
				if (ritornoValore == JOptionPane.YES_OPTION) // e' confermata l'eliminazione del nodo
				{
					try
					{
						((Element)nodoEliminare).removeAttribute(nomeAttributo); // rimuove un attributo avendone indicato il nome
						aggiornaAttributiNodoInTavola(nodoEliminare); // aggiorna l'albero xml con il nodo attualmente selezionato
					}
					catch (Exception e)
					{
						JOptionPaneEsteso mostraEccezione = new JOptionPaneEsteso(oggettoXMLStudioGlobale, "Informazione", "Eccezione rilevata nell'eliminazione dell'attributo del nodoEliminare tag xml: " + e);
					}
				}
			}
		}
	}


	// **** restituisce vero se il testo e' un nome valido come tag di un documento xml ****

	private static boolean nomeXmlValido(String testo)
	{
		if (testo == null || testo.length() == 0)
			return false;
		for (int k=0; k < testo.length(); k++)
		{
			char carattere = testo.charAt(k);
			if (Character.isLetter(carattere) || (carattere == '_') || (carattere == ':') || (k>0 && (Character.isDigit(carattere) || (carattere == '.') || (carattere == '-'))))
				continue;
			return false;
		}
		return true;
	}


	// **** analizza il nodo attualmente selezionato e abilita e disabilita i pulsanti per il corretto funzionamento ****

	protected void abilitaPulsantiNodi()
	{
		boolean b1 = (restituisciNodoSelezionato() instanceof Element); // siamo in presenza di un nodo tag non text#
		boolean b2 = (restituisciNodoSelezionato() != null);

		aggiunginodoxmltag.setEnabled(b1);
		oggettoXMLStudioGlobale.jmi_aggiunginodotag.setEnabled(b1);

		editanodoxmltag.setEnabled(b2);
		oggettoXMLStudioGlobale.jmi_modificanodotag.setEnabled(b2);

		cancellanodoxmltag.setEnabled(b2);
		oggettoXMLStudioGlobale.jmi_eliminanodotag.setEnabled(b2);
	}


	// **** analizza il nodo attualmente selezionato e abilita e disabilita i pulsanti per il corretto funzionamento ****

	protected void abilitaPulsantiAttributi()
	{
		boolean b1 = (modelloTavolaAttributiXmlTag.restituisciNodoCorrente() instanceof Element); // siamo in presenza di un nodo tag non text#
		boolean b2 = (tavolaAttributiXmlTag.getSelectedRowCount() > 0); // e' selezionata una riga specifica della JTable

		aggiungiattributonodoxmltag.setEnabled(b1);
		oggettoXMLStudioGlobale.jmi_aggiungiattributonodotag.setEnabled(b1);

		modificaattributonodoxmltag.setEnabled(b2);
		oggettoXMLStudioGlobale.jmi_modificaattributonodotag.setEnabled(b2);

		cancellaattributonodoxmltag.setEnabled(b2);
		oggettoXMLStudioGlobale.jmi_eliminaattributonodotag.setEnabled(b2);
	}


	// **** deseleziona le celle degli attributi e valori evidenziati nella JTable ****

	protected void azioneCancellaSelezioneJTable()
	{
		tavolaAttributiXmlTag.clearSelection();
	}


	// **** definisce un'interfaccia per rilevare cambiamenti nell'editing della cella **** ooo

	class XmlEditorRileva implements CellEditorListener
	{
		// **** questo metodo e' invocato e segnala al listener che e' stato correttamente terminato l'editing ****

		public void editingStopped(ChangeEvent e)
		{
 			String valore = editorCellaAlbero.getCellEditorValue().toString(); // in valore vi e' il nuovo valore del nodo

 			if (nodoAttualmenteEditato != null) // in nodoAttualmenteEditato vi e' il nodo in fase di editing
 				nodoAttualmenteEditato.setNodeValue(valore); // imposta il nuovo valore nel nodo editato
 			TreePath path = alberoXmlTag.getSelectionPath(); // in path vi e' il cammino del nodo attualmente selezionato
 			if (path != null)
 			{
	 			// definisce un generico nodo dell'albero ricavandolo dal cammino path e da getLastPathComponent che restituisce il componente del path
	 			DefaultMutableTreeNode alberoNodi = (DefaultMutableTreeNode)path.getLastPathComponent();
	 			// imposta lo UserObject del nodo al nodoAttualmenteEditato
	 			alberoNodi.setUserObject(nodoAttualmenteEditato);
	 			// nodeStructureChanged e' opportuno invocarlo perche' e' cambiata la struttura dell'albero con questo nuovo alberoNodi
 				modelloAlberoXmlTag.nodeStructureChanged(alberoNodi);
 			}
 			nodoAttualmenteEditato = null; // si reimposta il fatto che attualmente non vi e' piu' alcun nodo editato
		}

		// **** questo metodo e' invocato e segnala al listener che e' stato interrotto l'editing della cella ****

		public void editingCanceled(ChangeEvent e)
		{
			nodoAttualmenteEditato = null;
		}
	}

}